@@ -65,8 +65,8 @@ import org.apache.commons.net.util.SubnetUtils.SubnetInfo;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import ch.qos.logback.classic.Level;
-import com.openexchange.guard.database.DbCommand;
-import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.globalSettings.GlobalSettingsStorage;
+import com.openexchange.guard.database.globalSettings.RestDbGlobalSettingsStorage;
 import com.openexchange.guard.dbpool.ContextDBConnection;
 import com.openexchange.guard.dbpool.Manager;
 import com.openexchange.guard.encr.EncrLib;
@@ -95,9 +95,9 @@ public class Config {
     public static String ogMySqlReadIP = null;
 
     public static String ogMySqlIP = "";
-    
+
     public static String ogShardReadIP = null;
-    
+
     public static String ogShardIP = null;
 
     public static String mysqlUsername = "";
@@ -221,7 +221,7 @@ public class Config {
     public static int rsa_cache_count = 100;
 
     public static int rsa_certainty = 256;
-    
+
     public static int key_valid_days = 3650;
 
     public static int min_password_length = 6;
@@ -240,7 +240,7 @@ public class Config {
 
     public static int shard_size = 10000;
 
-    public static String db_prefix = "oxguard_";
+    public static String dbSchemaBaseName = "oxguard";
 
     public static boolean demo = false;
 
@@ -263,14 +263,21 @@ public class Config {
     public static String mailIdDomain = "";
 
     public static ArrayList<SubnetInfo> pgpWhitelist = new ArrayList<SubnetInfo>();
-    
+
     public static String apiUserAgent = "Open-Xchange Guard Server";
 
+    public static boolean checkPasswordFile = true;
+
     /**
      * The amount of hours how long the deleted keys will be stayed "exposed" for downloading
      */
     public static int ExposedKeyDurationInHours = 168 /* 1 Week */;
 
+    /**
+     * The connection timeout used for HTTP connections
+     */
+    public static int connectionTimeout = 10000; /*10sec*/
+
     /**
      * Parse config file and update variables
      */
@@ -360,6 +367,13 @@ public class Config {
                                     demo = false;
                                 }
                                 break;
+                            case "com.openexchange.guard.checkpasswordfile":
+                                if (data[1].trim().toLowerCase().contains("true")) {
+                                    checkPasswordFile = true;
+                                } else {
+                                    checkPasswordFile = false;
+                                }
+                                break;
                             case "com.openexchange.guard.deleteonretract":
                             	if (data[1].trim().toLowerCase().contains("true")) {
                             		deleteOnRetract = true;
@@ -434,6 +448,9 @@ public class Config {
                             case "com.openexchange.guard.databasepassword":
                                 mysqlPassword = data[1].trim();
                                 break;
+                            case "com.openexchange.guard.dbschemabasename":
+                                dbSchemaBaseName = data[1].trim();
+                                break;
                             case "com.openexchange.guard.configdbname":
                             	configdbName = data[1].trim();
                             	break;
@@ -665,6 +682,7 @@ public class Config {
                             		}
 
                             	}
+                            	break;
                             case "com.openexchange.guard.exposedkeydurationinhours":
                                     try {
                                         ExposedKeyDurationInHours = Integer.parseInt(data[1].trim());
@@ -672,8 +690,17 @@ public class Config {
                                         logger.error(
                                             "Error while loading configuration com.openexchange.guard.exposedKeyDurationInHours. Using default value.",
                                             e);
-
                                     }
+                                    break;
+                            case "com.openexchange.guard.connectiontimeout":
+                                try{
+                                    connectionTimeout = Integer.parseInt(data[1].trim());
+                                } catch(Exception e) {
+                                    logger.error(
+                                        "Error while loading configuration com.openexchange.guard.connectionTimeout. Using default value.",
+                                        e);
+                                }
+                                break;
                             default:
                             	logger.info("Unknown configuration tag " + variable);
                                 break;
@@ -713,18 +740,9 @@ public class Config {
         ContextDBConnection dbcon = null;
         try {
 
-            DbCommand com = new DbCommand("SELECT global_settings.key, global_settings.value FROM global_settings");
-            DbQuery db = new DbQuery();
-            db.readOG(com);
-            while (db.next()) {
-                String field = db.rs.getString("key");
-                switch (field) {
-                case "token":
-                    token = db.rs.getString("value");
-                    break;
-                }
-            }
-            db.close();
+            //TODO: use a service locator / DI-Container to resolve
+            GlobalSettingsStorage globalSettingsStorage = new RestDbGlobalSettingsStorage();
+            token = globalSettingsStorage.getByKey("token");
         } catch (Exception ex) {
             logger.error("Error while loading mysql config", ex);
         }
@@ -826,7 +844,7 @@ public class Config {
     }
 
     public static String getMessageID () {
-        if (mailIdDomain.equals("")) {
+        if (mailIdDomain.equals("")) {  // If maildomain not defined, try to extract domain from URL
             if (OXUrl.contains("/")) {
                 mailIdDomain = OXUrl.substring(0, OXUrl.indexOf("/"));
             }
@@ -834,6 +852,11 @@ public class Config {
                 mailIdDomain = mailIdDomain.substring(0, mailIdDomain.indexOf(":"));
             }
         }
+        if (mailIdDomain.equals("")) mailIdDomain = "GUARD";  // Otherwise, just define it as Guard
         return ("<" + EncrLib.getUUID() + "@" + mailIdDomain + ">");
     }
+
+    public static String getShardBaseName() {
+        return (dbSchemaBaseName + "_");
+    }
 }
