@@ -48,13 +48,16 @@
  */
 package com.openexchange.guard.cron;
 
-import java.util.ArrayList;
+import java.util.List;
 import java.util.TimerTask;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.database.DbCommand;
-import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.fileCache.FileCacheItem;
+import com.openexchange.guard.database.fileCache.FileCacheStorage;
+import com.openexchange.guard.database.fileCache.RestDbFileCacheStorage;
+import com.openexchange.guard.database.remoteKeyCache.RemoteKeyCacheStorage;
+import com.openexchange.guard.database.remoteKeyCache.RestDbRemoteKeyCacheStorage;
 import com.openexchange.guard.storage.Storage;
 
 
@@ -64,75 +67,49 @@ public class CleanCache extends TimerTask {
 
     @Override
     public void run() {
-        DbQuery db = new DbQuery();
-
         try {
             logger.info("Running Scheduled cleanup of File Cache");
 
-            ArrayList<String> list = new ArrayList<String>();
-            String command = "select itemID, location from FileCache WHERE lastDate < DATE_SUB(NOW(), INTERVAL " + Config.cachedays + " DAY);";
-            DbCommand com = new DbCommand(command);
-            db.readOG(com);
-            if (db.next()) {
-                String location = db.rs.getString("location");
+            //TODO: Using a service locator / DI-Container for resolving
+            FileCacheStorage fileCacheStorage = new RestDbFileCacheStorage();
 
+            //Getting unused fileCache items
+            List<FileCacheItem> fileCacheItems = fileCacheStorage.findOld(Config.cachedays);
+            int failedItems = 0;
+
+            //Remove the unused fileCache items from the storages
+            for(FileCacheItem fileCacheItem : fileCacheItems) {
                 try {
-                	Storage.deleteEncrObj(location);
-                    list.add(db.rs.getString("itemID"));
+                    //Remove from the file storage
+                    Storage.deleteEncrObj(fileCacheItem.getLocation());
+                    //Remove from the persistent storage (usually the DB)
+                    fileCacheStorage.delete(fileCacheItem.getItemId());
                 } catch (Exception ex) {
-                	logger.error("Error delete object ", ex);
+                    logger.error("Error delete object ", ex);
+                    failedItems++;
                 }
-                list.add(db.rs.getString("itemID"));
-
             }
-            for (int j = 0; j < list.size(); j++) { // Delete from database all files removed
-                command = "DELETE FROM FileCache WHERE itemID = ?;";
-                com = new DbCommand(command);
-                com.addVariables(list.get(j));
-                db.writeOxGuard(com);
-            }
-            db.close();
-            logger.info("Deleted " + list.size() + " files");
-
-            logger.info("Running Scheduled cleanup of PGP Cache");
-            command = "SELECT id FROM remote_key_cache WHERE created < DATE_SUB(NOW(), INTERVAL " + Config.pgpCacheDays + " DAY);";
-            com = new DbCommand(command);
-            db.readOG(com);
-            StringBuilder ids = new StringBuilder();
-            int count = 0;
-            while (db.next()) {
-                Long id = db.rs.getLong("id");
-                if (count == 0) {
-                    ids.append("(" + Long.toString(id));
-                } else {
-                    ids.append(", " + Long.toString(id));
-                }
-                count++;
-            }
-            db.close();
-            if (count > 0) {
-                // If there are some, then delete corresponding reference keys
-                ids.append(")");
-                command = "DELETE FROM remote_keys WHERE ref IN " + ids.toString();
-                com = new DbCommand(command);
-                db = new DbQuery();
-                db.writeOxGuard(com);
-                db.close();
-                // OK, now clean up the original table
-                command = "DELETE FROM remote_key_cache WHERE created < DATE_SUB(NOW(), INTERVAL " + Config.pgpCacheDays + " DAY);";
-                com = new DbCommand(command);
-                db = new DbQuery();
-                db.writeOxGuard(com);
-                db.close();
-                logger.info("Deleted " + count + " old PGP key rings");
+            logger.info("Deleted " + fileCacheItems.size() + " files");
+            if(failedItems > 0) {
+                logger.error("Failed deleting " + failedItems + " files");
             }
 
         } catch (Exception e) {
             logger.error("Error while doing scheduled cleanup of file cache", e);
-        } finally {
-            db.close();
         }
 
+
+        try {
+            //Cleanup remote key cache (cached HKP Keys)
+            logger.info("Running Scheduled cleanup of PGP Cache");
+            RemoteKeyCacheStorage remoteKeyCacheStorage = new RestDbRemoteKeyCacheStorage();
+            int deletedKeys = remoteKeyCacheStorage.deleteOld(Config.pgpCacheDays);
+            if(deletedKeys > 0) {
+                logger.info("Deleted " + deletedKeys + " old PGP key rings from remote cache");
+            }
+        } catch (Exception e) {
+            logger.error("Error removing older PGP keys from remote cache", e);
+        }
     }
 
 }
