@@ -49,12 +49,12 @@
 package com.openexchange.guard.database;
 
 import java.io.ByteArrayInputStream;
-import java.io.IOException;
 import java.security.NoSuchAlgorithmException;
 import java.security.spec.InvalidKeySpecException;
 import java.sql.SQLException;
 import java.util.ArrayList;
-import java.util.Date;
+import java.util.Arrays;
+import java.util.List;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
@@ -63,8 +63,21 @@ import org.slf4j.LoggerFactory;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
-import com.google.gson.JsonObject;
 import com.openexchange.guard.config.Config;
+import com.openexchange.guard.database.ogEmail.OGEmail;
+import com.openexchange.guard.database.ogEmail.OGEmailStorage;
+import com.openexchange.guard.database.ogEmail.RestDbOGEmailStorage;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
+import com.openexchange.guard.database.ogPGPKeys.OGPGPKey;
+import com.openexchange.guard.database.ogPGPKeys.OGPGPKeysStorage;
+import com.openexchange.guard.database.ogPGPKeys.RestDbOGPGPKeysStorage;
+import com.openexchange.guard.database.oxguardSession.OXGuardSession;
+import com.openexchange.guard.database.oxguardSession.OXGuardSessionStorage;
+import com.openexchange.guard.database.oxguardSession.RestDbOXGuardSessionStorage;
+import com.openexchange.guard.database.pgpKeys.PGPKeys;
+import com.openexchange.guard.database.pgpKeys.PGPKeysStorage;
+import com.openexchange.guard.database.pgpKeys.RestDbPGPKeysStorage;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.pgp.PgpKeys;
@@ -109,7 +122,7 @@ public class Access {
 
     public Access() {
         ConfigmysqlConnectionString = Config.getConfigDbConnectionString();
-        OGmysqlConnectionString = "jdbc:mysql://" + OGmysqlIP + "/oxguard?user=" + mysqlUsername + "&password=" + mysqlPassword;
+        OGmysqlConnectionString = "jdbc:mysql://" + OGmysqlIP + "/" + Config.dbSchemaBaseName + "?user=" + mysqlUsername + "&password=" + mysqlPassword;
 
     }
 
@@ -143,203 +156,72 @@ public class Access {
         return (data);
     }
 
-    /**
-     * Execute a mysql command
-     *
-     * @param command
-     * @throws Exception
-     */
-    private void mysqlExecute(String command, int cid) throws Exception {
-        DbQuery db = new DbQuery();
-        DbCommand com = new DbCommand(command);
-        JsonObject result = db.writeToDB(com, cid);
-        // DBConnection conn = manager.getConn(cid);
-        // Statement stmt = null;
-        // try {
-        // stmt = conn.con.createStatement();
-        // stmt.execute(command);
-        // } catch (Exception ex) {
-        // logger.error(ex);
-        // } finally {
-        // manager.clean(cid, stmt, conn);
-        // }
-    }
 
-    /**
-     * Execute a mysql command
-     *
-     * @param command
-     * @throws Exception
-     */
-    private void mysqlExecute(String command) throws Exception {
-        DbCommand com = new DbCommand(command);
-        DbQuery db = new DbQuery();
-        db.writeOxGuard(com);
-        db.close();
-    }
+    public GuardKeys storeKeys(GuardKeys key, boolean userCreated) throws Exception {
+        GuardKeys ret = null;
 
-    /*
-     * Store newly created keys for user
-     */
-    public int storeKeys(int id, int cid, String email, GuardKeys key, String recovery, String language, boolean userCreated) throws Exception {
+        //Getting the shard to use
         int shard = 0;
-
-        if (cid < 0) { // If guest, get shard number, then check tables exist
+        if (key.contextid < 0) { // If guest, get shard number, then check tables exist
             shard = Sharding.getNextShard();
             if (shard == 0) {
-            	logger.error("Unable to get non-ox shard id");
-            	return(0);
+                logger.error("Unable to get non-ox shard id");
+                return null;
             }
             Sharding.checkTables(shard, false);
         } else {
-            if (cid > 0)
+            if (key.contextid > 0)
              {
-                Sharding.checkTables(cid, true); // Check these OG tables exist
+                Sharding.checkTables(key.contextid, true); // Check these OG tables exist
             }
         }
 
-        String command = "INSERT INTO og_KeyTable (id, cid, PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Recovery, Salt, email, lastMod, lang, keyid) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, ";
-        if (userCreated) {
-            command += "NOW()";
-        } else {
-            command += "NULL";
-        }
-        command += ", ?, ?)";
-        DbCommand com = new DbCommand(command);
-        com.addVariables((id == 0 ? null : id));
-        com.addVariables(cid);
-        com.addVariables(key.getEncodedPGPSecret());
-        com.addVariables(key.exportPgpPublicKey());
-        com.addVariables(key.getEncodedPrivate());
-        com.addVariables(key.getEncodedPublic());
-        com.addVariables(recovery);
-        com.addVariables(key.getSalt());
-        com.addVariables(email);
-        com.addVariables(language);
-        com.addVariables(key.pubring.getPublicKey().getKeyID());
-        try {
-            DbQuery db = new DbQuery();
-            if (cid >= 0) {// OX users go in OX database, otherwise guest shard
-                if (id < 0) {// Master key is negative, otherwise place in proper shard
-                    db.writeOxGuard(com);
-                } else {
-                    db.writeToDB(com, cid);
-                }
-            } else {
-                db.writeToDb(com, Sharding.database_name(shard));
-            }
+        //Adding the key to the storage
+        OGKeyTableStorage keyTableStorage = new RestDbOGKeyTableStorage();
+        keyTableStorage.insert(key,userCreated, shard);
+        //retrieve the created key in order to get the userid which was created while inserting
+        ret = keyTableStorage.getKeyForEmailAndContext(key.email,key.contextid, shard);
 
-            int userid = id;
-            if (id == 0) {// For guest accounts, we need to return the new userid
-                command = "SELECT id FROM og_KeyTable WHERE email = ?";
-                com = new DbCommand(command);
-                com.addVariables(email);
-                if (cid >= 0) {
-                    if (id > 0) {
-                        db.readFromDB(com, cid);
-                    } else {
-                        db.readOG(com);
-                    }
-                } else {
-                    db.readFromDB(com, Sharding.database_name(shard));
-                }
-                if (db.next()) {
-                    userid = db.rs.getInt("id");
-                }
-            }
-            if (userid > 0) {
-                addEmail(email, userid, cid, shard);
-            }
-            db.close();
+        //Store key mappings
+        if(ret.userid > 0) {
+            //Adding the new key to the email mapping storage
+            addEmail(ret.email, ret.userid, ret.contextid, shard);
 
-            key.contextid = cid;
-            if (userid > 0) {
-                PgpKeys.addPublicKeyIndex(key);
-            }
-
-            return (userid);
-        } catch (SQLException e) {
-            logger.error("SQL error while storing key", e);
-            return (0);
+            //Adding the new key to the PGP key mapping storage
+            PgpKeys.addPublicKeyIndex(ret);
         }
 
-    }
-
-    public boolean updatePubring (GuardKeys key) throws IOException {
-        String command = "UPDATE og_KeyTable SET PGPPublic = ? WHERE keyid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(key.exportPGPPublicKeyRing());
-        com.addVariables(key.keyid);
-        DbQuery db = new DbQuery();
-        try {
-            db.write(com, key.userid, key.contextid);
-        } catch (Exception e) {
-            logger.error("Error updating PGP Public KeyRing " , e);
-            return (false);
-        }
-        return (true);
+        return ret;
     }
 
     public boolean addNewKey (GuardKeys newkey) throws Exception {
+        //TODO: use a Service-Locator for resolving this
+        OGKeyTableStorage keyTableStorage = new RestDbOGKeyTableStorage();
 
-    	int version = 0;
-    	// Check for highest version of key
-    	String getversion = "SELECT MAX(version) FROM og_KeyTable WHERE id = ? and cid = ?";
-    	DbCommand ver = new DbCommand(getversion);
-    	ver.addVariables(newkey.userid);
-    	ver.addVariables(newkey.contextid);
-        String update = "UPDATE og_KeyTable SET current = 0 WHERE id = ? and cid = ?";
-        DbCommand up = new DbCommand(update);
-        up.addVariables(newkey.userid);
-        up.addVariables(newkey.contextid);
-        DbQuery db = new DbQuery();
-        if (newkey.contextid >= 0) {// OX users go in OX database, otherwise guest shard
-            db.writeToDB(up, newkey.contextid);
-        } else {
-            db.write(up, newkey.userid, newkey.contextid);
-        }
-        db.close();
-        String command = "INSERT INTO og_KeyTable (id, cid, PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Recovery, Salt, email, lastMod, lang, keyid, version, question, answer, settings) VALUES ( ?, ?, ?, ?, ?, ?, ?, ?, ?, " +
-            "NOW(), ?, ?, ?, ?, ?, ?)";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(newkey.userid);
-        com.addVariables(newkey.contextid);
-        com.addVariables(newkey.getEncodedPGPSecret());
-        com.addVariables(newkey.exportPgpPublicKey());
-        com.addVariables(newkey.getEncodedPrivate());
-        com.addVariables(newkey.getEncodedPublic());
-        com.addVariables(newkey.recovery);
-        com.addVariables(newkey.getSalt());
-        com.addVariables(newkey.email);
-        com.addVariables(newkey.language);
-        com.addVariables(newkey.pubring.getPublicKey().getKeyID());
-        com.addVariables(newkey.version);
-        com.addVariables(newkey.question);
-        com.addVariables(newkey.answer);
-        com.addVariables(newkey.settings == null ? null : newkey.settings.toString());
-        db = new DbQuery();
-        if (newkey.contextid >= 0) {// OX users go in OX database, otherwise guest shard
-            db.writeToDB(com, newkey.contextid);
-        } else {
-            db.write(com, newkey.userid, newkey.contextid);
-        }
-        db.close();
+        //removing the "current" flag from existing keys,
+        //because the new created key will be the "current" key by default
+        keyTableStorage.unsetCurrentFlag(newkey.userid, newkey.contextid);
+        //Marking the new key as the "current" key
+        newkey.current = true;
+
+        //Getting the key with the highest version
+        GuardKeys highestKeys = keyTableStorage.getHighestVersionKeyForUser(newkey.userid, newkey.contextid);
+
+        //Incrementing key version, if highest was found, and adding the new key to the storage
+        newkey.version = highestKeys != null ? highestKeys.version + 1 : 0;
+        keyTableStorage.insert(newkey,true /*set lastMod to NOW()*/);
+
+        //Adding the new key to the public key index (lookup table)
         PgpKeys.addPublicKeyIndex(newkey);
-        return (true);
+
+        return true;
+
     }
 
     public void updateKeys(int id, int cid, GuardKeys key, String recovery, boolean reset) throws Exception {
-        String command = "UPDATE og_KeyTable Set PGPSecret = '" + key.getEncodedPGPSecret() + "', RSAPrivate = " + (key.privKeyNull() ? "NULL" : ("'" + key.getEncodedPrivate() + "'")) + ", Recovery = '" + recovery +
-        		"', lastMod = " + (reset ? "NULL" : "NOW()") + " WHERE id = " + id + " AND cid = " + cid + " AND current = 1;";
-        try {
-            DbCommand com = new DbCommand(command);
-            DbQuery db = new DbQuery();
-            db.write(com, id, cid);
-            db.close();
-        } catch (SQLException e) {
-            logger.error("SQL error while updating keys", e);
-        }
-
+        //TODO: Use a DI Container to resolve this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        ogKeyTableStorage.updatePassword(key, recovery, !reset /*setLastMod*/);
     }
 
     /**
@@ -351,58 +233,24 @@ public class Access {
      * @throws Exception
      */
     public void updateDuplPGPKey (int id, int cid, GuardKeys key, String recovery, Long dupl) throws Exception {
-        String command = "UPDATE og_KeyTable SET PGPSecret = ?, PGPPublic = ?, Recovery = ?, Salt = ? WHERE id = ? AND cid = ? and keyid = ?";
-        try {
-            DbCommand com = new DbCommand(command);
-            com.addVariables(key.getEncodedPGPSecret());
-            com.addVariables(key.exportPgpPublicKey());
-            com.addVariables(recovery);
-            com.addVariables(key.getSalt());
-            com.addVariables(id);
-            com.addVariables(cid);
-            com.addVariables(dupl);
-            DbQuery db = new DbQuery();
-            db.write(com, id, cid);
-            db.close();
-        } catch (SQLException e) {
-            logger.error("SQL error while updating PGP keys", e);
-        }
+        //TODO: Use a service locator for resolving
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        ogKeyTableStorage.updateDuplicate(key, recovery, dupl);
     }
 
     public void updatePGPKeys (int id, int cid, GuardKeys key) throws Exception {
-    	String command = "UPDATE og_KeyTable Set PGPSecret = ?, PGPPublic = ? WHERE id = ? AND cid = ? AND keyid = ?";
-        try {
-            DbCommand com = new DbCommand(command);
-            com.addVariables(key.getEncodedPGPSecret());
-            com.addVariables(key.exportPgpPublicKey());
-            com.addVariables(id);
-            com.addVariables(cid);
-            com.addVariables(key.keyid);
-            DbQuery db = new DbQuery();
-            db.write(com, id, cid);
-            db.close();
-            PgpKeys.addPublicKeyIndex(key);
-        } catch (SQLException e) {
-            logger.error("SQL error while updating PGP keys", e);
-        }
+        //TODO: Use a service locator for resovling this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        ogKeyTableStorage.updatePublicAndPrivateKey(key);
+        PgpKeys.addPublicKeyIndex(key);
     }
 
     public static void updatePin(int id, int cid, String pin) throws Exception {
-    	String command = "UPDATE og_KeyTable Set question = 'PIN', answer = ? WHERE id = ? AND cid = ?;";
-        try {
-            DbCommand com = new DbCommand(command);
-            com.addVariables(pin);
-            com.addVariables(id);
-            com.addVariables(cid);
-            DbQuery db = new DbQuery();
-            db.write(com, id, cid);
-            db.close();
-        } catch (SQLException e) {
-            logger.error("SQL error while updating keys", e);
-        }
+        //TODO: Use a service locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        ogKeyTableStorage.updatePin(id,cid,pin);
     }
 
-
     public GuardKeys getKeys(int id, int cid) throws Exception {
         int shard = 0;
         if (cid < 0) {
@@ -431,39 +279,12 @@ public class Access {
      * @throws Exception
      */
     public GuardKeys getKeys(int id, int cid, int shard, String keyid, boolean current) throws Exception {
-        GuardKeys key = null;
-        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE id = ? AND cid = ?";
-        if (current) {
-            command += " AND og_KeyTable.current = 1 ";
+        //TODO: use a service-locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        if(current) {
+            return ogKeyTableStorage.getCurrentKeyForUser(id, cid, shard);
         }
-        if (keyid != null) {
-            command += " AND keyid = ?";
-        }
-        DbCommand com = new DbCommand(command);
-        com.addVariables(id);
-        com.addVariables(cid);
-        if (keyid != null) {
-            com.addVariables(keyid);
-        }
-        DbQuery db = new DbQuery();
-        if (cid >= 0) {
-            if (id < 0) {// minus 1 is the recovery password, which is in OG
-                db.readOG(com);
-            } else {
-                db.readFromDB(com, cid);
-            }
-        } else {
-            db.readFromShard(com, shard);
-        }
-        try {
-            if (db.next()) {
-                key = getKeyFromRS (db.rs);
-            }
-        } catch (Exception ex) {
-            logger.error("Error while iterating result set for keys", ex);
-
-        }
-        return (key);
+        return ogKeyTableStorage.getKeyForUserById(id, cid, shard, keyid);
     }
 
     /**
@@ -504,42 +325,6 @@ public class Access {
         return (key);
     }
 
-    /**
-     * Get a list of all the Keys associated with the account
-     * @param id
-     * @param cid
-     * @return
-     * @throws Exception
-     */
-    public ArrayList<GuardKeys> getAllKeys (int id, int cid) throws Exception {
-        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE id = ? AND cid = ? ORDER BY version DESC";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(id);
-        com.addVariables(cid);
-        DbQuery db = new DbQuery();
-        db.read(com, id, cid);
-        ArrayList<GuardKeys> keys = new ArrayList<GuardKeys> ();
-        while (db.next()) {
-            GuardKeys key = getKeyFromRS (db.rs);
-            keys.add(key);
-        }
-        return (keys);
-    }
-
-    public ArrayList<GuardKeys> getAllKeysInContext (int cid) throws Exception {
-        String command = "SELECT PGPSecret, PGPPublic, RSAPrivate, RSAPublic, Salt, misc, settings, email, lastMod, question, answer, cid, id, keyid, version, current, lang, Recovery FROM og_KeyTable WHERE cid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(cid);
-        DbQuery db = new DbQuery();
-        db.readFromDB(com, cid);
-        ArrayList<GuardKeys> keys = new ArrayList<GuardKeys> ();
-        while (db.next()) {
-            GuardKeys key = getKeyFromRS (db.rs);
-            keys.add(key);
-        }
-        return (keys);
-    }
-
     /**
      * Create GuardKey from database result set
      * @param rs
@@ -553,7 +338,7 @@ public class Access {
     private GuardKeys getKeyFromRS (OxResultSet rs) throws InvalidKeySpecException, NoSuchAlgorithmException, Exception {
         GuardKeys key = new GuardKeys();
         key.setPGPSecretFromString(rs.getString("PGPSecret"));
-        key.setPGPPublicKeyRing(rs.getString("PGPPublic"));
+        key.setPGPKeyRingFromAsc(rs.getString("PGPPublic"));
         key.setEncodedPrivate(rs.getString("RSAPrivate"));
         key.setPublicKeyFrom64String(rs.getString("RSAPublic"));
         key.setSalt(rs.getString("Salt"));
@@ -590,24 +375,26 @@ public class Access {
      * @throws Exception
      */
     public GuardKeys getKeysFromEmail(String email, Long keyid) throws Exception {
-        GuardKeys key = null;
-        String command = "SELECT cid, id, db FROM og_email WHERE email LIKE (?);";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(email);
-        DbQuery db = new DbQuery();
-        db.readOG(com);
-        if (db.next()) {
-            int cid = db.rs.getInt("cid");
-            int id = db.rs.getInt("id");
-            int shard = db.rs.getInt("db");
-            db.close();
+        GuardKeys ret = null;
+
+        //TODO: Use a DI container to resolve this
+        OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+        OGEmail ogEmail = ogEmailStorage.getByEmail(email);
+
+        if(ogEmail != null) {
             if (keyid != 0L) {
-                return (getKeys(id, cid, shard, Long.toString(keyid), false));
+                ret = getKeys(ogEmail.getUserId(),
+                              ogEmail.getContextId(),
+                              ogEmail.getShardingId(),
+                              Long.toString(keyid), false);
+            }
+            else {
+                ret = getKeys(ogEmail.getUserId(),
+                              ogEmail.getContextId(),
+                              ogEmail.getShardingId());
             }
-            return (getKeys(id, cid, shard));
         }
-        db.close();
-        return (key);
+        return ret;
     }
 
     /**
@@ -652,15 +439,6 @@ public class Access {
         db.write(com, userid, cid);
     }
 
-    public void deleteRecovery (int id, int cid) throws Exception {
-        String command = "UPDATE og_KeyTable SET Recovery = '' WHERE id = ? and cid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(id);
-        com.addVariables(cid);
-        DbQuery db = new DbQuery();
-        db.write(com, id, cid);
-    }
-
     private String convertDate(java.util.Date dt) {
         if (dt == null) {
             return (null);
@@ -781,29 +559,22 @@ public class Access {
     /**
      * Insert new token to oxGuard_sessions for encrypting passwords
      *
-     * @param sessionID
-     * @param userid
-     * @return
-     * @throws Exception
+     * @param sessionId the session ID to create a new token for
+     * @param userId the user's id
+     * @param userid the user's id
+     * @return the new created token
+     * @throws Exception due an error
      */
-    public String newToken(String sessionID, int userid, int cid) throws Exception {
-        // String token = EncrLib.getsalt();
+    public String newToken(String sessionId, int userId, int contextId) throws Exception {
+        //Create a new token
         String token = EncrLib.genRandom64Key();
-        String command = "INSERT INTO oxguard_session (sessionid, token, date, userid, cid) VALUES (?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE sessionid = ?, token = ?, date = ?;";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(sessionID);
-        com.addVariables(token);
-        String now = convertDate(new Date());
-        com.addVariables(now);
-        com.addVariables(userid);
-        com.addVariables(cid);
-        com.addVariables(sessionID);
-        com.addVariables(token);
-        com.addVariables(now);
-        DbQuery db = new DbQuery();
-        db.writeOxGuard(com);
-        db.close();
-        return (token);
+
+        //Insert or update the token
+        //TODO: Use a service locator / DI Container for resolving
+        OXGuardSessionStorage guardSessionStorage = new RestDbOXGuardSessionStorage();
+        guardSessionStorage.InsertOrUpdate(sessionId, token, userId, contextId);
+
+        return token;
     }
 
     /**
@@ -813,63 +584,36 @@ public class Access {
      * @throws Exception
      */
     public void deleteToken(String sessionID) throws Exception {
-        if (sessionID == null) {
-            return;
+        if (sessionID != null) {
+            //TODO: use a DI-Container for resolving this
+            OXGuardSessionStorage guardSessionStorage = new RestDbOXGuardSessionStorage();
+            guardSessionStorage.deleteById(sessionID);
+            guardSessionStorage.deleteOldSessions(1);
         }
-        String command = "DELETE FROM oxguard_session WHERE sessionid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(sessionID);
-        DbQuery db = new DbQuery();
-        try {
-            db.writeOxGuard(com);
-            command = "delete from oxguard_session where date < NOW() - INTERVAL 1 DAY"; // cleanup old sessions
-            com = new DbCommand(command);
-            db.writeOxGuard(com);
-        } catch (SQLException e) {
-            logger.error("SQL error while deleting oxguard session " + sessionID, e);
-        }
-        db.close();
     }
 
     /**
      * Get the token associated with the sessionID
      *
-     * @param sessionID
-     * @return
+     * @param sessionID the session to get the token for
+     * @return the token related to the given session or null if no token was found for the sessions
      */
-    public String getToken(String sessionID) {
+    public String getToken(String sessionId) {
         try {
-            String command = "SELECT token FROM oxguard_session WHERE sessionid = ?";
-            DbCommand com = new DbCommand(command);
-            com.addVariables(sessionID);
-            DbQuery db = new DbQuery();
-            db.readOG(com);
-            String token = null;
-            if (db.next()) {
-                token = db.rs.getString("token");
+
+            //TODO: using a DI-Container to resolve this
+            OXGuardSessionStorage guardSessionStorage = new RestDbOXGuardSessionStorage();
+
+            OXGuardSession session = guardSessionStorage.getById(sessionId);
+            if(session != null) {
+                return session.getToken();
             }
-            db.close();
-            return (token);
         } catch (Exception e) {
-            logger.error("Error while retrieving token from session " + sessionID, e);
+            logger.error("Error while retrieving token from session " + sessionId, e);
         }
         return (null);
     }
 
-    public void saveCache(String itemID, int userid, String location) throws Exception {
-        String command = "INSERT INTO FileCache VALUES (?, ?, ?, NOW())";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(itemID);
-        com.addVariables(userid);
-        com.addVariables(location);
-        DbQuery db = new DbQuery();
-        try {
-            db.writeOxGuard(com);
-        } catch (Exception e) {
-            logger.error("Error while saving in file cache", e);
-        }
-        db.close();
-    }
 
     // Lookup the public key and userid for a given email address. Returns null if not found
 
@@ -975,15 +719,9 @@ public class Access {
         if (answer != null) {
             encranswer = encr.encryptAES(answer, Config.rpass, question, Config.AESKeyLen);
         }
-        String command = "UPDATE og_KeyTable SET question = ?, answer = ?" + " WHERE id = ? AND cid = ?";
         try {
-            DbQuery db = new DbQuery();
-            DbCommand com = new DbCommand(command);
-            com.addVariables(question);
-            com.addVariables(encranswer);
-            com.addVariables(id);
-            com.addVariables(cid);
-            db.write(com, id, cid);
+            OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+            ogKeyTableStorage.updateQuestionForUser(id, cid, question, encranswer);
         } catch (Exception e) {
             logger.error("Error while storing reminder question", e);
             return(false);
@@ -991,149 +729,94 @@ public class Access {
         return(true);
     }
 
-    public ArrayList<PGPPublicKeyRing> getPublicKeyRing(String email, int userid, int cid) {
-        return (getPublicKeyRing(email, userid, cid, false));
-    }
     /**
      * Search for public keys for email
-     * CID = 0 if search for local only
      * @param email
-     * @param cid
-     * @param oxOnly  exclude guests
      * @return
+     * @throws Exception due an error
      */
-    public ArrayList<PGPPublicKeyRing> getPublicKeyRing(String email, int userid, int cid, boolean oxOnly) {
-        try {
-        	ArrayList<PGPPublicKeyRing> pubRings = new ArrayList<PGPPublicKeyRing>();
-        	String command = "select id, cid, local FROM PGPKeys WHERE email = ?";
-        	if (oxOnly) {
-                command += " AND cid > 0";
+    public ArrayList<PGPPublicKeyRing> getPublicKeyRing(String email) throws Exception {
+            ArrayList<PGPPublicKeyRing> ret = new ArrayList<PGPPublicKeyRing>();
+
+            //TODO: Use a service locator for resolving this
+            PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+            List<PGPKeys> internalKeyMappings = pgpKeysStorage.getInternalByEmail(email);
+            for(PGPKeys internalMapping : internalKeyMappings) {
+                if(internalMapping.isLocal()) {
+                    GuardKeys key = getKeysFromEmail(email);
+                    ret.add(key.pubring);
+
+                }
             }
-        	DbCommand com = new DbCommand(command);
-        	com.addVariables(email);
-        	DbQuery db = new DbQuery();
-        	db.readOG(com);
-        	while (db.next()) {
-        		String id = db.rs.getString("id");
-        		int key_cid = db.rs.getInt("cid");
-        		boolean local = db.rs.getBit("local");
-        		if (local) {  // If local, we just use the one we have
-        			GuardKeys key = getKeysFromEmail(email);
-        			pubRings.add(key.pubring);
-        			db.close();
-        			return (pubRings);
-        		} else {
-        			if (cid == 0) {
-                        return (null);
-                    }
-        			command = "SELECT PGPPublic FROM og_pgp_keys WHERE Email = ? AND ids LIKE ? AND cid = ? AND (userid = ? OR share_level > 0)";
-        			DbCommand com2 = new DbCommand(command);
-        			com2.addVariables(email);
-        			com2.addVariables("%" + id + "%");
-        			com2.addVariables(cid);
-        			com2.addVariables(userid);
-        			DbQuery db2 = new DbQuery();
-        			db2.readFromDB(com2, key_cid);
-        			if (db2.next()) {
-        				GuardKeys key = new GuardKeys();
-        				key.setPGPPublicKeyRing(db2.rs.getString("PGPPublic"));
-        				pubRings.add(key.pubring);
-        			}
-        			db2.close();
-        		}
-        	}
-        	return (pubRings);
-        } catch (Exception ex) {
-            logger.error("Error while retrieving public key ring for " + email, ex);
-            return (null);
-        }
+            return ret;
     }
 
     /**
-     * Get the public keyring associated with the id.  Can be long or Hex String (8 or 16 byte)
+     * Get the public keyring associated with the id.
      * @param req_id
      * @param userid
      * @param cid
      * @return
      */
-    public PGPPublicKeyRing getPublicKeyRingById(Object req_id, int userid, int cid) {
+    public PGPPublicKeyRing getPublicKeyRingById(long req_id, int userid, int cid) {
         try {
-        	String command = "";
-        	String hexid = "";
-        	DbCommand com;
-        	if (req_id.getClass() == String.class) {
-        		command = "select email, cid, keyid, local FROM PGPKeys WHERE hexid = ?";
-        		hexid = (String) req_id;
-        		if (hexid.length() > 8) {  // If this is a 16 byte key, then lets convert it back to the long id
-        			req_id = Core.getLongPGP(hexid);
-        		}
-        	}
-        	if (req_id.getClass() == Long.class) {
-        		command = "select email, cid, keyid, local FROM PGPKeys WHERE id = ?";
-        		hexid = Core.PGPKeyId((Long) req_id);
-        	}
-        	DbQuery db = new DbQuery();
-        	com = new DbCommand(command);
-    		com.addVariables(req_id);
-        	db.readOG(com);
-        	if (db.next()) {
-        		String email = db.rs.getString("email");
-        		int key_cid = db.rs.getInt("cid");
-        		long keyid = db.rs.getLong("keyid");
-        		boolean local = db.rs.getBit("local");
-        		if (local) {  // If local, we just use the one we have
-        			GuardKeys key = getKeysFromEmail(email, keyid);
-        			db.close();
-        			return (key.pubring);
-        		} else {
-        			db.close();
-        			if (cid == 0) {
-                        return (null);
-                    }
-        			return (getUploadedPublicKeyRingById (hexid, userid, cid));
-        		}
-        	}
-        	db.close();
-        	if (cid == 0) {
-                return (null);
+            PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+            PGPKeys keyMapping = pgpKeysStorage.getById(req_id);
+            if(keyMapping != null) {
+                if(keyMapping.isLocal()) {
+                    GuardKeys key = getKeysFromEmail(keyMapping.getEmail(), keyMapping.getKeyId());
+                    return key.pubring;
+                }
             }
-        	return (getUploadedPublicKeyRingById (hexid, userid, cid));
+
+            String hexid = Core.PGPKeyId(req_id);
+            return (getUploadedPublicKeyRingById (hexid, userid, cid));
         } catch (Exception ex) {
             logger.error("Error while retrieving public key ring for " + req_id, ex);
             return (null);
         }
     }
 
+    /**
+     * Get the public keyring associated with the id.
+     * @param req_id
+     * @param userid
+     * @param cid
+     * @return
+     */
+    public PGPPublicKeyRing getPublicKeyRingById(String hexid, int userid, int cid) {
+        try {
+            PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+            PGPKeys keyMapping = pgpKeysStorage.getByHexId(hexid);
+            if(keyMapping != null) {
+                if(keyMapping.isLocal()) {
+                    GuardKeys key = getKeysFromEmail(keyMapping.getEmail(), keyMapping.getKeyId());
+                    return key.pubring;
+                }
+            }
+            return (getUploadedPublicKeyRingById (hexid, userid, cid));
+
+        } catch (Exception ex) {
+            logger.error("Error while retrieving public key ring for " + hexid, ex);
+            return (null);
+        }
+    }
+
     private PGPPublicKeyRing getUploadedPublicKeyRingById (String req_id, int userid, int cid) throws Exception {
-    	String command = "SELECT PGPPublic FROM og_pgp_keys WHERE cid = ? AND (userid = ? OR share_level = 1) AND ids LIKE ?";
-		DbCommand com2 = new DbCommand(command);
-		com2.addVariables(cid);
-		com2.addVariables(userid);
-		com2.addVariables("%" + req_id + "%");
-		DbQuery db2 = new DbQuery();
-		db2.readFromDB(com2, cid);
-		if (db2.next()) {
-			GuardKeys key = new GuardKeys();
-			key.setPGPPublicKeyRing(db2.rs.getString("PGPPublic"));
-			db2.close();
-			return(key.pubring);
-		}
-		db2.close();
-		return (null);
+        //TODO: Use a DI-Container or Service locator to resolve this
+        if (cid == 0) return (null);
+        OGPGPKeysStorage ogpgpKeysStorage = new RestDbOGPGPKeysStorage();
+        List<OGPGPKey> pgpKeys = ogpgpKeysStorage.getForUserByIds(userid, cid, Arrays.asList(new String[] {req_id}));
+        if(pgpKeys.size() > 0) {
+            GuardKeys key = new GuardKeys();
+            key.setPGPKeyRingFromAsc(pgpKeys.get(0).getPublicPGPAscData());
+            return key.pubring;
+        }
+        return null;
     }
 
     public static void addEmail(String email, int id, int cid, int shard) throws Exception {
-        String command = "INSERT INTO og_email VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE id = ?, cid = ?, db = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(email);
-        com.addVariables(id);
-        com.addVariables(cid);
-        com.addVariables(shard);
-        com.addVariables(id);
-        com.addVariables(cid);
-        com.addVariables(shard);
-        DbQuery db = new DbQuery();
-        db.writeOxGuard(com);
-        db.close();
+        OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+        ogEmailStorage.insertOrUpdate(email, cid, id, shard);
     }
 }
