@@ -136,7 +136,7 @@ public class DbQuery {
     private JsonObject putOgRequest(String data, boolean write) throws Exception {
 
 
-        String url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/pool/r/" + OGInit.read_id + "/w/" + OGInit.write_id + "/oxguard/" + (write ? "writable" : "readOnly");
+        String url = "http" + (Config.backend_ssl ? "s" : "") + "://" + Config.getOxBackend() + ":" + Config.ox_backend_port + "/preliminary/database/v1/pool/r/" + OGInit.read_id + "/w/" + OGInit.write_id + "/" + Config.dbSchemaBaseName + "/" + (write ? "writable" : "readOnly");
         if (keepOpen) {
             url += "?keepOpen=true";
         }
