@@ -4,36 +4,34 @@ import java.io.ByteArrayInputStream;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.util.ArrayList;
-
+import java.util.List;
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 import org.xml.sax.InputSource;
 import org.xml.sax.SAXException;
-
-import com.google.gson.JsonArray;
-import com.google.gson.JsonObject;
-import com.openexchange.guard.server.Guest;
+import com.openexchange.guard.database.ogEmail.OGEmail;
+import com.openexchange.guard.database.ogEmail.OGEmailStorage;
+import com.openexchange.guard.database.ogEmail.RestDbOGEmailStorage;
 
 public class EmailInfo {
 
 	private static Logger logger = LoggerFactory.getLogger(EmailInfo.class);
-	
+
 	public String senderEmail = "";
 	public String fromEmail = "";
 	public int owner_id = 0;
 	public int owner_cid = 0;
 	public ArrayList <String> recips = new ArrayList <String> ();
-	
+
 	public EmailInfo () {
-		
+
 	}
-	
+
 	public EmailInfo (String emailId, int id, int cid) {
 		try {
 			if (emailId.startsWith("pgp")) {
@@ -52,33 +50,25 @@ public class EmailInfo {
 			}
 			db.close();
 			if (owner_cid != 0) {
-				command = "SELECT email from og_email WHERE id = ? AND cid = ?";
-				com = new DbCommand(command);
-				com.addVariables(owner_id);
-				com.addVariables(owner_cid);
-				db = new DbQuery();
-				db.readOG(com);
-				if (db.next()) {
-					senderEmail = db.rs.getString("email");
-				}
-				db.close();
-				command = "SELECT email from og_email WHERE id = ? AND cid = ?";
-				com = new DbCommand(command);
-				com.addVariables(id);
-				com.addVariables(cid);
-				db = new DbQuery();
-				db.readOG(com);
-				if (db.next()) {
-					fromEmail = db.rs.getString("email");
-				}
-				db.close();
+			    //TODO: Use a DI-Container to resolve this
+			    OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+
+			    List<OGEmail> ogOwnerEmails = ogEmailStorage.getById(owner_cid, owner_id);
+			    if(ogOwnerEmails.size() > 0) {
+			        senderEmail = ogOwnerEmails.get(0).getEmail();
+			    }
+
+			    List<OGEmail> ogEmails = ogEmailStorage.getById(cid, id);
+			    if(ogEmails.size() > 0) {
+			        fromEmail = ogEmails.get(0).getEmail();
+                }
 			}
-			
+
 		} catch (Exception ex) {
 			logger.error("Problem retrieving email info ", ex);
 		}
 	}
-	
+
 	private void populate_recips (String XML) throws ParserConfigurationException, UnsupportedEncodingException, SAXException, IOException {
 		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
         DocumentBuilder builder = factory.newDocumentBuilder();
