@@ -50,9 +50,6 @@ package com.openexchange.guard.database;
 
 import java.io.IOException;
 import java.io.StringReader;
-import java.sql.ResultSet;
-import java.sql.Statement;
-
 import org.bouncycastle.crypto.AsymmetricCipherKeyPair;
 import org.bouncycastle.crypto.params.AsymmetricKeyParameter;
 import org.bouncycastle.crypto.util.PrivateKeyFactory;
@@ -61,10 +58,10 @@ import org.bouncycastle.openssl.PEMKeyPair;
 import org.bouncycastle.openssl.PEMParser;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.dbpool.ContextDBConnection;
+import com.openexchange.guard.database.keyCache.KeyCacheStorage;
+import com.openexchange.guard.database.keyCache.RestDbKeyCacheStorage;
 import com.openexchange.guard.encr.EncrLib;
 
 /**
@@ -74,7 +71,7 @@ import com.openexchange.guard.encr.EncrLib;
  *
  */
 public class KeyPairGen {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(Cache.class);
 
 
@@ -84,31 +81,20 @@ public class KeyPairGen {
 	 * @throws IOException
 	 */
 	public static synchronized AsymmetricCipherKeyPair getKeyPair() throws IOException {
-		ContextDBConnection conn = null; 
-		Statement stmt = null;
-		ResultSet rs = null;
 		String data = "";
 		try {
-			String command = "SELECT id, privkey FROM key_cache LIMIT 1";
-			DbCommand com = new DbCommand (command);
-			DbQuery db = new DbQuery ();
-			db.readOG(com);
-			if (db.next()) {
-				int id = db.rs.getInt("id");
-				data = db.rs.getString("privkey");
-				command = "DELETE FROM key_cache WHERE id = " + id;
-				com = new DbCommand (command);
-				db.writeOxGuard(com);
-			} else {
-				logger.warn("No more pre-compiles RSA keys");
-			}
-			db.close();
-			}
-			catch (Exception ex) {
-				logger.error("Error getting prepared key pair", ex);
-			} 
-		
-		if (data == "") return (null);
+		    //TODO: use a service locator or DI Container to resolve this
+		    KeyCacheStorage keyCacheStorage = new RestDbKeyCacheStorage();
+		    data =keyCacheStorage.popNext();
+		}
+		catch (Exception ex) {
+		    logger.error("Error getting prepared key pair", ex);
+		}
+
+		if (data == null) {
+            return (null);
+        }
+
 		EncrLib encr = new EncrLib();
 		data = encr.decryptAES(data, Config.rpass, "key", Config.AESKeyLen);
 		if (data.equals("")) {
