@@ -70,7 +70,7 @@ public class OGInit {
      */
     public static void initGuardDatabase() throws Exception {
         DbQuery db = new DbQuery();
-        String command = "SELECT write_db_pool_id, read_db_pool_id FROM db_cluster dc INNER JOIN db_pool dp ON dc.write_db_pool_id = dp.db_pool_id WHERE dp.name = 'oxguard';";
+        String command = "SELECT write_db_pool_id, read_db_pool_id FROM db_cluster dc INNER JOIN db_pool dp ON dc.write_db_pool_id = dp.db_pool_id WHERE dp.name = '" + Config.dbSchemaBaseName +  "';";
         DbCommand com = new DbCommand(command);
         db.readFromConfigDb(com);
         if (db.next()) {
@@ -82,7 +82,7 @@ public class OGInit {
             throw (new Exception("Write/Master Database not registered"));
         }
 
-        command = "SELECT write_db_pool_id, read_db_pool_id, name FROM db_cluster dc INNER JOIN db_pool dp ON dc.write_db_pool_id = dp.db_pool_id WHERE name LIKE '" + Config.db_prefix + "%';";
+        command = "SELECT write_db_pool_id, read_db_pool_id, name FROM db_cluster dc INNER JOIN db_pool dp ON dc.write_db_pool_id = dp.db_pool_id WHERE name LIKE '" + Config.getShardBaseName() + "%';";
         com = new DbCommand(command);
         db.readFromConfigDb(com);
         while (db.next()) {
