@@ -75,17 +75,15 @@ public class RecipKey {
     public String lang;
 
     public String email;
-    
+
     public boolean pgp;
-    
+
     public boolean expired;
-    
+
     public boolean inline;
-    
+
     public PGPPublicKeyRing pgpring;
 
-    public boolean remote;
-    
     public JsonObject settings;
 
     public RecipKey() {
@@ -93,7 +91,6 @@ public class RecipKey {
         userid = 0;
         pubkey = null;
         newGuestPass = null;
-        remote = false;
         pgp = false;
         pgpring = null;
         expired = false;
@@ -107,7 +104,6 @@ public class RecipKey {
         inline = key.inline;
         pubkey = null;
         newGuestPass = null;
-        remote = false;
         pgp = true;
         pgpring = key.pubring;
         email = key.email;
@@ -115,15 +111,26 @@ public class RecipKey {
         expired = PgpKeys.isExpired(key.getPGPPublicKey());
         settings = key.settings;
 	}
-	
+
+	/**
+	 * A remote key is a key which does not belong to an OX user but to an external recipient
+	 * @return Whether the key belongs to an external PGP user or not
+	 */
+	public boolean isRemoteKey() {
+	    return cid == 0;
+	}
 	   /**
      * Check if the user settings require inline
      * @param settings
      * @return
      */
     public boolean reqInline () {
-        if (inline) return(true);
-        if (settings == null) return (false);
+        if (inline) {
+            return(true);
+        }
+        if (settings == null) {
+            return (false);
+        }
         if (settings.has("inline")) {
             return (settings.get("inline").getAsBoolean());
         }
