@@ -54,14 +54,16 @@ import java.sql.DriverManager;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.Statement;
-
+import java.util.List;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
-import com.openexchange.guard.cacheitems.Cache;
 import com.openexchange.guard.config.Config;
+import com.openexchange.guard.database.ogEmail.OGEmail;
+import com.openexchange.guard.database.ogEmail.OGEmailStorage;
+import com.openexchange.guard.database.ogEmail.RestDbOGEmailStorage;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
 import com.openexchange.guard.dbpool.ContextDBConnection;
-import com.openexchange.guard.dbpool.DbConnection;
 import com.openexchange.guard.dbpool.Manager;
 import com.openexchange.guard.setup.RegisterDb;
 
@@ -78,12 +80,12 @@ public class Sharding {
     private static String OGmysqlIP = com.openexchange.guard.config.Config.getMysqlIPOxGuard();
 
     public static String database_name(int db) {
-        return (Config.db_prefix + db);
+        return (Config.getShardBaseName() + db);
     }
 
     /**
      * Create the tables within a shard
-     * 
+     *
      * @param database
      * @return
      */
@@ -106,8 +108,9 @@ public class Sharding {
         try {
             if (OX) {
                 db.writeToDB(com, database);
-            } else
+            } else {
                 db.writeToDb(com, dbase);
+            }
         } catch (Exception e) {
             logger.error("Failed to create Content Key Table in shard " + dbase, e);
             return (false);
@@ -116,8 +119,9 @@ public class Sharding {
         try {
             if (OX) {
                 db.writeToDB(com, database);
-            } else
+            } else {
                 db.writeToDb(com, dbase);
+            }
         } catch (Exception e) {
             logger.error("Failed to create Encrypted Object Table in shard " + dbase, e);
             return (false);
@@ -126,8 +130,9 @@ public class Sharding {
         try {
             if (OX) {
                 db.writeToDB(com, database);
-            } else
+            } else {
                 db.writeToDb(com, dbase);
+            }
         } catch (Exception e) {
             logger.error("Failed to create Encrypted Object Table in shard " + dbase, e);
             return (false);
@@ -135,56 +140,49 @@ public class Sharding {
         logger.info("Created tables in " + dbase);
         return (true);
     }
-    
+
     /**
      * Set auto_increment value of keytable above largest current guest id
      * @param database
      * @throws Exception
      */
     public static void setAutoIncr(int database) throws Exception {
-    	String command = "SELECT MAX(ID) FROM og_email WHERE cid < 0";
-    	DbCommand com = new DbCommand (command);
-    	DbQuery db = new DbQuery();
-    	db.readOG(com);
-    	db.next();
-    	int max = 1;
-    	if (!db.rs.isNull("MAX(ID)")) max = db.rs.getInt("MAX(ID)");
-    	db.close();
-    	command = "ALTER TABLE `og_KeyTable` AUTO_INCREMENT=?";
-    	com = new DbCommand (command);
-    	com.addVariables(max + 10);
-    	db = new DbQuery();
-    	db.writeToDb(com, database_name(database));
-    	db.close();
-    	
+
+        //TODO: Use a service locator for resolving this
+        RestDbOGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+
+        int max = ogEmailStorage.getHighestGuestId();
+        ogKeyTableStorage.updateAutoIncrementInShard(database_name(database), max+10);
     }
 
     public static boolean checkTables(int database, boolean OX) {
-        String command = "SELECT id FROM og_KeyTable LIMIT 1";
-        DbCommand com = new DbCommand(command);
-        DbQuery db = new DbQuery();
+        //TODO: Use a service locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        boolean exists = false;
         try {
-            if (OX) {
-                db.readFromDB(com, database);
-            } else
-                db.readFromDB(com, database_name(database));
-            if (db.hasResults()) {
-                db.close();
-                return (true);
+            if(OX) {
+                exists = ogKeyTableStorage.exists(database);
             }
-            db.close();
-            return (true);
-        } catch (Exception ex) {
-        	if (ex.getMessage().contains("doesn't exist")) {
-        		if (OX) {
+            else {
+                exists = ogKeyTableStorage.existsInShard(database_name(database));
+            }
+            if(!exists) {
+                if (OX) {
                     logger.info("Tables don't exist for OG Database for cid " + database);
-                } else
+                }
+                else {
                     logger.info("Tables don't exist for shard " + database_name(database));
-        		return (createTables(database, OX));
-        	}
-            
+                }
+                return createTables(database, OX);
+            }
+            else {
+                return true;
+            }
+        }
+        catch(Exception e) {
+            return false;
         }
-        return (false);
     }
 
     // On hold for now.  Trouble with Lock Tables and API
@@ -222,7 +220,7 @@ public class Sharding {
         db.close();
         return (shard);
     }
-    
+
     /**
      * Get the next shard number.  Locks tables for singular action
      * @return
@@ -247,7 +245,9 @@ public class Sharding {
 	                if (count == 0) { // If brand new shard, create it
 	                    if (newShard(shard)) {
 	                    	createTables(shard, false);
-	                    } else return (0);
+	                    } else {
+                            return (0);
+                        }
 	                }
 	        	} catch (Exception ex) {
 	        		logger.error("Problem creating shard ", ex);
@@ -256,7 +256,9 @@ public class Sharding {
         } catch (Exception ex) {
         	logger.error("Error getting next shard" , ex);
         }
-        if (rs != null) rs.close();
+        if (rs != null) {
+            rs.close();
+        }
         command = "UNLOCK TABLES";
         stm.execute(command);
         stm.close();
@@ -265,41 +267,31 @@ public class Sharding {
     }
 
     public static int getShard(int id, int cid) throws Exception {
+        //TODO: resolve using a DI Container
+        OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
 
-        String command = "SELECT db FROM og_email WHERE id = ? and cid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(id);
-        com.addVariables(cid);
-        DbQuery db = new DbQuery();
-        db.readOG(com);
-        if (db.next()) {
-            int shard = db.rs.getInt("db");
-            db.close();
-            return (shard);
+        List<OGEmail> ogEmails = ogEmailStorage.getById(cid, id);
+        if(ogEmails.size() > 0) {
+            return ogEmails.get(0).getShardingId();
         }
-        db.close();
-        return (-1);
+        return -1;
     }
 
     public static int getShard(String email) throws Exception {
-        String command = "SELECT db FROM og_email WHERE email = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(email);
-        DbQuery db = new DbQuery();
-        db.readOG(com);
-        if (db.next()) {
-            int shard = db.rs.getInt("db");
-            db.close();
-            return (shard);
+        //TODO: resolve using a DI Container
+        OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+
+        OGEmail ogEmail = ogEmailStorage.getByEmail(email);
+        if(ogEmail != null) {
+            return ogEmail.getShardingId();
         }
-        db.close();
-        return (-1);
+        return -1;
     }
 
     /**
      * Create new shard database and register with OX backend SHOULD BE REWRITTEN WITH API WHEN
      * AVAILABLE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
-     * 
+     *
      * @param shard
      * @return
      * @throws InstantiationException
@@ -309,7 +301,9 @@ public class Sharding {
      */
     public static boolean newShard(int shard) throws InstantiationException, IllegalAccessException, ClassNotFoundException, SQLException {
         Class.forName("com.mysql.jdbc.Driver").newInstance();
-        if (Config.ogShardIP == null) Config.ogShardIP = OGmysqlIP;
+        if (Config.ogShardIP == null) {
+            Config.ogShardIP = OGmysqlIP;
+        }
         String OGmysqlConnectionString = "jdbc:mysql://" + Config.ogShardIP + "/?user=" + mysqlUsername + "&password=" + mysqlPassword;
         String OGShardRead = Config.ogShardReadIP == null ? null : ("jdbc:mysql://" + Config.ogShardReadIP + "/");
         String OgShardWrite = "jdbc:mysql://" + Config.ogShardIP + "/";
@@ -317,17 +311,17 @@ public class Sharding {
 
         // Register database
 
-        String createDatabase = "CREATE DATABASE oxguard_" + shard + " DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci";
+        String createDatabase = "CREATE DATABASE " + Config.getShardBaseName() + shard + " DEFAULT CHARACTER SET utf8 DEFAULT COLLATE utf8_general_ci";
 
         try {
             mysqlExecute(createDatabase, intconn);
         } catch (Exception ex) {
-            logger.error("Unable to create OxGuard_" + shard + " database", ex);
+            logger.error("Unable to create " + Config.getShardBaseName()+ shard + " database", ex);
             return (false);
         }
         Connection configdb = DriverManager.getConnection(Config.getConfigDbConnectionString());
         try {
-			RegisterDb.register(configdb, OgShardWrite, OGShardRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, "oxguard_" + shard);
+			RegisterDb.register(configdb, OgShardWrite, OGShardRead, "com.mysql.jdbc.Driver", mysqlUsername, mysqlPassword, Config.getShardBaseName() + shard);
 		} catch (Exception e) {
 			logger.error("Problem registering new database shard", e);
 	        configdb.close();
