@@ -86,7 +86,6 @@ import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.exceptions.NoContentKeyException;
 import com.openexchange.guard.exceptions.NoItemException;
 import com.openexchange.guard.exceptions.NoMasterKeysException;
-import com.openexchange.guard.ox.InterOx;
 import com.openexchange.guard.pgp.PgpKeys;
 import com.openexchange.guard.server.CheckBad;
 import com.openexchange.guard.server.OxCookie;
@@ -106,7 +105,7 @@ public class Crypto {
         StringBuilder addxml = new StringBuilder();
         addxml.append("<share>");
         for (RecipKey user : recipients) {
-            addxml.append("<u><e>" + StringEscapeUtils.escapeXml11(user.email) + "</e><r>" + (user.remote ? "Y" : "N") + "</r><i>" + user.userid + "</i><c>" + user.cid + "</c></u>");
+            addxml.append("<u><e>" + StringEscapeUtils.escapeXml11(user.email) + "</e><i>" + user.userid + "</i><c>" + user.cid + "</c></u>");
         }
         addxml.append("</share>");
         String finalXML = XML + addxml.toString();
@@ -119,33 +118,22 @@ public class Crypto {
             return(null);
         }
         for (RecipKey user : recipients) {
-            if (user.remote) {// If remote user, we're going to send the data.
-                EncrLib crypto = new EncrLib();
-                String encryptedContentKey = crypto.encryptRSA(contentKey, user.pubkey);
-                InterOx iox = new InterOx();
-                if (!iox.sendItem(user.email, itemID, encryptedContentKey, exp, type, XML, salt)) {
-                    return (null);
-                }
+            if (user.cid < 0) // Guest account
+            {
+                // user.cid = -user.cid;
+                logger.debug("Non OX user " + user.cid + " sending to filesystem");
 
-            } else {// If local, we need to store it here
-                if (user.cid < 0) // Guest account
-                {
-                    // user.cid = -user.cid;
-                    logger.debug("Non OX user " + user.cid + " sending to filesystem");
-
-                    if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID, data.length()))) {
-                        return (null);
-                    }
-                }
-                if (!saveContentKey(itemID, user.userid, user.cid, user.pubkey, contentKey, acc)) {
+                if (!Storage.saveEncrObj(user.userid, user.cid, itemID, addHeader(encrypted, itemID, data.length()))) {
                     return (null);
                 }
-                acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
-                if (user.cid != owner_cid) { // We need to save the item in each context, but as not owner, userid is 0
-                    acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
-                }
             }
-
+            if (!saveContentKey(itemID, user.userid, user.cid, user.pubkey, contentKey, acc)) {
+                return (null);
+            }
+            acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
+            if (user.cid != owner_cid) { // We need to save the item in each context, but as not owner, userid is 0
+                acc.storeItemID(itemID, user.userid, user.cid, owner, owner_cid, exp, type, XML, salt);
+            }
         }
         logger.info("Encrypted email" + start);
         return (encrypted);
