@@ -65,7 +65,6 @@ import java.security.spec.RSAPrivateKeySpec;
 import java.security.spec.RSAPublicKeySpec;
 import java.util.Random;
 import java.util.UUID;
-
 import javax.crypto.Cipher;
 import javax.crypto.KeyGenerator;
 import javax.crypto.SecretKey;
@@ -73,18 +72,16 @@ import javax.crypto.SecretKeyFactory;
 import javax.crypto.spec.IvParameterSpec;
 import javax.crypto.spec.PBEKeySpec;
 import javax.crypto.spec.SecretKeySpec;
-
 import org.apache.commons.codec.binary.Base64;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.openexchange.guard.config.Config;
 
 public class EncrLib {
 	private static Logger logger = LoggerFactory.getLogger(EncrLib.class);
     /**
      * Encrypt String and return base64 encrypted string
-     * 
+     *
      * @param data string to encrypt
      * @param Password
      * @param Salt
@@ -99,7 +96,7 @@ public class EncrLib {
             SecretKey secretKey = getKey(Password, Salt, keylen);
             return (encryptAES(data, secretKey));
         } catch (Exception e) {
-            e.printStackTrace();
+            logger.error("Error encrypting", e);
             return ("");
         }
     }
@@ -116,7 +113,7 @@ public class EncrLib {
             String combo = iVString + encrypted;
             return (combo);
         } catch (Exception e) {
-            e.printStackTrace();
+            logger.error("Error encrypting", e);
             return ("");
         }
     }
@@ -132,7 +129,7 @@ public class EncrLib {
 
     /**
      * Decrypt a base64 string
-     * 
+     *
      * @param data base64 string to decrypt
      * @param Password
      * @param Salt
@@ -145,8 +142,9 @@ public class EncrLib {
             SecretKey secretKey = getKey(Password, Salt, keylen);
             return (decryptAES(data, secretKey, logbad));
         } catch (Exception e) {
-            if (logbad)
+            if (logbad) {
                 logger.info("Bad Pass");
+            }
         }
         return ("");
     }
@@ -167,15 +165,16 @@ public class EncrLib {
             String plaintext = new String(ciper.doFinal(databytes), "UTF-8");
             return (plaintext);
         } catch (Exception e) {
-            if (logbad)
-                logger.info("Bad Pass");
+            if (logbad) {
+                logger.info(e.getMessage());
+            }
         }
         return ("");
     }
 
     /**
      * Function to generate random salt
-     * 
+     *
      * @return string of random salt
      */
     public static String getsalt() {
@@ -187,7 +186,7 @@ public class EncrLib {
 
     /**
      * Generate a random SecretKey
-     * 
+     *
      * @return
      * @throws NoSuchProviderException
      * @throws NoSuchAlgorithmException
@@ -210,7 +209,7 @@ public class EncrLib {
 
     /**
      * Generate key from password and salt
-     * 
+     *
      * @param Password
      * @param Salt
      * @return SecretKey
@@ -248,7 +247,7 @@ public class EncrLib {
             cipher.init(Cipher.ENCRYPT_MODE, key);
             cipherText = cipher.doFinal(text.getBytes("UTF-8"));
         } catch (Exception e) {
-            e.printStackTrace();
+            logger.error("Error RSA encrypt", e);
         }
         return (Base64.encodeBase64String(cipherText));
     }
@@ -262,7 +261,7 @@ public class EncrLib {
             cipher.init(Cipher.ENCRYPT_MODE, key);
             cipherText = cipher.doFinal(text.getBytes("UTF-8"));
         } catch (Exception e) {
-            e.printStackTrace();
+            logger.error("Error RSA encrypt", e);
         }
         return (Base64.encodeBase64String(cipherText));
 
@@ -270,7 +269,9 @@ public class EncrLib {
 
     public String decryptRSA(String enctext, PrivateKey key) {
         if (key == null)
+         {
             return (null); // Passed bad private key
+        }
         byte[] text = Base64.decodeBase64(enctext);
         byte[] dectyptedText = null;
         try {
@@ -282,7 +283,8 @@ public class EncrLib {
             dectyptedText = cipher.doFinal(text);
 
         } catch (Exception ex) {
-            ex.printStackTrace();
+            logger.error("Problem decoding RSA", ex);
+            return(null);
         }
 
         return new String(dectyptedText);
@@ -290,7 +292,9 @@ public class EncrLib {
 
     public String decryptRSA(String enctext, PublicKey key) {
         if (key == null)
+         {
             return (null); // Passed bad private key
+        }
         byte[] text = Base64.decodeBase64(enctext);
         byte[] dectyptedText = null;
         try {
@@ -302,7 +306,8 @@ public class EncrLib {
             dectyptedText = cipher.doFinal(text);
 
         } catch (Exception ex) {
-            ex.printStackTrace();
+            logger.error("Problemd decoding RSA ", ex);
+            return(null);
         }
 
         return new String(dectyptedText);
