@@ -49,52 +49,47 @@
 
 package com.openexchange.guard.encr;
 
-import org.apache.commons.codec.binary.Base64;
-import org.bouncycastle.bcpg.ArmoredOutputStream;
-import org.bouncycastle.openpgp.PGPKeyRingGenerator;
-import java.util.ArrayList;
-import java.util.Iterator;
-import java.util.List;
 import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.UnsupportedEncodingException;
 import java.security.KeyFactory;
 import java.security.NoSuchAlgorithmException;
-import java.security.PublicKey;
 import java.security.PrivateKey;
+import java.security.PublicKey;
 import java.security.interfaces.RSAPublicKey;
 import java.security.spec.InvalidKeySpecException;
 import java.security.spec.PKCS8EncodedKeySpec;
 import java.security.spec.X509EncodedKeySpec;
 import java.sql.Date;
+import java.util.Iterator;
+import org.apache.commons.codec.binary.Base64;
+import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.bouncycastle.openpgp.PGPException;
+import org.bouncycastle.openpgp.PGPKeyRingGenerator;
 import org.bouncycastle.openpgp.PGPPrivateKey;
-import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPPublicKey;
+import org.bouncycastle.openpgp.PGPPublicKeyRing;
 import org.bouncycastle.openpgp.PGPSecretKey;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
-import org.bouncycastle.openpgp.operator.PGPDigestCalculatorProvider;
 import org.bouncycastle.openpgp.operator.bc.BcKeyFingerprintCalculator;
 import org.bouncycastle.openpgp.operator.jcajce.JcePBESecretKeyDecryptorBuilder;
-import org.bouncycastle.openpgp.PGPUtil;
-import org.bouncycastle.openpgp.PGPObjectFactory;
-import org.bouncycastle.operator.bc.BcDigestCalculatorProvider;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.google.gson.JsonObject;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.RecipKey;
-import com.openexchange.guard.database.model.DeletedKey;
+import com.openexchange.guard.database.deletedKey.DeletedKey;
 import com.openexchange.guard.exceptions.BadPasswordException;
+import com.openexchange.guard.pgp.PGPPublicKeyRingFactory;
 
 /**
  * Manage PGP and RSA keys. Call create and encrypt of keys
- * 
+ *
  * @author greg
  */
 public class GuardKeys {
 	private static Logger logger = LoggerFactory.getLogger(GuardKeys.class);
-	
+
     PGPSecretKeyRing keyring;
 
     public PGPPublicKeyRing pubring;
@@ -118,23 +113,23 @@ public class GuardKeys {
     public JsonObject settings;
 
     public String email;
-    
+
     public String question;
-    
+
     public String answer;
-    
+
     public String recovery;
-    
+
     public long keyid;
-     
+
     public boolean current;
-    
+
     public boolean inline = false;
-    
+
     public int version;
-    
+
     public String language;
-    
+
     public boolean recoveryAvail = false;
 
     public GuardKeys() {
@@ -147,7 +142,7 @@ public class GuardKeys {
         pubkey = key.pubkey;
         email = key.email;
     }
-    
+
     /**
      * Populate guardkey based on deleted key
      * @param key
@@ -183,7 +178,7 @@ public class GuardKeys {
     }
     /**
      * Set the private RSA key, need password and salt to encrypt
-     * 
+     *
      * @param priv
      * @param password
      * @param salt
@@ -194,14 +189,15 @@ public class GuardKeys {
         privkey = crypto.encryptAES(Base64.encodeBase64String(priv.getEncoded()), EncrLib.getSHA(password, salt), salt, Config.AESKeyLen);
         if (privkey.equals("")) { // Return false if failed to save
             return (false);
-        } else
+        } else {
             return (true);
+        }
 
     }
 
     /**
      * Set the already encoded private RSA
-     * 
+     *
      * @param priv
      */
     public void setEncodedPrivate(String priv) {
@@ -211,21 +207,15 @@ public class GuardKeys {
     public PrivateKey getDecodedPrivate(String password) throws BadPasswordException, UnsupportedEncodingException {
         return (getDecodedPrivate(password, true));
     }
-    
+
     public boolean verifyPassword (String password) throws BadPasswordException, PGPException, UnsupportedEncodingException {
-        if (password == null) return(false);
-        if (privkey != null) {
-        	try {
-        		if (getDecodedPrivate (password, true) != null) {
-        			return(true);
-        		} else return (false);
-        	} catch (BadPasswordException e) {
-        		logger.warn("Bad password");
-        		return(false);
-        	}
+        if (password == null) {
+            return(false);
         }
         try {
-            if (getPGPEncrKey (password) != null) return (true);
+            if (getPGPEncrKey (password) != null) {
+                return (true);
+            }
         } catch (Exception e) {
             return(false);
         }
@@ -234,19 +224,20 @@ public class GuardKeys {
 
     /**
      * Retrieve the Private RSA key using password and salt
-     * 
+     *
      * @param password
      * @param logbad Log Bad passwords, false if just testing
      * @return
-     * @throws BadPasswordException 
-     * @throws UnsupportedEncodingException 
+     * @throws BadPasswordException
+     * @throws UnsupportedEncodingException
      */
     public PrivateKey getDecodedPrivate(String password, boolean logbad) throws BadPasswordException, UnsupportedEncodingException {
-    	
+
             EncrLib crypto = new EncrLib();
             byte[] decoded = crypto.decryptAES(privkey, EncrLib.getSHA(password, salt), salt, Config.AESKeyLen, logbad).getBytes("UTF-8");
-            if (decoded.length == 0)
-            	throw new BadPasswordException("Bad Password");
+            if (decoded.length == 0) {
+                throw new BadPasswordException("Bad Password");
+            }
             // create a keyfactory - use whichever algorithm and provider
             KeyFactory kf;
 			try {
@@ -269,7 +260,7 @@ public class GuardKeys {
 
     /**
      * Retrieve the Private RSA key using password and salt
-     * 
+     *
      * @param password
      * @param salt
      * @return
@@ -292,7 +283,7 @@ public class GuardKeys {
 
     /**
      * Set the RSA public key from stored string
-     * 
+     *
      * @param data
      * @throws InvalidKeySpecException
      * @throws NoSuchAlgorithmException
@@ -311,7 +302,7 @@ public class GuardKeys {
 
     /**
      * Return public RSA key
-     * 
+     *
      * @return
      */
     public PublicKey getPublic() {
@@ -320,17 +311,19 @@ public class GuardKeys {
 
     /**
      * Get Base64 representation of Public key
-     * 
+     *
      * @return
      */
     public String getEncodedPublic() {
-        if (pubkey == null) return (null);
+        if (pubkey == null) {
+            return (null);
+        }
         return (Base64.encodeBase64String(pubkey.getEncoded()));
     }
 
     /**
      * Get the Base64 Encoded and encrypted Private key
-     * 
+     *
      * @return
      */
     public String getEncodedPrivate() {
@@ -339,7 +332,7 @@ public class GuardKeys {
 
     /**
      * Set RSA Public Key
-     * 
+     *
      * @param pub
      */
     public void setPublic(PublicKey pub) {
@@ -348,7 +341,7 @@ public class GuardKeys {
 
     /**
      * Return the Public Key
-     * 
+     *
      * @return
      */
     public PGPPublicKey getPGPPublicKey() {
@@ -366,7 +359,7 @@ public class GuardKeys {
 
     /**
      * Get Base64 Encoded String
-     * 
+     *
      * @return
      * @throws IOException
      */
@@ -376,22 +369,26 @@ public class GuardKeys {
 
     /**
      * Return the secret Key
-     * 
+     *
      * @return
      */
     public PGPSecretKey getPGPSecretKey() {
-        if (keyring == null) return(null);
+        if (keyring == null) {
+            return(null);
+        }
         return (keyring.getSecretKey());
     }
-    
+
     public PGPSecretKey getPGPSigningKey() {
     	Iterator it = keyring.getSecretKeys();
     	while (it.hasNext()) {
     		PGPSecretKey key = (PGPSecretKey) it.next();
     		if (key.isMasterKey()) {
     		    return (key);
-    		} else 
-    			if (key.isSigningKey()) return(key);
+    		} else
+    			if (key.isSigningKey()) {
+                    return(key);
+                }
     	}
     	return (null);
     }
@@ -402,18 +399,20 @@ public class GuardKeys {
 
     /**
      * Get Base64 Encoded Secret Keys
-     * 
+     *
      * @return
      * @throws IOException
      */
     public String getEncodedPGPSecret() throws IOException {
-        if (keyring == null) return(null);
+        if (keyring == null) {
+            return(null);
+        }
         return (Base64.encodeBase64String(keyring.getEncoded()));
     }
 
     /**
      * Set PGPSecret info from base64 string
-     * 
+     *
      * @param keystring
      * @return
      * @throws IOException
@@ -430,7 +429,7 @@ public class GuardKeys {
 
     /**
      * Get public key from base64 String
-     * 
+     *
      * @param keystring Base64String
      * @return
      * @throws IOException
@@ -449,84 +448,8 @@ public class GuardKeys {
 
     }
 
-    /**
-     * Get encryption public key from ASC data
-     * 
-     * @param ascdata
-     * @return
-     * @throws IOException
-     */
-    public PGPPublicKey getPGPKeyfromASC(String ascdata) throws IOException {
-        return (getEncryptionKey(getKeyring(ascdata)));
-    }
-
-    public void setPGPKeyRingFromAsc (String ascdata) throws IOException {
-        pubring = getKeyring(ascdata);
-    }
-    
-    private PGPPublicKeyRing getKeyring(String AscData) throws IOException {
-        // PGPUtil.getDecoderStream() will detect ASCII-armor automatically and decode it,
-        // the PGPObject factory then knows how to read all the data in the encoded stream
-        try {
-            PGPObjectFactory factory = new PGPObjectFactory(PGPUtil.getDecoderStream(new java.io.ByteArrayInputStream(AscData.getBytes("UTF-8"))), new BcKeyFingerprintCalculator());
-            // these files should really just have one object in them,
-            // and that object should be a PGPPublicKeyRing.
-            Object o = factory.nextObject();
-            if (o instanceof PGPPublicKeyRing) {
-                return (PGPPublicKeyRing) o;
-            }
-        } catch (Exception ex) {
-            logger.error("Error creating keyring " ,ex);
-             
-        }
-
-        throw new IllegalArgumentException("Input text does not contain a PGP Public Key");
-    }
-
-    /**
-     * Get the first encyption key off the given keyring.
-     */
-    private PGPPublicKey getEncryptionKey(PGPPublicKeyRing keyRing) {
-        if (keyRing == null)
-            return null;
-
-        // iterate over the keys on the ring, look for one
-        // which is suitable for encryption.
-        Iterator keys = pubring.getPublicKeys();
-        PGPPublicKey key = null;
-        while (keys.hasNext()) {
-            key = (PGPPublicKey) keys.next();
-            if (key.isEncryptionKey()) {
-                if (!key.isMasterKey()) {
-                    return key;
-                }
-            }
-        }
-        return null;
-    }
-
-    /**
-     * Export PGP Public keys in standard armored format
-     * 
-     * @return
-     * @throws IOException
-     */
-    public String exportPgpPublicKey() throws IOException {
-        ByteArrayOutputStream out = new ByteArrayOutputStream();
-        ArmoredOutputStream arm = new ArmoredOutputStream(out);
-        // PGPPublicKeyRing pbkeyring = keyring.get
-        // arm.write(pbkeyring.getEncoded());
-        Iterator<PGPPublicKey> keys = pubring.getPublicKeys();
-        ByteArrayOutputStream bout = new ByteArrayOutputStream();
-        while (keys.hasNext()) {
-            PGPPublicKey k = keys.next();
-            k.encode(bout);
-        }
-        arm.write(bout.toByteArray());
-        arm.close();
-        bout.close();
-        out.close();
-        return (new String(out.toByteArray()));
+   public void setPGPKeyRingFromAsc (String ascdata) throws IOException {
+        pubring = PGPPublicKeyRingFactory.create(ascdata);
     }
 
     public String exportPGPPublicKeyRing() throws IOException {
@@ -538,19 +461,19 @@ public class GuardKeys {
         return (new String(out.toByteArray()));
     }
 
-    public void setPGPPublicKeyRing(String data) {
-        try {
-            pubring = getKeyring(data);
-        } catch (IOException e) {
-            logger.error("Failed to store in public key ring", e);
-        }
-    }
-    
+//    public void setPGPPublicKeyRing(String data) {
+//        try {
+//            pubring = PGPPublicKeyRingFactory.createPublicKeyRing(data);
+//        } catch (IOException e) {
+//            logger.error("Failed to store in public key ring", e);
+//        }
+//    }
+
     /**
      * Get PGP Private Signing Key
      * @param password
      * @return
-     * @throws PGPException 
+     * @throws PGPException
      */
     public PGPPrivateKey getPGPSignKey (String password) throws PGPException {
     	PGPSecretKey sec_key = null;
@@ -564,7 +487,7 @@ public class GuardKeys {
     	logger.error("Unable to find PGP signing key");
     	return (null);
     }
-    
+
     public PGPSecretKey getMasterPGPSecretKey () {
     	PGPSecretKey sec_key = null;
     	Iterator<PGPSecretKey> it = keyring.getSecretKeys();
@@ -577,12 +500,12 @@ public class GuardKeys {
     	logger.error("Unable to find PGP signing key");
     	return (null);
     }
-    
+
     /**
      * Get PGP private key for encrypting
      * @param password
      * @return
-     * @throws PGPException 
+     * @throws PGPException
      */
     public PGPPrivateKey getPGPEncrKey (String password) throws PGPException {
     	PGPSecretKey sec_key = null;
@@ -592,19 +515,23 @@ public class GuardKeys {
     		PGPSecretKey key = it.next();
     		if (!key.isMasterKey()) { // We prefer to not return the master.  Only return master if no other encr keys found
     			return (getDecodedPrivate(password, key));
-    		} else master = key;
+    		} else {
+                master = key;
+            }
     	}
-    	if (master != null) return (getDecodedPrivate(password, master));
+    	if (master != null) {
+            return (getDecodedPrivate(password, master));
+        }
     	logger.error("Unable to find PGP encrypting key");
     	return (null);
     }
-    
+
     /**
      * Get PGP Private key associated with ID
      * @param password
      * @param id
      * @return
-     * @throws PGPException 
+     * @throws PGPException
      */
     public PGPPrivateKey getPGPKey (String password, long id) throws PGPException {
     	PGPSecretKey sec_key = null;
@@ -618,7 +545,7 @@ public class GuardKeys {
     	logger.error("Unable to find PGP key for id " + Long.toString(id));
     	return (null);
     }
-    
+
     private PGPPrivateKey getDecodedPrivate (String password, PGPSecretKey sec_key) throws PGPException
     {
     	char [] pass = EncrLib.getSHA(password, salt).toCharArray();
@@ -626,9 +553,9 @@ public class GuardKeys {
     	//org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor decryptor = new org.bouncycastle.openpgp.operator.bc.BcPBESecretKeyDecryptorBuilder(new org.bouncycastle.openpgp.operator.bc.BcPGPDigestCalculatorProvider()).build(pass);
         return (decryptor);
     }
-    
+
     public PGPPublicKey getPGPPublicKey (long id) {
-    	
+
     	Iterator<PGPPublicKey> it = pubring.getPublicKeys();
     	while (it.hasNext()) {
     		PGPPublicKey key = it.next();
