@@ -15,6 +15,7 @@ import com.openexchange.guard.database.Access;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.logging.LogAction;
 import com.openexchange.guard.mailcreator.Attachment;
@@ -26,21 +27,21 @@ import com.openexchange.guard.util.Core;
 public class Attachments {
 
 	private static Logger logger = LoggerFactory.getLogger(Attachments.class);
-	
+
     OxCookie cookie;
 
     public Attachments(OxCookie ck) {
         cookie = ck;
     }
-    
+
 	   /**
      * Function to return encrypted attachment (non email)
-     * 
+     *
      * @param request
      * @param response
      * @param cookie
      * @throws IOException
-	 * @throws GuardMissingParameter 
+	 * @throws GuardMissingParameter
      */
     public void getEncrAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
     	String emailid = Core.getStringParameter(request, "emailid", true);
@@ -114,7 +115,7 @@ public class Attachments {
 
     }
 
-    public void saveDecodedAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
+    public void saveDecodedAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter, BadRSAException {
         JsonObject json = com.openexchange.guard.util.Core.getJSON(request);
         String folder = Core.getStringFromJson(json, "folder", true);
         String emailid = json.get("id").getAsString();
@@ -225,7 +226,7 @@ public class Attachments {
                         decoded.decoded64 = encr.decryptAES(decoded.decoded64.substring(5), extrapass, attdata.ItemID, attdata.keyLength);
                     }
                 }
-                
+
                 return (decoded);
             }
         } catch (Exception e) {
@@ -235,7 +236,7 @@ public class Attachments {
         return (null);
     }
 
- 
+
 
 
 
@@ -243,7 +244,7 @@ public class Attachments {
 
     /**
      * Send attachment contents to browser with flags to save to disk
-     * 
+     *
      * @param response
      * @param type
      * @param answer
