@@ -54,11 +54,15 @@ public class Attachment {
     public String filename;
 
     public String type;
+    
+    public String content_id = null;
 
     public byte[] content;
 
     public byte[] encrContent;
     
     public boolean sizeMatch;
+    
+    public boolean pgp;
 
 }
