@@ -444,12 +444,21 @@ public class Api {
         return (null);
     }
 
+    /**
+     * Send an email through the backend in MIME format
+     * @param data - email data
+     * @param folder - folder if should be stored.  Null if sending
+     * @return
+     * @throws UnsupportedEncodingException
+     */
     public String sendMimeMail (String data, String folder) throws UnsupportedEncodingException {
     	String params = "mail?action=new";
         params += "&session=" + session;
         if (folder != null) {
         	params += "&folder=" + URLEncoder.encode(folder, "UTF-8");
         	params += "&flags=32"; //Marks the email as "read" while putting it in the folder
+        } else {
+            params += "&copy2Sent=false";
         }
         Gson gson = new GsonBuilder().disableHtmlEscaping().create();
 
@@ -517,7 +526,7 @@ public class Api {
 
     public EncryptedObj getAttachment(String id, int attachment, String folder) throws UnsupportedEncodingException {
         long start = System.currentTimeMillis();
-        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder) + "&id=" + URLEncoder.encode(id, "UTF-8") + "&attachment=" + attachment;
+        String params = "mail?action=attachment" + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&id=" + URLEncoder.encode(id, "UTF-8") + "&attachment=" + attachment;
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -649,8 +658,8 @@ public class Api {
         return ("");
     }
 
-    public void getFileList(int folder) {
-        String params = "files?action=all&folder=" + folder + "&session=" + session + "&columns=1,711";
+    public void getFileList(String folder) throws UnsupportedEncodingException {
+        String params = "files?action=all&folder=" + URLEncoder.encode(folder, "UTF-8") + "&session=" + session + "&columns=1,711";
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -684,7 +693,7 @@ public class Api {
         }
     }
 
-    public String updateFile(Attachment file, String data, int id, int folder) throws UnsupportedEncodingException {
+    public String updateFile(Attachment file, String data, int id, String folder) throws UnsupportedEncodingException {
 
         ArrayList<Attachment> attachments = new ArrayList<Attachment>();
         attachments.add(file);
@@ -704,17 +713,18 @@ public class Api {
         return ("");
     }
 
-    public JsonObject getInfoItemJson (String id, int folder) throws UnsupportedEncodingException {
-    	String json = getInfoItemJson (id, folder, -1);
+
+    public JsonObject getInfoItemJson (String id, String folder) throws UnsupportedEncodingException {
+    	String json = getInfoItemJson (id, folder, null);
     	JsonParser parser = new JsonParser();
         JsonObject result = (JsonObject) parser.parse(json);
         return(result);
     }
 
-    public String getInfoItemJson(String id, int folder, int version) throws UnsupportedEncodingException {
-    	String params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&version=" + version + "&session=" + session;
-    	if (version == -1) {
-    		params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&session=" + session;
+    public String getInfoItemJson(String id, String folder, String version) throws UnsupportedEncodingException {
+    	String params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&version=" + version + "&session=" + session;
+    	if (version == null) {
+    		params = "files?action=get&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&session=" + session;
     	}
         try {
             CloseableHttpResponse response = getResponse(params);
@@ -746,8 +756,8 @@ public class Api {
         return (null);
     }
 
-    public JsonObject importVcard (Attachment vcard, String folder) {
-        String params = "import?action=VCARD&folder=" + URLEncoder.encode(folder) + "&session=" + session;
+    public JsonObject importVcard (Attachment vcard, String folder) throws UnsupportedEncodingException {
+        String params = "import?action=VCARD&folder=" + URLEncoder.encode(folder, "UTF-8") + "&session=" + session;
         try {
             ArrayList<Attachment> atts = new ArrayList<Attachment>();
             atts.add(vcard);
@@ -759,8 +769,8 @@ public class Api {
         return (null);
     }
 
-    public byte[] getPlainTextInfoStore(int id, int folder, int version) {
-        String params = "files?action=document&id=" + id + "&folder=" + folder + "&version=" + version + "&session=" + session;
+    public byte[] getPlainTextInfoStore(int id, String folder, String version) throws UnsupportedEncodingException {
+        String params = "files?action=document&id=" + id + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&version=" + version + "&session=" + session;
         try {
             CloseableHttpResponse response = getResponse(params);
             HttpEntity ent = response.getEntity();
@@ -783,8 +793,8 @@ public class Api {
      */
     public InputStream openDriveItem(String fileId,
         long fileSize,
-        int folderId,
-        int fileVersion) throws URISyntaxException, IOException {
+        String folderId,
+        String fileVersion) throws URISyntaxException, IOException {
 
         //building the uri
         URIBuilder builder = new URIBuilder().
@@ -796,7 +806,7 @@ public class Api {
             setParameter("id", fileId).
             setParameter("folder",String.valueOf(folderId)).
             setParameter("session",String.valueOf(session));
-        if(fileVersion > -1) {
+        if(fileVersion != null) {
             builder.setParameter("version",String.valueOf(fileVersion));
         }
 
@@ -843,10 +853,10 @@ public class Api {
      * @return
      * @throws UnsupportedEncodingException
      */
-    public EncryptedObj getInfoStore(String id, int folder, int version) throws UnsupportedEncodingException {
-        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&version=" + version + "&session=" + session;
-        if (version == -1) {
-        	params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + "&session=" + session;
+    public EncryptedObj getInfoStore(String id, String folder, String version) throws UnsupportedEncodingException {
+        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&version=" + version + "&session=" + session;
+        if (version == null) {
+        	params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8") + "&session=" + session;
         }
         long start = System.currentTimeMillis();
         try {
@@ -876,8 +886,8 @@ public class Api {
      * @return
      * @throws UnsupportedEncodingException
      */
-    public Attachment getInfoStoreAttach(String id, int folder, int version) throws UnsupportedEncodingException {
-        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + folder + ((version >= 0) ? ("&version=" + version) : "") + "&session=" + session;
+    public Attachment getInfoStoreAttach(String id, String folder, String version) throws UnsupportedEncodingException {
+        String params = "files?action=document&id=" + URLEncoder.encode(id, "UTF-8") + "&folder=" + URLEncoder.encode(folder, "UTF-8") + ((version != null) ? ("&version=" + version) : "") + "&session=" + session;
         try {
             Attachment attach = new Attachment();
             CloseableHttpResponse response = getResponse(params);
@@ -911,13 +921,16 @@ public class Api {
     public Attachment getInlineImage (String url) {
     	try {
     		url = java.net.URLDecoder.decode(url, "UTF-8").replace("&amp;","&");
-    		if (url.contains("file?action") || url.contains("image/mail")) {
+    		if (url.contains("file?action") || url.contains("image/mail") || url.contains("image/snippet")){
     			int index = 0;
     			if (url.contains("file?action")) {
                     index = url.indexOf("file?action");
                 }
     			if (url.contains("image/mail")) {
                     index = url.indexOf("image/mail");
+                }
+                if (url.contains("image/snippet")) {
+                    index = url.indexOf("image/snippet");
                 }
 	        	String params = url.substring(index);
 	            Attachment attach = new Attachment();
@@ -1131,7 +1144,8 @@ public class Api {
                                              String fileName,
                                              ContentType contentType,
                                              InputStream jsonStream,
-                                             JsonStreamHandler jsonHandler) throws Exception {
+                                             JsonStreamHandler jsonHandler,
+                                             boolean update) throws Exception {
         try {
             if (httpClient == null) {
                 //httpClient = OxDbConn.httpClient;
@@ -1157,9 +1171,14 @@ public class Api {
 
 
             //Posting the data
-            String params = "files?action=new&filename=" +
+            String params = "files?action=" + (update ? "update" : "new" ) + "&filename=" +
                              URLEncoder.encode(fileName) +
                              "&session=" + this.session;
+            if (update) {
+                Date date = new Date();
+                params = params + "&timestamp=" + date.getTime();
+            }
+
             HttpPost postRequest =
                 new HttpPost("http" +
                              (Config.backend_ssl ? "s" : "") +
