@@ -70,12 +70,16 @@ import org.bouncycastle.openpgp.PGPSignature;
 import org.bouncycastle.openpgp.PGPUserAttributeSubpacketVector;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
+import com.amazonaws.services.cloudfront_2012_03_15.model.InvalidArgumentException;
 import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.openexchange.guard.database.Access;
-import com.openexchange.guard.database.DbCommand;
-import com.openexchange.guard.database.DbQuery;
 import com.openexchange.guard.database.Keymanager;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
+import com.openexchange.guard.database.pgpKeys.PGPKeys;
+import com.openexchange.guard.database.pgpKeys.PGPKeysStorage;
+import com.openexchange.guard.database.pgpKeys.RestDbPGPKeysStorage;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.exceptions.BadPasswordException;
 import com.openexchange.guard.logging.LogAction;
@@ -90,7 +94,7 @@ import com.openexchange.guard.util.Core;
 public class PGPPrivateKeyHandler {
     private static Logger logger = LoggerFactory.getLogger(PGPPrivateKeyHandler.class);
 
-    public void incomingPGPForm(HttpServletRequest request, HttpServletResponse response, OxCookie ck) throws Exception {
+    public void incomingPGPForm(HttpServletRequest request, HttpServletResponse response, OxCookie cookie) throws Exception {
         FileItemFactory fif = new DiskFileItemFactory();
         ServletFileUpload sfu = new ServletFileUpload(fif);
         int userid = Core.getIntParameter(request, "userid", true);
@@ -104,6 +108,15 @@ public class PGPPrivateKeyHandler {
         String pubdata = "";
         String pgppassword = "";
         String password = null;
+
+        //Check authentication
+        Api ap = new Api(cookie, request);
+        if (!ap.verifyLogin()) {
+            LogAction.setFail();
+            logger.info("User not logged into UI");
+            Core.sendFail(response, "Must be logged in to UI");
+            return;
+        }
         while (iterator.hasNext()) {
             FileItem fi = (FileItem) iterator.next();
             String name = fi.getFieldName();
@@ -184,7 +197,7 @@ public class PGPPrivateKeyHandler {
             }
         }
         try {
-            Api api = new Api(ck,request);
+            Api api = new Api(cookie,request);
             newkey.email = api.getPrimary();
             newkey.contextid = api.getContextId();
             newkey.userid = userid;
@@ -203,7 +216,7 @@ public class PGPPrivateKeyHandler {
 
                 // Create recovery
                 if (password != null) {
-                    if (Keymanager.checkCreateRecovery(ck, request)) {
+                    if (Keymanager.checkCreateRecovery(cookie, request)) {
                         newkey.recovery = Keymanager.createRecovery(newkey, password);
                     }
                 }
@@ -222,10 +235,12 @@ public class PGPPrivateKeyHandler {
                     }
                     if (newkey.getPGPSecretKeyRing() == null) {  // If overwriting public only, and we have a private for that key, keep it and only update PubRing
                         if (existing.getPGPSecretKeyRing() != null) {
-                            acc.updatePubring(newkey);
+                            OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+                            ogKeyTableStorage.updatePublicKey(newkey);
                         }
                     } else {
                         // Update the key
+                        newkey = updateOldGuardKey (newkey, existing, password);
                         acc.updateDuplPGPKey(userid, cid, newkey, recovery, dupl);
                     }
                 }
@@ -306,7 +321,7 @@ public class PGPPrivateKeyHandler {
                 existing.pubring = PGPPublicKeyRing.insertPublicKey(existing.pubring, keyToImport);
             }
         }
-        PgpKeys.addPublicKeyIndex(existing.pubring, existing.email, existing.contextid, true, existing.keyid);
+        PgpKeys.addPublicKeyIndex(existing.pubring, existing.email, existing.contextid, existing.keyid);
         return (existing.pubring);
         } catch (Exception e) {
             logger.error("Problem merging duplicate public keys, ", e);
@@ -314,6 +329,23 @@ public class PGPPrivateKeyHandler {
         }
     }
 
+    /**
+     * Since the salt changes, we need to re-encrypt the old Guard proprietary priv key.
+     * @param newkey
+     * @param existing
+     * @param password
+     * @return
+     */
+    private GuardKeys updateOldGuardKey (GuardKeys newkey, GuardKeys existing, String password) {
+        if (existing.privKeyNull()) return (newkey);
+        try {
+            newkey.setPrivate(existing.getDecodedPrivate(password), password);
+        } catch (Exception ex) {
+            logger.error("Problem copying over old Guard Password ", ex);
+            return (newkey);
+        }
+        return (newkey);
+    }
     /**
      * Delete the private/public key pair for the user
      * @param request
@@ -364,21 +396,18 @@ public class PGPPrivateKeyHandler {
             return;
         }
         boolean iscurrent = key.current;
-        // OK, password and login verified.  Delete the key
-        String command = "DELETE FROM og_KeyTable WHERE cid = ? AND id = ? AND keyid = ?";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(cid);
-        com.addVariables(userid);
-        com.addVariables(keyid);
-        DbQuery db = new DbQuery();
-        db.write(com, userid, cid);
-        db.close();
-        command = "DELETE FROM PGPKeys WHERE keyid = ?";
-        com = new DbCommand(command);
-        com.addVariables(keyid);
-        db = new DbQuery();
-        db.writeOxGuard(com);
-        db.close();
+
+        //TODO: use a service locator for resolvin this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        //OK, password and login verified.  Delete the key
+        ogKeyTableStorage.delete(key);
+
+        //TODO: use a DI-Container to resolve this
+        PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+        //Delete from
+        pgpKeysStorage.deleteByKeyId(keyid);
+
+
         //if key was the current key, then upgrade highest version key to current
         if (iscurrent) {
         	resetCurrentKey(userid, cid);
@@ -396,31 +425,13 @@ public class PGPPrivateKeyHandler {
      * @throws Exception
      */
     private void resetCurrentKey(int userid, int cid) throws Exception {
-        String command;
-        DbCommand com;
-        DbQuery db;
-        command = "SELECT MAX(version) FROM og_KeyTable WHERE cid = ? and id = ?";
-        com = new DbCommand(command);
-        com.addVariables(cid);
-        com.addVariables(userid);
-        db = new DbQuery();
-        db.read(com, userid, cid);
-        if (db.next()) {
-        	if (!db.rs.isNull("MAX(version)")) {
-        		int version = db.rs.getInt("MAX(version)");
-        		db.close();
-        		command = "UPDATE og_KeyTable SET current = 1 WHERE cid = ? and id = ? and version = ?";
-        		com = new DbCommand(command);
-        		com.addVariables(cid);
-        		com.addVariables(userid);
-        		com.addVariables(version);
-        		db = new DbQuery();
-        		db.write(com, userid, cid);
-        		db.close();
-        	}
-        	else {
-                db.close();
-            }
+        //TODO: Using a Service Locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+
+        //Getting the keys with the highest version from the user, and mark it as the "current" key
+        GuardKeys highestKey = ogKeyTableStorage.getHighestVersionKeyForUser(userid, cid);
+        if(highestKey != null) {
+            ogKeyTableStorage.setCurrentKey(highestKey);
         }
     }
 
@@ -471,7 +482,10 @@ public class PGPPrivateKeyHandler {
             return;
         }
         key = PgpKeys.revoke(key, data.encr_password, reason);
-        acc.updatePubring(key);
+
+        OGKeyTableStorage keyTableStorage = new RestDbOGKeyTableStorage();
+        keyTableStorage.updatePublicKey(key);
+
         Core.sendOK(response, "OK");
         logger.info("Success deleting private key");
 
@@ -516,21 +530,16 @@ public class PGPPrivateKeyHandler {
             logger.error("Attempt to delete public when private key exists");
             return;
         }
-        String command = "DELETE FROM og_KeyTable WHERE id = ? AND cid = ? AND keyid = ?";
-        DbCommand com = new DbCommand (command);
-        com.addVariables(userid);
-        com.addVariables(cid);
-        com.addVariables(keyid);
-        DbQuery db = new DbQuery();
-        db.write(com, userid, cid);
-        db.close();
-        command = "DELETE FROM PGPKeys WHERE cid = ? and keyid = ?";
-        com = new DbCommand (command);
-        com.addVariables(cid);
-        com.addVariables(keyid);
-        db = new DbQuery();
-        db.writeOxGuard(com);
-        db.close();
+
+        //TODO: Use a service locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+
+        //Delete the key
+        ogKeyTableStorage.delete(key);
+
+        //delete the key mapping
+        pgpKeysStorage.deleteByKeyIdInContext(keyid, cid);
 
         //if key was the current key, then upgrade highest version key to current:
         if (key.current) {
@@ -644,35 +653,19 @@ public class PGPPrivateKeyHandler {
      * @throws Exception
      */
     private long isDuplicate (GuardKeys gkey) throws Exception {
-       Iterator keys = gkey.pubring.getPublicKeys();
-       PGPPublicKey key = null;
-       StringBuilder search = new StringBuilder();
-       int i = 0;
+       List<Long> ids = new ArrayList<Long>();
+       Iterator<PGPPublicKey> keys = gkey.pubring.getPublicKeys();
        while (keys.hasNext()) {
-           key = (PGPPublicKey) keys.next();
-           Long keyid = key.getKeyID();
-           if (i++ == 0) {
-               search.append( "(" + Long.toString(keyid));
-           } else {
-               search.append(", " + Long.toString(keyid));
-           }
+           ids.add(keys.next().getKeyID());
+       }
 
+       //TODO: Use a DI Container for resolving this
+       PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
+       List<PGPKeys> keyMappings = pgpKeysStorage.getByIds(ids);
+       if(keyMappings.size() > 0) {
+           return keyMappings.get(0).getKeyId();
        }
-       search.append(")");
-       if (search.length() > 5) {
-           String command = "SELECT keyid FROM PGPKeys WHERE id IN " + search.toString();
-           DbCommand com = new DbCommand(command);
-           DbQuery db = new DbQuery();
-           db.readOG(com);
-           if (db.hasResults()) {
-               db.next();
-               Long keyid = db.rs.getLong("keyid");
-               db.close();
-               return(keyid);
-           }
-           db.close();
-       }
-       return (0);
+       return 0;
     }
 
     /**
@@ -683,41 +676,21 @@ public class PGPPrivateKeyHandler {
      * @return
      * @throws Exception
      */
-    public static String makeCurrent (int id, int cid, String keyid) throws Exception {
-        String command = "SELECT version FROM og_KeyTable WHERE id = ? and cid = ? and keyid = ?";
-        DbCommand com = new DbCommand (command);
-        com.addVariables(id);
-        com.addVariables(cid);
-        com.addVariables(keyid);
-        DbQuery db = new DbQuery();
-        db.read(com, id, cid);
-        if (!db.hasResults()) {
-            db.close();
-            return("Key Not Found");
+    public static void makeCurrent (int id, int cid, String keyid) throws Exception {
+        //TODO: use a Service Locator for resolving this
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
 
-        } else {
-            db.close();
-            try {
-                command = "UPDATE og_KeyTable SET current = 0 WHERE id = ? and cid = ?";
-                com = new DbCommand (command);
-                com.addVariables(id);
-                com.addVariables(cid);
-                db = new DbQuery();
-                db.write(com, id, cid);
-                db.close();
-                command = "UPDATE og_KeyTable SET current = 1 WHERE id = ? and cid = ? and keyid = ?";
-                com = new DbCommand (command);
-                com.addVariables(id);
-                com.addVariables(cid);
-                com.addVariables(keyid);
-                db = new DbQuery();
-                db.write(com, id, cid);
-                db.close();
-            } catch (Exception ex) {
-                logger.error("Problem updating current key setting", ex);
-                return("Error " + ex.getLocalizedMessage());
-            }
-            return(null);
+        //Getting the key which should be marked as "current"
+        GuardKeys key = ogKeyTableStorage.getKeyForUserById(id, cid, keyid);
+        if(key != null) {
+            //Removing the current flag from all keys
+            ogKeyTableStorage.unsetCurrentFlag(id, cid);
+
+            //Setting the current flag to the key
+            ogKeyTableStorage.setCurrentKey(key);
+        }
+        else {
+            throw new InvalidArgumentException("Key Not Found");
         }
     }
 
@@ -738,14 +711,7 @@ public class PGPPrivateKeyHandler {
         int cid = Core.getIntParameter(request, "cid", true);
         LogAction.setUser(userid, cid);
         String keyid = Core.getStringParameter(request, "keyid", true);
-        String result = makeCurrent (userid, cid, keyid);
-        if (result == null) {
-            LogAction.setSuccess();
-            Core.sendOK(response, "OK");
-        } else {
-            LogAction.setFail();
-            Core.sendFail(response, result);
-        }
+        makeCurrent (userid, cid, keyid);
     }
 
     public void addUID (HttpServletRequest request, HttpServletResponse response, OxCookie ck) throws Exception {
