@@ -424,7 +424,7 @@ public class PGPUtils {
     		result.verified = true;
     		return(result);
     	} else {
-    		result.verified = true;
+    		result.verified = false;
     		return(result);
     	}
 
@@ -471,8 +471,10 @@ public class PGPUtils {
 	        builder.setSecureRandom(new SecureRandom());
 	        PGPEncryptedDataGenerator encryptedDataGenerator = new PGPEncryptedDataGenerator(builder);
 	        for (int i = 0; i < recipients.size(); i++) {
-	        	BcPublicKeyKeyEncryptionMethodGenerator encKeyGen = new BcPublicKeyKeyEncryptionMethodGenerator(getCryptKey(recipients.get(i).pgpring));
-	        	encryptedDataGenerator.addMethod(encKeyGen);
+	            if (recipients.get(i).pgpring != null) {
+	                BcPublicKeyKeyEncryptionMethodGenerator encKeyGen = new BcPublicKeyKeyEncryptionMethodGenerator(getCryptKey(recipients.get(i).pgpring));
+	                encryptedDataGenerator.addMethod(encKeyGen);
+	            }
 	        }
 
 	        OutputStream encryptedOut = encryptedDataGenerator.open(out, new byte[4028]);
@@ -639,7 +641,8 @@ public class PGPUtils {
             out = new ArmoredOutputStream(out);
         }
 
-        PGPSignatureGenerator       sgen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(key.getPGPPublicKey().getAlgorithm(), PGPUtil.SHA256).setProvider("BC"));
+        int algorithm = key.getPGPSigningKey().getPublicKey().getAlgorithm();
+        PGPSignatureGenerator       sgen = new PGPSignatureGenerator(new JcaPGPContentSignerBuilder(algorithm, PGPUtil.SHA256).setProvider("BC"));
         sgen.init(PGPSignature.BINARY_DOCUMENT, key.getPGPSignKey(password));
 
         BCPGOutputStream bout = new BCPGOutputStream (out);
