@@ -54,29 +54,20 @@ import java.io.ByteArrayOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.io.StringWriter;
-import java.io.UnsupportedEncodingException;
 import java.net.URLEncoder;
-import java.nio.charset.Charset;
-import java.security.NoSuchProviderException;
 import java.security.SignatureException;
 import java.util.ArrayList;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.Properties;
 import javax.mail.Address;
-import javax.mail.Authenticator;
 import javax.mail.BodyPart;
-import javax.mail.Message;
 import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Multipart;
 import javax.mail.Part;
-import javax.mail.PasswordAuthentication;
 import javax.mail.Session;
-import javax.mail.Transport;
 import javax.mail.internet.InternetAddress;
-import javax.mail.internet.MimeBodyPart;
 import javax.mail.internet.MimeMessage;
 import javax.mail.internet.MimeMultipart;
 import javax.mail.internet.MimeUtility;
@@ -84,26 +75,14 @@ import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import org.apache.commons.io.IOUtils;
 import org.apache.commons.net.util.Base64;
-import org.bouncycastle.bcpg.ArmoredOutputStream;
-import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
 import org.bouncycastle.bcpg.attr.ImageAttribute;
 import org.bouncycastle.openpgp.PGPException;
-import org.bouncycastle.openpgp.PGPPrivateKey;
 import org.bouncycastle.openpgp.PGPPublicKey;
 import org.bouncycastle.openpgp.PGPPublicKeyRing;
-import org.bouncycastle.openpgp.PGPPublicKeyRingCollection;
 import org.bouncycastle.openpgp.PGPSecretKey;
 import org.bouncycastle.openpgp.PGPSecretKeyRing;
 import org.bouncycastle.openpgp.PGPSignature;
-import org.bouncycastle.openpgp.PGPSignatureGenerator;
-import org.bouncycastle.openpgp.PGPSignatureSubpacketGenerator;
-import org.bouncycastle.openpgp.PGPSignatureSubpacketVector;
 import org.bouncycastle.openpgp.PGPUserAttributeSubpacketVector;
-import org.bouncycastle.openpgp.PGPUtil;
-import org.bouncycastle.openpgp.operator.bc.BcPGPContentSignerBuilder;
-import org.bouncycastle.openpgp.operator.jcajce.JcaKeyFingerprintCalculator;
-import org.bouncycastle.openpgp.operator.jcajce.JcaPGPContentSignerBuilder;
-import org.bouncycastle.openpgp.operator.jcajce.JcePBESecretKeyDecryptorBuilder;
 import org.eclipse.jetty.server.Response;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -112,16 +91,16 @@ import com.google.gson.JsonObject;
 import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.Access;
-import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
-import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.exceptions.BadPasswordException;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.logging.LogAction;
+import com.openexchange.guard.mail.MimeParser;
+import com.openexchange.guard.mail.SignMime;
 import com.openexchange.guard.mailcreator.Attachment;
-import com.openexchange.guard.mailcreator.PGPMailCreator;
 import com.openexchange.guard.ox.Api;
 import com.openexchange.guard.server.CheckBad;
 import com.openexchange.guard.server.OxCookie;
@@ -133,7 +112,7 @@ import com.openexchange.guard.validator.Normalizer;
 public class PgpHandler {
 
 	private static Logger logger = LoggerFactory.getLogger(PgpHandler.class);
-	
+
 
 	/**
 	 * Decode PGP Inline message
@@ -162,8 +141,9 @@ public class PgpHandler {
 	        			com.openexchange.guard.util.Core.sendFail(response, "Bad authorization");
 	                    CheckBad.addBad(cookie.JSESSIONID);
 	                    return;
-	        		} else
-	        			password = data.encr_password;
+	        		} else {
+                        password = data.encr_password;
+                    }
      		}
      	}
      	if (password == null) {
@@ -208,9 +188,9 @@ public class PgpHandler {
             	Core.sendFail(response, decoded.error);
             	return;
             }
-            results.addProperty("newcontent", Normalizer.normalizeString(checkHTML(new String (decoded.decoded, "UTF-8"))));
+            results.addProperty("newcontent", Normalizer.normalizeString(new String (decoded.decoded, "UTF-8")));
             JsonObject pgpresults = new JsonObject();
-            
+
             pgpresults.add("signature", new JsonPrimitive(decoded.signature));
             pgpresults.add("verified", new JsonPrimitive(decoded.verified));
             pgpresults.add("missingPublicKey", new JsonPrimitive(decoded.missingPublicKey));
@@ -225,15 +205,9 @@ public class PgpHandler {
             Core.sendFail(response, "Bad Password");
             return;
         }
-        
+
 	 }
-	 
-	private String checkHTML(String data) {
-		if (data.contains("<div") || data.contains("<br")) return(data);
-		data = data.replace("\n", "<br>");
-		return (data);
-	}
-	
+
 	/**
 	 * Verify the signature
 	 * inline implies search for inline signature
@@ -247,7 +221,7 @@ public class PgpHandler {
         int userid = Integer.parseInt(request.getParameter("userid"));
         int cid = Integer.parseInt(request.getParameter("cid"));
         LogAction.setUser(userid, cid);
-        String emailid = request.getParameter("emailid"); 
+        String emailid = request.getParameter("emailid");
         int index = Core.getIntParameter(request, "attach", true);
         String inline = Core.getStringParameter(request, "inline");
         com.openexchange.guard.ox.Api ap = new com.openexchange.guard.ox.Api(cookie, request);
@@ -257,10 +231,10 @@ public class PgpHandler {
         MimeMessage message = new MimeMessage(s, in);
         message.saveChanges();
 
-        
+
         String signature = "";
         String content = "";
-        
+
         if (inline != null) {
             Object body = message.getContent();
         	PGPResult result = new PGPResult();
@@ -303,7 +277,7 @@ public class PgpHandler {
         			    section = section.substring(1);
         			}
         		}
-        		
+
         	} else {
         		result.signature = false;
         		LogAction.setSuccess();
@@ -326,9 +300,14 @@ public class PgpHandler {
 	        	mp.getBodyPart(i).writeTo(writer);
 	        	String section = writer.toString("UTF-8");
 	        	if (section.contains("application/pgp-signature")) {
-	        		signature = section;
+	        	    if (section.contains("Encoding: base64")) {
+	        	        signature = Core.decodeBase64(section);
+	        	    } else {
+	        	        signature = section;
+	        	    }
+
 	        	} else {
-	        		content = section;
+	        		content = SignMime.normalize(MimeParser.getMimeSection(mime, MimeParser.getBoundry(message)));
 	        	}
 	        	writer.close();
 	        }
@@ -341,8 +320,8 @@ public class PgpHandler {
         logger.info("Signature verification, signature verification: " + Boolean.toString(result.verified));
         Core.sendOK(response, resultJson.toString());
 	}
-	
-	
+
+
 	/**
 	 * Get an inline PGP attachment (essentially file decode)
 	 * @param request
@@ -368,7 +347,9 @@ public class PgpHandler {
         }
         boolean download = false;
         if (downloadStr != null) {
-        	if (downloadStr.toLowerCase().trim().equals("true")) download = true;
+        	if (downloadStr.toLowerCase().trim().equals("true")) {
+                download = true;
+            }
         }
         if (password == null) {
         	LogAction.setFail();
@@ -377,14 +358,15 @@ public class PgpHandler {
         	CheckBad.addBad(emailid + "a:" + attach);
         	return;
         }
-        
-        if (download)
-        	response.setHeader("Content-Disposition", "attachment; filename=\"" + filename + "\"");
+
+        if (download) {
+            response.setHeader("Content-Disposition", "attachment; filename=\"" + filename + "\"");
+        }
         decodeInlineAttach(emailid, folder, attach, filename, userid, cid, password, response, request, cookie);
         LogAction.setSuccess(true);
- 
+
 	}
-	
+
 	/**
 	 * Do the actual decode to stream of the inline file.  Retrieve from OX backend and send decoded
 	 * @param emailid
@@ -416,7 +398,9 @@ public class PgpHandler {
             resp.addHeader("Access-Control-Allow-Origin", "*");
             resp.setStatus(HttpServletResponse.SC_OK);
             out = ((HttpServletResponse) response).getOutputStream();
-        } else out = (OutputStream) response;
+        } else {
+            out = (OutputStream) response;
+        }
         try {
             PGPResult result = PGPUtils.decryptFile(in, out, userid, cid, password, true);
             if (result.error != null) {
@@ -442,14 +426,14 @@ public class PgpHandler {
         out.close();
         logger.info("Attachment retrieved " + filename);
 	}
-	
+
 	public byte[] decodeInlineAttach (String emailid, String folder, String attach, String filename, int userid, int cid, String password, HttpServletRequest request, OxCookie ck) throws Exception {
 	    ByteArrayOutputStream out = new ByteArrayOutputStream();
 	    decodeInlineAttach(emailid, folder, attach, filename, userid, cid, password, out, request, ck);
 	    out.close();
 	    return(out.toByteArray());
 	}
-	
+
 	public void getAttach (HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
 		String name = "";
 		try {
@@ -462,7 +446,7 @@ public class PgpHandler {
 		}
 		DecodePGPemail (request, response, cookie, "open", name);
 	}
-	
+
 	/**
 	 * Get an attachment from PGP Mime email for frowarding or from draft
 	 * @param out
@@ -482,7 +466,7 @@ public class PgpHandler {
 	    out.close();
 	    return(out.toByteArray());
 	}
-	
+
 	public void getAttachCID (HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
         String contentid = "";
         try {
@@ -495,11 +479,11 @@ public class PgpHandler {
         }
         DecodePGPemail (request, response, cookie, "cid", contentid);
     }
-	
+
 	   public void getPGPemail(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) {
 	        DecodePGPemail (request, response, cookie, "mail", null);
 	    }
-	   
+
 	/**
 	 * Decode PGP Mime email
 	 * @param request
@@ -510,7 +494,15 @@ public class PgpHandler {
 	 */
     public void DecodePGPemail(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, String action, String name) {
         try {
-        	JsonObject jdata = Core.getJSON(request);
+            JsonObject jdata;
+            try {
+                jdata = Core.getJSON(request);
+            } catch (BadRSAException e) {
+                LogAction.setFail();
+                logger.error("Bad RSA encryption of password", e);
+                Core.sendFail(response, "bad rsa");
+                return;
+            }
             int userid = Integer.parseInt(request.getParameter("userid"));
             int cid = Integer.parseInt(request.getParameter("cid"));
             LogAction.setUser(userid, cid);
@@ -531,8 +523,9 @@ public class PgpHandler {
 	                    LogAction.setFail();
 	                    logger.info("Bad authorization");
 	                    return;
-	        		} else
-	        			password = data.encr_password;
+	        		} else {
+                        password = data.encr_password;
+                    }
         		}
         	}
         	if (password == null) {
@@ -547,9 +540,9 @@ public class PgpHandler {
         		logger.info("No Password");
         		return;
         	}
-        	String folder = request.getParameter("folder"); 
+        	String folder = request.getParameter("folder");
             String emailid = request.getParameter("emailid");
-            
+
             doDecode(emailid, folder, userid, cid, action, name, password, request, cookie, response);
 
         } catch (Exception ex) {
@@ -558,7 +551,7 @@ public class PgpHandler {
         }
 
     }
-    
+
     private void sendFail (Object response, String answer) throws IOException {
         if (response.getClass().equals(Response.class)) {
             Core.sendFail((Response) response, answer);
@@ -566,7 +559,7 @@ public class PgpHandler {
             ((OutputStream) response).write(answer.getBytes());
         }
     }
-    
+
     private void sendOK (Object response, String answer) throws IOException {
         if (response.getClass().equals(Response.class)) {
             Core.sendOK((Response) response, answer);
@@ -574,7 +567,7 @@ public class PgpHandler {
             ((OutputStream) response).write(answer.getBytes());
         }
     }
-    
+
     /**
      * Do the actual decode of PGP Mime email, send to response object
      * @param emailid
@@ -617,11 +610,13 @@ public class PgpHandler {
             LogAction.setFail();
             logger.error("Problem getting PGP email components");
             if (response.getClass().equals(Response.class)) {
-                Core.sendFail((HttpServletResponse) response, "Not properly formatted PGP email");
-            } else 
-            return;
+                Core.sendFail((HttpServletResponse) response, "Not properly formatted PGP email or missing attachment");
+                return;
+            } else {
+                return;
+            }
         }
-        
+
 
         PGPStringProcessor pgp = new PGPStringProcessor();
         Access acc = new Access();
@@ -631,11 +626,6 @@ public class PgpHandler {
         if (fromemail.contains("<")) {
             fromemail = fromemail.substring(fromemail.indexOf("<") + 1, fromemail.indexOf(">"));
         }
-        ArrayList<PGPPublicKeyRing> senderRing = acc.getPublicKeyRing(fromemail, userid, cid);
-        PGPPublicKeyRing sender = senderRing.size() == 0 ? null : senderRing.get(0);
-        if (sender == null) {
-            logger.info("No public key for " + fromemail + " found");
-        }
         InputStream inp = p.getInputStream();
         PGPResult result = null;
         try {
@@ -653,13 +643,13 @@ public class PgpHandler {
             sendFail(response, result.error);
         } else {
             switch (action) {
-            case "mail": 
+            case "mail":
                 LogAction.setSuccess(true);
                 logger.info("Decoded PGP Mime email");
                 sendOK(response, createReturnJson(result, password, userid, cid, cookie));
                 break;
             case "open":
-                boolean download = (Core.getStringParameter(request, "download") == null ? false : (Core.getStringParameter(request, "download").toLowerCase().equals("true"))); 
+                boolean download = (Core.getStringParameter(request, "download") == null ? false : (Core.getStringParameter(request, "download").toLowerCase().equals("true")));
                 getAttach (response, result, name, download);
                 break;
             case "cid":
@@ -667,7 +657,7 @@ public class PgpHandler {
             }
         }
     }
-    
+
 
     /**
      * Search Mulipart message for PGP MIME
@@ -682,20 +672,24 @@ public class PgpHandler {
     	for (int i = 0; i < mp.getCount(); i++) {
     		Object content = mp.getBodyPart(i).getContent();
     		String type = mp.getBodyPart(i).getContentType();
-    		if (type.contains("pgp-encrypted")) pgp = true;
+    		if (type.contains("pgp-encrypted")) {
+                pgp = true;
+            }
     		if (type.contains("application/octet-stream") && pgp) {
     			return (mp.getBodyPart(i));
     		}
     		if (content instanceof Multipart || content instanceof MimeMultipart) {
     			p = getPGPPart ((Multipart) content);
     		}
-    		if (p!= null) return (p);
-    		
+    		if (p!= null) {
+                return (p);
+            }
+
     	}
     	return (null);
-        
+
     }
-    
+
     private Part getGuestPGPPart(Multipart mp) throws MessagingException, IOException {
         Part p = null;
         for (int i = 0; i < mp.getCount(); i++) {
@@ -707,11 +701,13 @@ public class PgpHandler {
             if (content instanceof Multipart || content instanceof MimeMultipart) {
                 p = getGuestPGPPart ((Multipart) content);
             }
-            if (p!= null) return (p);
+            if (p!= null) {
+                return (p);
+            }
         }
         return(null);
     }
-    
+
     /**
      * Save vcard attachment into contacts
      * @param request
@@ -737,7 +733,7 @@ public class PgpHandler {
         }
         byte[] vcard = null;
         if (inline) {
-            vcard = decodeInlineAttach (emailid, folder, attachid, filename, userid, cid, password, request, ck);  
+            vcard = decodeInlineAttach (emailid, folder, attachid, filename, userid, cid, password, request, ck);
         } else {
             vcard = getAttach (request, ck, emailid, folder, userid, cid, filename, password);
         }
@@ -751,8 +747,13 @@ public class PgpHandler {
                 vc.type = "text/vcard";
                 JsonObject resp = ap.importVcard(vc, contactFolder);
                 if (Core.checkFail(resp)) {
-                    Core.sendFail(response, "unable to import");
-                    return;
+                    if (resp.get("error").getAsString().contains("Warnings")) {
+                        Core.sendOK(response, "Warn");
+                        return;
+                    } else {
+                        Core.sendFail(response, "unable to import");
+                        return;
+                    }
                 }
                 if (resp.has("data")) {
                     try {
@@ -770,10 +771,10 @@ public class PgpHandler {
             }
 
         }
-        
-        
+
+
     }
-    
+
     /**
      * Get an attachment from PGP Mime email and return to stream
      * @param response
@@ -819,7 +820,7 @@ public class PgpHandler {
                     } else {
                         IOUtils.copy(p.getInputStream(), (OutputStream) response);
                     }
-            		
+
             		LogAction.setSuccess(true);
             		logger.info("Sending decoded attachment " + filename);
             		break;
@@ -828,7 +829,7 @@ public class PgpHandler {
         }
         in2.close();
     }
-    
+
     /**
      * Get an attachment from PGP Mime email and return to stream
      * @param response
@@ -848,7 +849,7 @@ public class PgpHandler {
             String [] cids = p.getHeader("Content-ID");
             String cid = null;
             if (cids != null) {
-                cid = cids[0].replace("<", "").replace(">", ""); 
+                cid = cids[0].replace("<", "").replace(">", "");
                 if (cid.toLowerCase().trim().equals(match_cid.toLowerCase().trim())) {
                     String type = p.getContentType();
                     if (type!=null) {
@@ -868,7 +869,7 @@ public class PgpHandler {
         }
         in2.close();
     }
-    
+
     public static void ParseMultipart (Multipart mp, JsonArray attachments) throws MessagingException, IOException {
     	 JsonObject plain = null;
     	 boolean html = false;
@@ -881,38 +882,50 @@ public class PgpHandler {
                  if (contentType.startsWith("multip")) {
                 	 ParseMultipart ((Multipart) p.getContent(), attachments);
                  } else {
-	                 if (contentType.contains(";"))
-	                     contentType = contentType.substring(0, contentType.indexOf(";"));
+	                 if (contentType.contains(";")) {
+                        contentType = contentType.substring(0, contentType.indexOf(";"));
+                    }
 	                 attachment.add("content_type", new JsonPrimitive(contentType));
 	                 if (contentType.contains("text/html") || contentType.contains("text/plain")) {
 	                 	attachment.add("content", new JsonPrimitive((String) p.getContent()));
 	                     if (p.getFileName() == null) {
 	                         attachment.add("disp", new JsonPrimitive("inline"));
-	                     } else isAttachment = true;
+	                     } else {
+                            isAttachment = true;
+                        }
 	                 } else {
 	                 	attachment.addProperty("content", "");
 	                 	}
-	
+
 	             }
-	             if (p.getFileName() != null)
-	                 attachment.add("filename", new JsonPrimitive(p.getFileName()));
-	             if (p.getDisposition() != null)
-	                 attachment.add("disp", new JsonPrimitive(p.getDisposition()));
+	             if (p.getFileName() != null) {
+                    attachment.add("filename", new JsonPrimitive(p.getFileName()));
+                }
+	             if (p.getDisposition() != null) {
+                    attachment.add("disp", new JsonPrimitive(p.getDisposition()));
+                }
 	             attachment.add("size", new JsonPrimitive(p.getSize()));
-	             
+
 	             attachment.add("id", new JsonPrimitive(i));
-	             if (contentType.contains("text/html")) html = true;
+	             if (contentType.contains("text/html")) {
+                    html = true;
+                }
 	             if (contentType.contains("text/plain")) {
 	                 if (isAttachment) {  // If attachment, then just append
 	                     attachments.add(attachment);
-	                 } else  // otherwise, if plaintext inline, then save in case we don't have HTML
-	                     plain = attachment;
-	             } else 
-	            	 if (!contentType.startsWith("multip")) attachments.add(attachment);
+	                 } else {
+                        plain = attachment;
+                    }
+	             } else
+	            	 if (!contentType.startsWith("multip")) {
+                        attachments.add(attachment);
+                    }
              }
          }
     	 if (!html) {  // If no HTML, then add the plain
-    		 if (plain != null) attachments.add(plain);
+    		 if (plain != null) {
+                attachments.add(plain);
+            }
     	 }
     }
 
@@ -925,7 +938,7 @@ public class PgpHandler {
      * @param password
      * @param cookie
      * @return
-     * @throws Exception 
+     * @throws Exception
      */
     public static String createReturnJson(PGPResult result, String password, int userid, int cid, com.openexchange.guard.server.OxCookie cookie) throws Exception {
         Session s = Session.getDefaultInstance(new Properties());
@@ -952,15 +965,18 @@ public class PgpHandler {
         if (password != null) {
         	Access acc = new Access();
             String token = acc.getToken(cookie.JSESSIONID);
-            if (token == null) token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            if (token == null) {
+                token = acc.newToken(cookie.JSESSIONID, userid, cid);
+            }
         	EncrLib encr = new EncrLib();
         	Results.addProperty("epassword", encr.encryptAES(password, token));
         }
-        if (result.error != null)
+        if (result.error != null) {
             Results.add("error", new JsonPrimitive(result.error));
+        }
         JsonObject data = new JsonObject();
         data.add("attachments", attachments);
-        
+
         Address [] toaddrs = message2.getRecipients(RecipientType.TO);
         if (toaddrs != null) {
             JsonArray to = new JsonArray();
@@ -975,7 +991,7 @@ public class PgpHandler {
                 to.add(recip);
             }
             data.add("to", to);
-            
+
         }
         Address [] ccaddrs = message2.getRecipients(RecipientType.CC);
         if (ccaddrs != null) {
@@ -990,7 +1006,7 @@ public class PgpHandler {
                 cc.add(recip);
             }
             data.add("cc", cc);
-            
+
         }
         data.add("results", Results);
         in2.close();
@@ -1002,7 +1018,7 @@ public class PgpHandler {
      * Display Armor output of PGP Key Ring from OX user
      * @param request
      * @param response
-     * @throws GuardMissingParameter 
+     * @throws GuardMissingParameter
      */
     public void getKey(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
         String email = Core.getStringParameter(request, "email", true);
@@ -1037,7 +1053,7 @@ public class PgpHandler {
         }
 
     }
-    
+
     /**
      * Export private key by ID
      * @param request
@@ -1065,7 +1081,7 @@ public class PgpHandler {
     	            return;
     	        }
     	}
-    	
+
 
     	if (guest != null) {
     		cookie.JSESSIONID = cookie.OxReaderID;
@@ -1080,8 +1096,9 @@ public class PgpHandler {
 	                    LogAction.setFail();
 	                    logger.info("Bad authorization");
 	                    return;
-	        		} else
-	        			password = data.encr_password;
+	        		} else {
+                        password = data.encr_password;
+                    }
      		}
      	}
      	Access acc = new Access();
@@ -1090,15 +1107,15 @@ public class PgpHandler {
      	if (keyid != 0) {
      	    key = acc.getKeys(id, cid, keyid, false);
      	} else {
-     	    key = acc.getKeys(id, cid);  // If keyid isn't specified, then just get the current  
+     	    key = acc.getKeys(id, cid);  // If keyid isn't specified, then just get the current
      	}
-     	 
+
      	if (key == null) {
      		LogAction.setFail();
      		logger.error("Unable to get private key");
      		Core.sendFail(response, "Unable to get key");
      		return;
-     		
+
      	}
      	String exported = PgpKeys.ExportPGPPrivate(key, password);
      	if (exported == null) {
@@ -1127,8 +1144,9 @@ public class PgpHandler {
     	 String keyid = Core.getStringParameter(request, "keyid");
     	 Long kid = (long) 0;
     	 try {
-    	     if (keyid != null)
-    	         kid = Long.parseLong(keyid);
+    	     if (keyid != null) {
+                kid = Long.parseLong(keyid);
+            }
     	 } catch (Exception ex) {
     		 LogAction.setFail();
     	     logger.error("problem converting keyid " , ex);
@@ -1140,8 +1158,9 @@ public class PgpHandler {
              GuardKeys keys;
              if (kid != 0) {
                  keys = acc.getKeys(id, cid, kid, false);
-             } else 
-                 keys = acc.getKeys(id, cid);
+             } else {
+                keys = acc.getKeys(id, cid);
+            }
              if (keys != null) {
             	 String filename = "public.asc";  // Default filename
          		 Iterator it = keys.pubring.getPublicKey().getUserIDs(); // Change filename to first ID if avail
@@ -1158,7 +1177,7 @@ public class PgpHandler {
              Core.sendFail(response, "Error processing request");
          }
     }
-    
+
     public void getUserKeys (HttpServletRequest request, HttpServletResponse response, OxCookie ck) throws Exception {
         int id = Core.getIntParameter(request, "userid", true);
         int cid = Core.getIntParameter(request, "cid", true);
@@ -1174,21 +1193,25 @@ public class PgpHandler {
         Core.sendOK(response, PgpKeys.getAllKeys(id, cid).toString());
         return;
     }
-    
+
     /**
      * Return JsonArray of info for PGPSecretKeyRing
      * @param skeyring
      * @return
      */
     public static JsonArray getPrivateInfo (PGPSecretKeyRing skeyring) {
-        if (skeyring == null) return (null);
+        if (skeyring == null) {
+            return (null);
+        }
         Iterator skeys = skeyring.getSecretKeys();
         JsonArray keyRingData = new JsonArray();
         String self = "";
         while (skeys.hasNext()) {
             PGPSecretKey key = (PGPSecretKey) skeys.next();
             JsonObject pubKey = new JsonObject();
-            if (key.isMasterKey()) self = toHex(key.getKeyID());
+            if (key.isMasterKey()) {
+                self = toHex(key.getKeyID());
+            }
             pubKey.addProperty("Key", toHex(key.getKeyID()));
             pubKey.addProperty("KeyID", toFullHex(key.getKeyID()));
             pubKey.addProperty("Master", key.isMasterKey());
@@ -1203,7 +1226,7 @@ public class PgpHandler {
         }
         return (keyRingData);
     }
-    
+
     public static JsonArray getKeyInfo (ArrayList<PGPPublicKeyRing> pkeys) throws SignatureException, PGPException {
         JsonArray data = new JsonArray();
         for (int i = 0; i < pkeys.size(); i++) {
@@ -1211,7 +1234,7 @@ public class PgpHandler {
         }
         return (data);
     }
-    
+
     public void checkSignatures (HttpServletRequest request, HttpServletResponse response, OxCookie ck) throws Exception {
         int userid = Core.getIntParameter(request, "userid", true);
         int cid = Core.getIntParameter(request, "cid", true);
@@ -1242,9 +1265,9 @@ public class PgpHandler {
         ArrayList <SignatureResults> results = PgpKeys.checkSignatures(pubring.getPublicKey(keyid), userid, cid);
         LogAction.setSuccess();
         Core.sendOK(response, getJson(results).toString());
-        
+
     }
-    
+
     private JsonArray getJson (ArrayList<SignatureResults> sigs) {
         JsonArray results = new JsonArray();
         for (int i = 0; i < sigs.size(); i++) {
@@ -1261,7 +1284,7 @@ public class PgpHandler {
         }
         return (results);
     }
-    
+
     /**
      * Print the user info for a PGP public key ring.  Returns as JSON
      * @param pkey
@@ -1277,7 +1300,9 @@ public class PgpHandler {
         while (keys.hasNext()) {
         	key = (PGPPublicKey) keys.next();
         	JsonObject pubKey = new JsonObject();
-        	if (key.isMasterKey()) self = toHex(key.getKeyID());
+        	if (key.isMasterKey()) {
+                self = toHex(key.getKeyID());
+            }
         	pubKey.addProperty("Key", toHex(key.getKeyID()));
         	pubKey.addProperty("KeyID", toFullHex(key.getKeyID()));
         	pubKey.addProperty("KeyLong", Long.toString(key.getKeyID()));
@@ -1324,7 +1349,7 @@ public class PgpHandler {
         	        imagecount++;
         	        images.add(imageJson);
         	    }
-        	    
+
         	}
         	pubKey.add("images", images);
         	it = key.getSignatures();
@@ -1339,9 +1364,9 @@ public class PgpHandler {
         	keyRingData.add(pubKey);
         }
         return (keyRingData);
-    	
+
     }
-    
+
     public void getKeyImage (HttpServletRequest request, HttpServletResponse response, OxCookie ck) throws Exception {
         int userid = Core.getIntParameter(request, "userid", true);
         int cid = Core.getIntParameter(request, "cid", true);
@@ -1373,15 +1398,17 @@ public class PgpHandler {
                                 Core.sendOK(response, "application", image.getImageData());
                             }
                             return;
-                        } else count++;
+                        } else {
+                            count++;
+                        }
                     }
                 }
             }
         }
-        
+
     }
-   
-    
+
+
  /**
   * Convert byte array fingerprint into standard String format with spacing
   * @param fingerprint
@@ -1399,20 +1426,23 @@ public class PgpHandler {
                 hex = hex.substring(hex.length() - 2);
             }
             fpstring.append(hex);
-            if (i % 2 == 1) fpstring.append(" ");
+            if (i % 2 == 1) {
+                fpstring.append(" ");
+            }
             }
             return fpstring.toString();
     }
-    
+
     /**
      * Convert PGP long key ID into standard HEX
      * @param num
      * @return
      */
     private static String toHex(Long num) {
+        if (Long.toHexString(num).length() < 8) return (Long.toHexString(num));
     	return Long.toHexString(num).substring(8);
     }
-    
+
     /**
      * Get full hex ID of PGP long id
      * @param num
