@@ -66,8 +66,6 @@ import java.util.Iterator;
 import java.util.List;
 import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.bouncycastle.bcpg.HashAlgorithmTags;
-import org.bouncycastle.bcpg.PublicKeyPacket;
-import org.bouncycastle.bcpg.SignatureSubpacketTags;
 import org.bouncycastle.bcpg.SymmetricKeyAlgorithmTags;
 import org.bouncycastle.bcpg.sig.KeyFlags;
 import org.bouncycastle.bcpg.sig.RevocationReasonTags;
@@ -90,7 +88,6 @@ import org.bouncycastle.openpgp.PGPSignatureSubpacketGenerator;
 import org.bouncycastle.openpgp.PGPSignatureSubpacketVector;
 import org.bouncycastle.openpgp.PGPUserAttributeSubpacketVector;
 import org.bouncycastle.openpgp.PGPUtil;
-import org.bouncycastle.openpgp.operator.KeyFingerPrintCalculator;
 import org.bouncycastle.openpgp.operator.PBESecretKeyDecryptor;
 import org.bouncycastle.openpgp.operator.PBESecretKeyEncryptor;
 import org.bouncycastle.openpgp.operator.PGPDigestCalculator;
@@ -111,17 +108,18 @@ import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.Access;
-import com.openexchange.guard.database.DbCommand;
-import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
+import com.openexchange.guard.database.pgpKeys.PGPKeysStorage;
+import com.openexchange.guard.database.pgpKeys.RestDbPGPKeysStorage;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.exceptions.BadPasswordException;
-import com.openexchange.guard.translate.GetText;
 import com.openexchange.guard.util.Core;
 
 /**
  * Class to create PGPKeys
- * 
+ *
  * @author greg
  */
 public class PgpKeys {
@@ -145,7 +143,7 @@ public class PgpKeys {
 
     /**
      * Generate oxGuard KeyGroup
-     * 
+     *
      * @param id ID of the user
      * @param password for encryption
      * @return
@@ -173,7 +171,7 @@ public class PgpKeys {
             new int[] {
                 HashAlgorithmTags.SHA256, HashAlgorithmTags.SHA1, HashAlgorithmTags.SHA384, HashAlgorithmTags.SHA512,
                 HashAlgorithmTags.SHA224 });
-        
+
         // Now for subKey (encrypting)
         PGPSignatureSubpacketGenerator enchashgen = new PGPSignatureSubpacketGenerator();
 
@@ -187,13 +185,13 @@ public class PgpKeys {
             new int[] {
                 HashAlgorithmTags.SHA256, HashAlgorithmTags.SHA1, HashAlgorithmTags.SHA384, HashAlgorithmTags.SHA512,
                 HashAlgorithmTags.SHA224 });
-        
+
         if (Config.key_valid_days != 0) {  // If expiration date is chosen, set now
             int seconds = Config.key_valid_days * 24 * 60 * 60;
             signhashgenGenerator.setKeyExpirationTime(false, seconds);
             enchashgen.setKeyExpirationTime(false, seconds);
         }
-        
+
 
         PGPDigestCalculator sha1Calc = new BcPGPDigestCalculatorProvider().get(HashAlgorithmTags.SHA1);
 
@@ -244,9 +242,10 @@ public class PgpKeys {
     public GuardKeys changePassword(String oldpass, String newpass, GuardKeys oldKey) throws NoSuchProviderException, PGPException, BadPasswordException, UnsupportedEncodingException {
         if (!oldKey.privKeyNull()) {
             PrivateKey priv = oldKey.getDecodedPrivate(oldpass);
-    
-            if (priv == null)
+
+            if (priv == null) {
                 return (null);
+            }
             oldKey.setPrivate(priv, newpass);
         }
         oldpass = EncrLib.getSHA(oldpass, oldKey.getSalt());
@@ -260,8 +259,9 @@ public class PgpKeys {
                     keyring,
                     oldencr,
                     newencr);
-            if (newkeyring == null)
+            if (newkeyring == null) {
                 return (null);
+            }
             oldKey.setPGPSecretKeyRing(newkeyring);
             return (oldKey);
         } catch (Exception ex) {
@@ -274,9 +274,10 @@ public class PgpKeys {
 
         if (!oldKey.privKeyNull()) {
             PrivateKey priv = oldKey.getDecodedPrivateWithRecovery(oldpassHash);
-    
-            if (priv == null)
+
+            if (priv == null) {
                 return (null);
+            }
             oldKey.setPrivate(priv, newpass);
         }
         newpass = EncrLib.getSHA(newpass, oldKey.getSalt());
@@ -289,8 +290,9 @@ public class PgpKeys {
                 keyring,
                 oldencr,
                 newencr);
-        if (newkeyring == null)
+        if (newkeyring == null) {
             return (null);
+        }
         oldKey.setPGPSecretKeyRing(newkeyring);
         return (oldKey);
     }
@@ -312,47 +314,38 @@ public class PgpKeys {
     private static int getHashAlgorithmTags() {
         return (HashAlgorithmTags.SHA256);
     }
-    
-    
+
+
     public static void addPublicKeyIndex (GuardKeys key) throws Exception {
-    	addPublicKeyIndex (key.pubring, key.email, key.contextid, true, 0l);
-    }
-    
-    public static void addPublicKeyIndex (PGPPublicKeyRing pubring, String email, int cid, boolean local) throws Exception {
-        addPublicKeyIndex (pubring, email, cid, local, 0l);
+    	addPublicKeyIndex (key.pubring, key.email, key.contextid, 0l);
     }
+
 /**
  * Add the key to the Public Key table index
  * @param key
  * @throws Exception
  */
-    public static void addPublicKeyIndex (PGPPublicKeyRing pubring, String email, int cid, boolean local, long keyid) throws Exception {
+    public static void addPublicKeyIndex (PGPPublicKeyRing pubring, String email, int cid, long keyid) throws Exception {
+
+        PGPKeysStorage pgpKeysStorage = new RestDbPGPKeysStorage();
 		Iterator<PGPPublicKey> it = pubring.getPublicKeys();
-		String command = "INSERT INTO PGPKeys (id, email, hexid, cid, local, keyid) VALUES (?, ?, ?, ?, ?, ?) ON DUPLICATE KEY UPDATE cid = ?";
 		while (it.hasNext()) {
 			PGPPublicKey pubkey = it.next();
-			if (keyid == 0) keyid = pubkey.getKeyID();
-			DbCommand com2 = new DbCommand(command);
-			com2.addVariables(pubkey.getKeyID());
-			com2.addVariables(email);
-			String hex = Core.PGPKeyId(pubkey.getKeyID());
-			com2.addVariables(hex);
-			com2.addVariables(cid);
-			if (local) {
-				com2.addVariables(1);
-			} else com2.addVariables(0);
-			com2.addVariables(keyid);
-			com2.addVariables(cid);
-			DbQuery db2 = new DbQuery();
-			db2.writeOxGuard(com2);
-			db2.close();
+			if(keyid == 0) {
+			    keyid = pubkey.getKeyID();
+			}
+			pgpKeysStorage.insertOrUpdate(pubkey.getKeyID(),
+			                              email,
+			                              Core.PGPKeyId(pubkey.getKeyID()),
+			                              cid,
+			                              keyid);
 		}
     }
-    
+
     public static PGPPrivateKey decodePrivate (PGPSecretKey secretKey, String password, String salt) throws PGPException {
     	return(secretKey.extractPrivateKey(new BcPBESecretKeyDecryptorBuilder(new BcPGPDigestCalculatorProvider()).build(EncrLib.getSHA(password, salt).toCharArray())));
     }
-    
+
     public static String ExportPGPPrivate (GuardKeys keys, String password) throws PGPException, BadPasswordException, IOException {
     	PGPDigestCalculator sha256Calc = new BcPGPDigestCalculatorProvider().get(getHashAlgorithmTags());
     	String oldpass = EncrLib.getSHA(password, keys.getSalt());
@@ -372,7 +365,7 @@ public class PgpKeys {
         out.close();
         return (new String(out.toByteArray()));
     }
-    
+
     public static GuardKeys ImportPGPPrivate (GuardKeys keys, String PGPPrivateASC, String pgpPassword, String guardPassword) throws UnsupportedEncodingException, IOException, BadPasswordException {
     	// PGPUtil.getDecoderStream() will detect ASCII-armor automatically and decode it,
         // the PGPObject factory then knows how to read all the data in the encoded stream
@@ -388,7 +381,7 @@ public class PgpKeys {
             	 String newpass = EncrLib.getSHA(guardPassword, keys.getSalt());
                  Security.addProvider(new BouncyCastleProvider());
                  PGPSecretKeyRing keyring = keys.getPGPSecretKeyRing();
-                 
+
                  PGPDigestCalculator sha256Calc = new BcPGPDigestCalculatorProvider().get(PgpKeys.getHashAlgorithmTags());
                  PBESecretKeyDecryptor oldencr = new JcePBESecretKeyDecryptorBuilder().setProvider("BC").build(pgpPassword.toCharArray());
                  PBESecretKeyEncryptor newencr = (new BcPBESecretKeyEncryptorBuilder(PgpKeys.getPGPEncryptedData(), sha256Calc, 0x60)).build(newpass.toCharArray());
@@ -409,12 +402,11 @@ public class PgpKeys {
                  if (newkeyring == null) {
                      logger.error("Error importing PGP Private key");
                      return (null);
-                 }
-                 else 
-                     keys.setPGPSecretKeyRing(newkeyring);
-                 
-                 
-                 
+                 } else {
+                    keys.setPGPSecretKeyRing(newkeyring);
+                }
+
+
 			} catch (PGPException e) {
 				logger.error("Problem importing PGP Private " , e);
 				return (null);
@@ -429,34 +421,12 @@ public class PgpKeys {
 			    logger.error("Problem importing PGP Private " , e);
                 return (null);
 			}
-            
-            
+
+
         }
         return (keys);
     }
-    
-    /**
-     * Create public key ring from asc armor
-     * @param asc
-     * @return
-     */
-    public static PGPPublicKeyRing createKeyRing (String asc) {
-        try {
-            PGPObjectFactory factory = new PGPObjectFactory(PGPUtil.getDecoderStream(new java.io.ByteArrayInputStream(asc.getBytes("UTF-8"))), new BcKeyFingerprintCalculator());
-            // these files should really just have one object in them,
-            // and that object should be a PGPPublicKeyRing.
-            Object o = factory.nextObject();
-            if (o instanceof PGPPublicKeyRing) {
-                return (PGPPublicKeyRing) o;
-            }
-        } catch (Exception ex) {
-            logger.error("Error creating keyring " ,ex);
-             
-        }
 
-        throw new IllegalArgumentException("Input text does not contain a PGP Public Key");
-    }
-    
     /**
      * Create public keyring from private key
      * @param keys
@@ -465,13 +435,13 @@ public class PgpKeys {
     public static GuardKeys createPubFromPrivate (GuardKeys keys) throws IOException {
         PGPPublicKeyRing npr = null;
         PGPSecretKeyRing newkeyring = keys.getPGPSecretKeyRing();
-        
+
         for (Iterator it = newkeyring.getPublicKeys(); it.hasNext();)
         {
             PGPPublicKey key = (PGPPublicKey) it.next();
             npr = addToKeyRing (npr, key);
         }
-        for (Iterator it2 = newkeyring.getExtraPublicKeys(); it2.hasNext();) 
+        for (Iterator it2 = newkeyring.getExtraPublicKeys(); it2.hasNext();)
         {
             PGPPublicKey key = (PGPPublicKey) it2.next();
             PGPPublicKeyRing.insertPublicKey(npr, key);
@@ -479,7 +449,7 @@ public class PgpKeys {
         keys.pubring = npr;
         return(keys);
     }
-    
+
     /**
      * Add to PGPPublicKey Ring.  Ring should be null if empty (start)
      * @param npr
@@ -495,7 +465,7 @@ public class PgpKeys {
         }
         return (npr);
     }
-    
+
     private static byte revokeReason (String reason) {
         switch (reason) {
             case "NO_REASON":
@@ -511,7 +481,7 @@ public class PgpKeys {
         }
         return (RevocationReasonTags.NO_REASON);
     }
-    
+
     /**
      * Revokes the PGP Public keys associated with the GuardKeys and returns with the revokation certificate
      * @param keys
@@ -550,13 +520,13 @@ public class PgpKeys {
                 PGPSignature signature = generator.generateCertification(master, pub);
                 pub = PGPPublicKey.addCertification(pub, signature);
             }
-            
+
             keys.pubring = PGPPublicKeyRing.insertPublicKey(keys.pubring, pub);
         }
-        
+
         return(keys);
     }
-    
+
 ////////SIGN PUBLIC KEY.  NOT YET WORKING
     public static PGPPublicKey signKey (PGPPublicKey tosign, GuardKeys key, String password) throws PGPException, SignatureException {
         Iterator<String> ids = tosign.getUserIDs();
@@ -569,11 +539,11 @@ public class PgpKeys {
         signatureGenerator.init(PGPSignature.DIRECT_KEY, priv);
         PGPSignature sig = signatureGenerator.generateCertification(tosign);
         return (PGPPublicKey.addCertification(tosign, sig));
-        
-        
+
+
     }
-    
-    
+
+
     public static PGPPublicKeyRing sign (PGPSecretKey secretKey, PGPPrivateKey priv, PGPPublicKeyRing keyring) throws IOException, Exception {
         PGPPublicKeyRing sRing = new PGPPublicKeyRing(new ByteArrayInputStream(signPublicKey(secretKey, priv, keyring.getPublicKey())), new JcaKeyFingerprintCalculator());
 		return (sRing);
@@ -597,7 +567,7 @@ public class PgpKeys {
 
         return PGPPublicKey.addCertification(keyToBeSigned, sGen.generate()).getEncoded();
     }
-    
+
     /**
      * Add new User ID to publicKeyRing.  Must have private key for signing
      * @param priv
@@ -610,17 +580,17 @@ public class PgpKeys {
     public static PGPPublicKeyRing addUID (PGPPrivateKey priv, PGPPublicKeyRing keyring, String newid) throws SignatureException, PGPException {
     			PGPPublicKey pub = keyring.getPublicKey();
     			PGPSignatureGenerator generator = new PGPSignatureGenerator(new BcPGPContentSignerBuilder(PGPPublicKey.RSA_GENERAL,
-    			  PGPUtil.SHA1)); 
+    			  PGPUtil.SHA1));
     			generator.init(PGPSignature.POSITIVE_CERTIFICATION, priv);
-    			PGPSignatureSubpacketGenerator signhashgen = new PGPSignatureSubpacketGenerator();      
-    			generator.setHashedSubpackets(signhashgen.generate());    
+    			PGPSignatureSubpacketGenerator signhashgen = new PGPSignatureSubpacketGenerator();
+    			generator.setHashedSubpackets(signhashgen.generate());
     			PGPSignature certification = generator.generateCertification(newid, pub);
     			PGPPublicKey newPubKey = PGPPublicKey.addCertification(pub, newid, certification);
     			keyring = PGPPublicKeyRing.removePublicKey(keyring, pub);
     			keyring = PGPPublicKeyRing.insertPublicKey(keyring, newPubKey);
     			return (keyring);
     }
-    
+
     public static void checkBadUID (GuardKeys key, String password) throws Exception {
     	PGPPublicKey pubkey = key.pubring.getPublicKey();
     	Iterator it = pubkey.getUserIDs();
@@ -634,7 +604,7 @@ public class PgpKeys {
     			if (i < 0) {
     				// no email address found
     				return;
-    			} 
+    			}
     			String newuid = uid.substring(0, i) + " " + uid.substring(i);
     			PGPPrivateKey priv = PgpKeys.decodePrivate(key.getMasterPGPSecretKey(), password, key.getSalt());
     			if (priv != null) {
@@ -647,7 +617,7 @@ public class PgpKeys {
     		}
     	}
     }
-    
+
     public static String getSignatureType (int signature) {
     	switch (signature) {
     	case -1:
@@ -671,7 +641,7 @@ public class PgpKeys {
     	}
     	return ("");
     }
-    
+
     /**
      * Verify a specific signature in key
      * @param sig
@@ -682,10 +652,14 @@ public class PgpKeys {
      * @throws Exception
      */
     public static ArrayList<SignatureResults> verifySignature (PGPSignature sig, PGPPublicKey key, int userid, int cid) throws Exception {
+
+        ArrayList<SignatureResults> results = new ArrayList<SignatureResults> ();
+        if (sig.getKeyID() == 0) {
+            results.add(new SignatureResults("", 0, sig.getKeyID()));
+            return(results);
+        }
         Access acc = new Access();
         PGPPublicKeyRing ring = acc.getPublicKeyRingById(sig.getKeyID(), userid, cid);
-            
-        ArrayList<SignatureResults> results = new ArrayList<SignatureResults> ();
         if (ring == null) {
             results.add(new SignatureResults("", 0, sig.getKeyID()));
             return(results);
@@ -719,7 +693,7 @@ public class PgpKeys {
                     logger.debug("Bad signature length");
                     SignatureResults result = new SignatureResults("", -1, uid);
                     results.add(result);
-                    return(results);      
+                    return(results);
                 }
                 if (sig.verifyCertification(id, key)) {
                     SignatureResults result = new SignatureResults(id, sig.getSignatureType(), uid);
@@ -752,7 +726,7 @@ public class PgpKeys {
                     verified = true;
                 }
             } catch (Exception e) {
-                
+
             }
         }
         if (!verified) {
@@ -761,7 +735,7 @@ public class PgpKeys {
         }
         return (results);
     }
-    
+
     /**
      * Check all of the signatures in a key
      * @param key
@@ -779,7 +753,7 @@ public class PgpKeys {
         }
         return (results);
     }
-    
+
     /**
      * Check all the signatures of the keys in a pubring
      * @param ring
@@ -797,37 +771,37 @@ public class PgpKeys {
         }
         return (results);
     }
-    
+
     /**
      * Check if a public key is expired
      * @param key
      * @return
      */
     public static boolean isExpired (PGPPublicKey key) {
-        if (key == null) return (false);
-        if (key.getValidSeconds() == 0) return (false);
+        if (key == null) {
+            return (false);
+        }
+        if (key.getValidSeconds() == 0) {
+            return (false);
+        }
         Date now = new Date();
         return (key.getCreationTime().getTime() + (key.getValidSeconds() * 1000) - now.getTime() < 0);
     }
-    
+
     private static void markUIDUpdated (GuardKeys key) {
-    	String command = "UPDATE og_KeyTable SET version = 0 WHERE cid = ? AND id = ? AND version = -1";
-    	DbCommand com = new DbCommand(command);
-    	com.addVariables(key.contextid);
-    	com.addVariables(key.userid);
-    	DbQuery db = new DbQuery();
     	try {
-			db.write(com, key.userid, key.contextid);
+            //TODO: Use a Service Locator for resolvin this
+            OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+            ogKeyTableStorage.updateKeyVersion(key, 0);
 		} catch (Exception e) {
 			logger.error("Problem updating version from -1 for PGP correction, User " + key.userid + " CID: " + key.contextid);
 		}
-    	db.close();
     }
-    
-    
+
+
     public static JsonArray getAllKeys (int id, int cid) throws Exception {
-        Access acc = new Access();
-        ArrayList <GuardKeys> keys = acc.getAllKeys(id, cid);
+        OGKeyTableStorage keyTableStorage = new RestDbOGKeyTableStorage();
+        List <GuardKeys> keys = keyTableStorage.getKeysForUser(id, cid);
         JsonArray keysData = new JsonArray();
         for (int i = 0; i < keys.size(); i++) {
             GuardKeys key = keys.get(i);
@@ -849,23 +823,27 @@ public class PgpKeys {
                     } else {
                         pubkey.addProperty("hasPrivate", false);
                     }
-                    if (key.current) pubkey.addProperty("current", true);
+                    if (key.current) {
+                        pubkey.addProperty("current", true);
+                    }
                     pubkey.addProperty("GuardKey", Long.toString(key.keyid));
-                }  
+                }
             } else {  // No private key, just public
                 for (int j = 0; j < pubKeyData.size(); j++) {
                     JsonObject pubkey = pubKeyData.get(j).getAsJsonObject();
-                    if (key.current) pubkey.addProperty("current", true);
+                    if (key.current) {
+                        pubkey.addProperty("current", true);
+                    }
                     pubkey.addProperty("GuardKey", Long.toString(key.keyid));
                     pubkey.addProperty("hasPrivate", false);
-                    
+
                 }
             }
-            
+
             keysData.add(pubKeyData);
         }
         return (keysData);
-        
+
     }
 
 }
