@@ -50,6 +50,7 @@
 package com.openexchange.guard.server;
 
 import java.io.IOException;
+import java.util.List;
 import javax.servlet.http.Cookie;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
@@ -69,10 +70,13 @@ import com.openexchange.guard.database.GetPubKey;
 import com.openexchange.guard.database.Keymanager;
 import com.openexchange.guard.database.RecipKey;
 import com.openexchange.guard.database.Sharding;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.encr.RSAGen;
 import com.openexchange.guard.exceptions.BadPasswordException;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.logging.LogAction;
 import com.openexchange.guard.logging.LogMarkers;
@@ -119,7 +123,7 @@ public class Auth {
                 name = json.get("name").getAsString();
             }
             Keymanager keyman = new Keymanager();
-            keys = keyman.createMasterKeys(userid, cid, email, name, password, language, true, Keymanager.checkCreateRecovery(cookie, request)); // Flag true for user created
+            keys = keyman.createMasterKeys(userid, cid, email, name, password, language, true, Keymanager.checkCreateRecovery(cookie, request),true); // Flag true for user created
             if (keys == null) {
                 logger.debug("Failed to create master key");
                 response.getWriter().write(" {\"result\":\"fail \"} ");
@@ -443,10 +447,9 @@ public class Auth {
             return;
         }
 
-        acc.deleteRecovery(userid, cid);
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        ogKeyTableStorage.deleteRecovery(key);
         Core.sendOK(response, "OK");
-
-
     }
 
     public void resetPass(int userid, int cid, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, boolean web) throws Exception {
@@ -599,8 +602,9 @@ public class Auth {
      * @param request
      * @param response
      * @throws GuardMissingParameter
+     * @throws BadRSAException
      */
-    public void guestlogin(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter {
+    public void guestlogin(HttpServletRequest request, HttpServletResponse response) throws GuardMissingParameter, BadRSAException {
     	JsonObject login_json = Core.getJSON(request);
         com.openexchange.guard.database.Access acc = new com.openexchange.guard.database.Access();
         response.setContentType("application/json");
@@ -684,7 +688,7 @@ public class Auth {
             sendjson.addProperty("lastMod", keys.lastup == null ? "null" : keys.lastup.toString());
             try {
             	Settings settings = new Settings();
-            	sendjson.add("settings", getSettings(keys.settings.toString(), settings));
+            	sendjson.add("settings", getSettings(keys.settings.toString(), settings, keys.userid, keys.contextid));
             } catch (Exception ex) {
             	logger.error("problem getting guest settings " , ex);
             }
@@ -712,10 +716,19 @@ public class Auth {
             JsonObject json = Core.getJSON(request);
             Settings settings = getPermissions(cookie, request);
             login(json, request, response, cookie, settings);
+        } catch (BadRSAException e) {
+            LogAction.setFail();
+            logger.error("Unable to decode RSA password");
+            JsonObject ret = new JsonObject();
+            ret.addProperty("auth", "Bad RSA");
+            Core.sendJson(response, ret);
+            return;
         } catch (Exception ex) {
             LogAction.setFail();
             logger.error("Error reading login JSON information", ex);
-            response.getWriter().write("{\"auth\":\"BAD FORMAT\"} ");
+            JsonObject ret = new JsonObject();
+            ret.addProperty("auth", "BAD FORMAT");
+            Core.sendJson(response, ret);
             return;
         }
     }
@@ -748,7 +761,7 @@ public class Auth {
             // Check lockout of bad attempts.  If excessive, don't give encrypted auth key
             if (CheckBad.isBad(Integer.toString(userid) + "-" + Integer.toString(cid), Config.bad_password_count)) {
             	returnJson.addProperty ("auth", "Lockout");
-            	returnJson.add ("settings", getSettings(null, settings));
+            	returnJson.add ("settings", getSettings(null, settings, userid, cid));
                 response.getWriter().write(returnJson.toString());
                 LogAction.setFail();
                 logger.info("Lockout auth due to bad attempts");
@@ -796,12 +809,12 @@ public class Auth {
             if (keys == null) {
                 LogAction.setSuccess();
             	returnJson.addProperty ("auth", "No Key");
-            	returnJson.add ("settings", getSettings(null, settings));
+            	returnJson.add ("settings", getSettings(null, settings, userid, cid));
                 response.getWriter().write(returnJson.toString());
                 logger.debug("Auth sent with No Key for cid = " + cid + " userid = " + userid);
                 return;
             }
-        	returnJson.add ("settings", getSettings(keys.settings.toString(), settings));
+        	returnJson.add ("settings", getSettings(keys.settings.toString(), settings, userid, cid));
         	returnJson.add("recoveryAvail", new JsonPrimitive(keys.recoveryAvail));
             if (keys.passwordNeeded == true) {
             	returnJson.addProperty ("auth", "Password Needed");
@@ -835,7 +848,7 @@ public class Auth {
                 String returndata = "Bad Password";
 
                     if (encrpass.length() > 1) { // If provided with encryption password, check
-                        if (keys.verifyPassword(encrpass)) {
+                        if (verifyPassword(keys, userid, cid, encrpass)) {
                             returndata = encr.encryptAES(json.toString(), token);
                             CheckBad.removeBad(Integer.toString(userid) + "-" + Integer.toString(cid)); // remove bad session listing count
                             logger.debug("Auth with proper password for cid = " + cid + " userid = " + userid);
@@ -849,7 +862,7 @@ public class Auth {
                         if (oxpassword.length() > 1) {
                         	try {
                                 LogAction.setSuccess();
-	                            if (keys.verifyPassword(oxpassword)) {
+	                            if (verifyPassword(keys, userid, cid, oxpassword)) {
 	                                json.remove("encr_password");
 	                                json.addProperty("encr_password", oxpassword); // The encryption password is our ox password
 	                                returndata = encr.encryptAES(json.toString(), token);
@@ -892,7 +905,7 @@ public class Auth {
                     logger.info("Created OG tables for cid " + cid);
                 }
                 returnJson.addProperty("auth", "No Key");
-                returnJson.add ("settings", getSettings(null, settings));
+                returnJson.add ("settings", getSettings(null, settings, 0, 0));
                 LogAction.setSuccess();
                 response.getWriter().write(returnJson.toString());
             } else {
@@ -904,13 +917,36 @@ public class Auth {
 
     }
 
+    /**
+     * First, verify the current password.  If fails, then check if one of the other keys
+     * @param key
+     * @param userid
+     * @param cid
+     * @param password
+     * @return
+     * @throws Exception
+     */
+    private boolean verifyPassword (GuardKeys key, int userid, int cid, String password) throws Exception {
+        if (key.verifyPassword(password)) return (true);
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+        List<GuardKeys> userKeys = ogKeyTableStorage.getKeysForUser(userid, cid);
+        if (userKeys.size() > 1) {
+            for (GuardKeys userKey : userKeys) {
+                if (!userKey.current) {  // Don't recheck the current key
+                    if (userKey.verifyPassword(password)) return (true);
+                }
+            }
+        }
+        return (false);
+    }
+
     /**
      * Create a json of settings, include the individuals settings, plus global
      *
      * @param settings
      * @return
      */
-    private JsonObject getSettings(String settings, Settings setting) {
+    private JsonObject getSettings(String settings, Settings setting, int id, int cid) {
         StringBuilder allSettings = new StringBuilder();
         if (settings == null) {
             settings = "";
@@ -963,6 +999,8 @@ public class Auth {
         json.addProperty("min_password_length", Config.min_password_length);
         json.addProperty("password_length", Config.newPassLength);
 
+        json.addProperty("secureReply", OxConfiguration.secureReply(id, cid));
+
         return (json);
     }
 
