@@ -48,93 +48,58 @@
  */
 /**
  * For lockout of bad IP / itemID / Sessions
- * 
+ *
  */
 
 package com.openexchange.guard.server;
 
-import java.sql.Timestamp;
-
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
-
 import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.DbCommand;
 import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.bad.Bad;
+import com.openexchange.guard.database.bad.BadStorage;
+import com.openexchange.guard.database.bad.RestDbBadStorage;
 
 
 public class CheckBad {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(CheckBad.class);
+	private static BadStorage badStorage = new RestDbBadStorage();
 
     /**
      * Check if id is listed as bad.
-     * 
+     *
      * @param id Could be IP, itemID, or session
      * @param threshold Number of bad tries within time period acceptable
      * @return
+     * @throws Exception
      */
-    public static boolean isBad(String id, int threshold) {
-        boolean bad = false;
-        try {
-            String command = "SELECT last, count, NOW() as cur FROM Bad WHERE Bad.id = ?";
-            DbCommand com = new DbCommand(command);
-            com.addVariables(id);
-            DbQuery db = new DbQuery();
-            db.readOG(com);
+    public static boolean isBad(String id, int threshold) throws Exception {
 
-            if (db.next()) {
-                Timestamp last = db.rs.getTimestamp("last");
-                Timestamp cur = db.rs.getTimestamp("cur");
-                long differential = Config.bad_minute_lock * 60 * 1000;
-                if (last.getTime() > (cur.getTime() - differential)) {
-                    if (db.rs.getInt("count") > threshold) {
-                        bad = true;
-                    }
+        Bad badEntry = badStorage.getBad(id);
+        if(badEntry != null) {
+            long differential = Config.bad_minute_lock * 60 * 1000;
+            if (badEntry.getLast().getTime() > (badEntry.getCurrentTime().getTime() - differential)) {
+                if(badEntry.getCount() > threshold) {
+                    return true;
                 }
             }
-            db.close();
-        } catch (Exception ex) {
-            logger.error("Error while checking whether ID " + id + " was bad", ex);
         }
-        return (bad);
+        return false;
     }
 
     /**
      * Add the id to bad count
-     * 
+     *
      * @param id
      */
-    public static void addBad(String id) {
-        try {
-            if (id == null)
-                return;
-            String command = "DELETE FROM Bad WHERE last < DATE_SUB(NOW(), INTERVAL " + Config.bad_minute_lock + " MINUTE);";
-            DbCommand com = new DbCommand(command);
-            DbQuery db = new DbQuery();
-            db.writeOxGuard(com);
-            command = "INSERT INTO Bad VALUES (?, 1, NOW()) ON DUPLICATE KEY UPDATE count = count + 1, last = NOW()";
-            com = new DbCommand(command);
-            com.addVariables(id);
-            db.writeOxGuard(com);
-            db.close();
-        } catch (Exception ex) {
-            logger.error("Error while adding ID " + id + " to the bad count", ex);
-        }
+    public static void addBad(String id) throws Exception {
+        badStorage.addBad(id);
     }
-    
-    public static void removeBad(String id) {
-        try {
-            if (id == null)
-                return;
-            String command = "DELETE FROM Bad WHERE id = ?";
-            DbCommand com = new DbCommand(command);
-            com.addVariables(id);
-            DbQuery db = new DbQuery();
-            db.writeOxGuard(com);
-            db.close();
-        } catch (Exception ex) {
-            logger.error("Error while removing ID " + id + " from the bad count", ex);
-        }
+
+    public static void removeBad(String id) throws Exception {
+        badStorage.removeBad(id);
     }
 }
