@@ -52,56 +52,45 @@ package com.openexchange.guard.server;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.database.DbCommand;
-import com.openexchange.guard.database.DbQuery;
+import com.openexchange.guard.database.ogEmail.OGEmail;
+import com.openexchange.guard.database.ogEmail.OGEmailStorage;
+import com.openexchange.guard.database.ogEmail.RestDbOGEmailStorage;
+import com.openexchange.guard.support.UserDeleter;
 import com.openexchange.guard.util.Core;
 
 public class DemoClass {
 
     /**
      * Resets/Deletes master keys for email address ONLY FOR USE ON DEMO SERVER
-     * 
+     *
      * @param request
      * @param response
      * @throws Exception
      */
     public void Reset(HttpServletRequest request, HttpServletResponse response) throws Exception {
-        if (!Config.demo)
-            return;
-        String email = request.getParameter("email");
-        String command = "SELECT id, cid FROM og_email WHERE email = ? LIMIT 1";
-        DbCommand com = new DbCommand(command);
-        com.addVariables(email);
-        DbQuery dq = new DbQuery();
-        dq.readOG(com);
-        if (dq.next()) {
-            int id = dq.rs.getInt("id");
-            int cid = dq.rs.getInt("cid");
-            dq.close();
-            command = "DELETE FROM og_KeyTable WHERE id = ? AND cid = ?";
-            com = new DbCommand(command);
-            com.addVariables(id);
-            com.addVariables(cid);
-            dq = new DbQuery();
-            dq.write(com, id, cid);
-            dq.close();
-            command = "DELETE FROM og_email WHERE id = ? AND cid = ?";
-            com = new DbCommand(command);
-            com.addVariables(id);
-            com.addVariables(cid);
-            dq = new DbQuery();
-            dq.writeOxGuard(com);
-            dq.close();
-            command = "DELETE FROM PGPKeys WHERE email = ?";
-            com = new DbCommand(command);
-            com.addVariables(email);
-            dq = new DbQuery();
-            dq.writeOxGuard(com);
-            dq.close();
-            Core.sendOK(response, "OK");
+        if (!Config.demo) {
             return;
         }
-        dq.close();
-        Core.sendOK(response, "No");
+        String email = request.getParameter("email");
+
+        if(email != null && !email.isEmpty()) {
+
+
+            OGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+            OGEmail ogEmail = ogEmailStorage.getByEmail(email);
+            if(ogEmail != null) {
+
+                int userId = ogEmail.getUserId();
+                int contextId = ogEmail.getContextId();
+
+                new UserDeleter().delete(userId, contextId);
+                Core.sendOK(response, "OK");
+                return;
+            }
+            Core.sendOK(response, "No");
+        }
+        else {
+            Core.sendOK(response, "Missing parameter email");
+        }
     }
 }
