@@ -78,6 +78,7 @@ import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.file.Fileshare;
 import com.openexchange.guard.logging.LogAction;
@@ -167,7 +168,7 @@ public class FileHandler {
             JsonStreamHandler jsonHandler = new JsonStreamHandler("meta",meta);
 
             //encrypting the input stream of the upload item
-            logger.debug("Decrypting new upload ...");
+            logger.debug("Encrypting new upload ...");
             StopWatch sw = new StopWatch();
             sw.start();
             FileEncrypter encryptor = new FileEncrypter(api);
@@ -176,9 +177,10 @@ public class FileHandler {
                                                contentType,
                                                jsonStream,
                                                jsonHandler,
-                                               recipients);
+                                               recipients,
+                                               update);
             sw.stop();
-            logger.debug(String.format("Decryption done for new upload [Time: %d ms]",
+            logger.debug(String.format("Encrypting done for new upload [Time: %d ms]",
                         sw.getTime()));
 
             //Sending response
@@ -186,7 +188,7 @@ public class FileHandler {
         }
     }
 
-    private String getFileJson(String itemId, String mime, int folder, String description, int id, int cid) {
+    private String getFileJson(String itemId, String mime, String folder, String description, int id, int cid) {
         Gson gson = new GsonBuilder().create();
         JsonObject json = new JsonObject();
         JsonObject meta = new JsonObject();
@@ -218,8 +220,8 @@ public class FileHandler {
                         OxCookie cookie) throws Exception {
 
         String fileId = Core.getStringParameter(request, "id", true);
-        int folderId = Core.getIntParameter(request, "folder", true);
-        int version = Core.getIntParameter(request, "version", true);
+        String folderId = Core.getStringParameter(request, "folder", true);
+        String version = Core.getStringParameter(request, "version");
         String auth = Core.getStringParameter(request, "auth");
         String lang = Core.getStringParameter(request, "lang");
         int userId = Core.getIntParameter(request, "userid");
@@ -232,6 +234,9 @@ public class FileHandler {
         //Get further meta data from the object to decrypt
         Api ap = new Api(cookie,request);
         String data = ap.getInfoItemJson(fileId, folderId, version);
+        if (!Core.getError(data).equals("")) {
+            throw new Exception(Core.getError(data));
+        }
         Gson gson = new Gson();
         JsonObject json = gson.fromJson(data, JsonObject.class);
         JsonObject jdata = json.get("data").getAsJsonObject();
@@ -265,7 +270,7 @@ public class FileHandler {
                                                      encrExtra,
                                                      cookie.JSESSIONID);
         //Preparing response
-        fileName = fileName.replace(".pgp", "").replace(".grd", "");
+        fileName = fileName.replace(".pgp", "").replace(".grd2", "").replace(".grd", "");
         String newContentType = streamHandler.getNewContentType(fileType, fileName);
         response.setContentType(newContentType);
         response.addHeader("Access-Control-Allow-Origin", "*");
@@ -281,6 +286,16 @@ public class FileHandler {
             GuardStreamUtils.Copy(inputStream,
                                   outputStream,
                                   streamHandler);
+        } catch (IOException ex) {
+            if (ex.getMessage() != null) {
+                // TODO will likely need translation
+                if (ex.getMessage().startsWith("No secret key")) {
+                    outputStream.write(ex.getMessage().getBytes());
+                    response.setStatus(HttpServletResponse.SC_OK);
+                    return;
+                }
+            }
+            throw ex;
         }
 
         response.setStatus(HttpServletResponse.SC_OK);
@@ -354,8 +369,8 @@ public class FileHandler {
         com.openexchange.guard.server.OxCookie cookie) throws Exception {
 
         String id = Core.getStringParameter(request, "id", true);
-        int folder = Core.getIntParameter(request, "folder", true);
-        int version = Core.getIntParameter(request, "version", true);
+        String folder = Core.getStringParameter(request, "folder", true);
+        String version = Core.getStringParameter(request, "version");
         int userid = Core.getIntParameter(request, "userid", true);
         int cid = Core.getIntParameter(request, "cid", true);
         LogAction.setUser(userid, cid);
@@ -397,7 +412,7 @@ public class FileHandler {
                 jdata.get("description").getAsString();
         String newjson = getFileJson(itemID, fileType, folder, description, userid, cid);
         JsonStreamHandler jsonHandler = new JsonStreamHandler();
-        logger.debug(String.format("Starting encryption for file %s in folder %d ...",
+        logger.debug(String.format("Starting encryption for file %s in folder %s ...",
                                     id,
                                     folder));
         StopWatch sw = new StopWatch();
@@ -413,10 +428,11 @@ public class FileHandler {
                                                        fileType,
                                                        jsonStream,
                                                        jsonHandler,
-                                                       recipients);
+                                                       recipients,
+                                                       false);
             sw.stop();
             long ms = sw.getTime();
-            logger.debug(String.format("Encryption done for file %s in folder %d [Time: %d ms]",
+            logger.debug(String.format("Encryption done for file %s in folder %s [Time: %d ms]",
                          id,
                          folder,
                          ms));
@@ -443,8 +459,8 @@ public class FileHandler {
 
         //parse parameters
         String fileId = Core.getStringParameter(request, "id", true);
-        int folderId = Core.getIntParameter(request, "folder", true);
-        int version = Core.getIntParameter(request, "version", true);
+        String folderId = Core.getStringParameter(request, "folder", true);
+        String version = Core.getStringParameter(request, "version");
         int userId = Core.getIntParameter(request, "userid", true);
         int cid = Core.getIntParameter(request, "cid", true);
         String auth = Core.getStringParameter(request, "auth");
@@ -489,7 +505,7 @@ public class FileHandler {
         try(InputStream jsonStream = new ByteArrayInputStream(newjson.getBytes()))
         {
             //De-Crypting the item
-            logger.debug(String.format("Decrypting file %s in folder %d ...",
+            logger.debug(String.format("Decrypting file %s in folder %s ...",
                          fileId,
                          folderId));
             StopWatch sw = new StopWatch();
@@ -510,7 +526,7 @@ public class FileHandler {
                                                      jsonHandler);
             sw.stop();
             long ms = sw.getTime();
-            logger.debug(String.format("Decryption done for file %s in folder %d [Time: %d ms]",
+            logger.debug(String.format("Decryption done for file %s in folder %s [Time: %d ms]",
                          fileId,
                          folderId,
                          ms));
@@ -690,7 +706,7 @@ public class FileHandler {
         }
     }
 
-    public void addExtraPassword(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) throws GuardMissingParameter {
+    public void addExtraPassword(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie oxCookie) throws GuardMissingParameter, BadRSAException {
 
         JsonObject rdata = Core.getJSON(request);
         String auth = Core.getStringParameter(request, "auth");
@@ -714,8 +730,8 @@ public class FileHandler {
         int userid = Core.getIntFromJson(rdata, "userid", true);
         int cid = Core.getIntFromJson(rdata, "cid", true);
         String id = Core.getStringFromJson(rdata, "id", true);
-        int folder = Core.getIntFromJson(rdata, "folder", true);
-        int version = Core.getIntFromJson(rdata, "version", true);
+        String folder = Core.getStringFromJson(rdata, "folder", true);
+        String version = Core.getStringFromJson(rdata, "version", true);
         String sessionID = Core.getStringParameter(request, "session", true);
         String extraPass = Core.getStringFromJson(rdata, "extrapass", true);
         if ((extraPass == null) || (extraPass.length() < 3)) {
