@@ -55,7 +55,6 @@ import java.io.InputStream;
 import java.io.OutputStream;
 import java.security.NoSuchAlgorithmException;
 import java.security.spec.InvalidKeySpecException;
-import java.sql.Date;
 import java.sql.SQLException;
 import java.text.SimpleDateFormat;
 import java.util.Enumeration;
@@ -66,7 +65,6 @@ import javax.mail.Header;
 import javax.mail.Message.RecipientType;
 import javax.mail.MessagingException;
 import javax.mail.Multipart;
-import javax.mail.Part;
 import javax.mail.Session;
 import javax.mail.internet.InternetAddress;
 import javax.mail.internet.MimeMessage;
@@ -80,15 +78,21 @@ import com.google.gson.JsonArray;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonPrimitive;
 import com.openexchange.guard.config.Config;
+import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.DbCommand;
 import com.openexchange.guard.database.DbQuery;
-import com.openexchange.guard.database.Sharding;
-import com.openexchange.guard.database.Access;
 import com.openexchange.guard.database.Keymanager;
+import com.openexchange.guard.database.Sharding;
+import com.openexchange.guard.database.ogEmail.OGEmail;
+import com.openexchange.guard.database.ogEmail.RestDbOGEmailStorage;
+import com.openexchange.guard.database.ogKeyTable.OGKeyTableStorage;
+import com.openexchange.guard.database.ogKeyTable.RestDbOGKeyTableStorage;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
+import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.exceptions.BadPasswordException;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.mailcreator.Mail;
 import com.openexchange.guard.pgp.PGPResult;
@@ -100,7 +104,7 @@ import com.openexchange.guard.validator.Normalizer;
 
 
 public class Guest {
-	
+
 	private static Logger logger = LoggerFactory.getLogger(Guest.class);
 
 
@@ -130,11 +134,12 @@ public class Guest {
      * @param response
      * @param cookie
      * @throws GuardMissingParameter
+     * @throws BadRSAException
      */
-    public void GetEmail(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
+    public void GetEmail(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter, BadRSAException {
     	decodeGuestPgp (request, response, cookie, "mail");
     }
-    
+
     /**
      * Decodes Guest PGP email.  Opens attachment if action open, returns email json if action mail
      * @param request
@@ -142,8 +147,9 @@ public class Guest {
      * @param cookie
      * @param action
      * @throws GuardMissingParameter
+     * @throws BadRSAException
      */
-    private void decodeGuestPgp(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, String action) throws GuardMissingParameter {
+    private void decodeGuestPgp(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie, String action) throws GuardMissingParameter, BadRSAException {
     	JsonObject jdata = Core.getJSON(request);
     	String itemid = "";
     	if (jdata != null) {
@@ -165,7 +171,7 @@ public class Guest {
                 	 PGPResult result = null;
                      try {
                     	 result = PGPUtils.decryptFile(in, data.userid, data.cid, data.encr_password);
-                    	 
+
                      } catch (BadPasswordException ex) {
                     	 in.close();
                      	Core.sendFail(response, "Bad password");
@@ -176,12 +182,12 @@ public class Guest {
                      	Core.sendFail(response, result.error);
                      } else {
                     	 switch (action) {
-                     		case "mail": 
+                     		case "mail":
                      			Core.sendOK(response, createReturnJson(result, data.encr_password, cookie));
                      		break;
                      		case "open":
                      			String filename = Core.getStringParameter(request, "attname", true);
-                     			boolean download = (Core.getStringParameter(request, "download") == null ? false : (Core.getStringParameter(request, "download").toLowerCase().equals("true"))); 
+                     			boolean download = (Core.getStringParameter(request, "download") == null ? false : (Core.getStringParameter(request, "download").toLowerCase().equals("true")));
                      			PgpHandler.getAttach (response, result, filename, download);
                      		break;
                      	}
@@ -192,13 +198,13 @@ public class Guest {
                 }
             } catch (Exception ex) {
             	logger.error("Problemd decoding Guest PGP ", ex);
-            	
+
             }
     	} else {  // Proprietary Guard
     		GetOldEmail (jdata, request, response, cookie);
     	}
     }
-    
+
     /**
      * Create resturn Json for display in Guest browser
      * @param result
@@ -226,8 +232,9 @@ public class Guest {
         	EncrLib encr = new EncrLib();
         	Results.addProperty("epassword", encr.encryptAES(password, token));
         }
-        if (result.error != null)
+        if (result.error != null) {
             Results.add("error", new JsonPrimitive(result.error));
+        }
         data.add("attachments", attachments);
         data.add("results", Results);
         Address[] fromarray = message.getFrom();
@@ -282,12 +289,12 @@ public class Guest {
         in.close();
         return (data.toString());
     }
-    
+
 
 
     /**
      * Function to return encrypted attachment (non email)
-     * 
+     *
      * @param request
      * @param response
      * @param cookie
@@ -303,7 +310,7 @@ public class Guest {
         	getEncrAttachOld(request, response, cookie);
         }
     }
-    
+
 
 
     /**
@@ -314,23 +321,25 @@ public class Guest {
      * @throws IOException
      * @throws GuardMissingParameter
      */
-    public void getQuestion(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
+    public void getQuestion(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception, GuardMissingParameter {
         String email = Core.getStringParameter(request, "email", true);
+        RestDbOGEmailStorage ogEmailStorage = new RestDbOGEmailStorage();
+        OGKeyTableStorage ogKeyTableStorage = new RestDbOGKeyTableStorage();
+
+        //Getting the mail mapping
         String question = "nf";
-        try {
-            String command = "SELECT question FROM og_KeyTable WHERE email = ?;";
-            DbCommand com = new DbCommand(command);
-            com.addVariables(email);
-            DbQuery db = new DbQuery();
-            db.readFromShard(com, Sharding.getShard(email));
-            if (db.next()) {
-                question = db.rs.getString("question");
+        OGEmail emailKeyMapping = ogEmailStorage.getByEmail(email);
+        if(emailKeyMapping != null) {
+            //Getting the guest's key
+            GuardKeys key = ogKeyTableStorage.getKeyForEmailAndContext(email, emailKeyMapping.getContextId());
+            if(key != null) {
+                question = key.question;
             }
-            db.close();
-        } catch (Exception ex) {
-            logger.error("Error getting question for email account " + email, ex);
         }
-        com.openexchange.guard.util.Core.sendOK(response, question);
+        else {
+            logger.error(String.format("No email mapping found for email %s",email));
+        }
+        Core.sendOK(response, question);
     }
 
     /**
@@ -347,13 +356,15 @@ public class Guest {
         String lang = Core.getStringParameter(request, "language");
         int templId = Core.getIntParameter(request, "templid");
         String question = "";
-        if (CheckBad.isBad("reset-" + email, Config.bad_password_count)) {
-        	Core.sendOK(response, "lockout");
-        	return;
-        }
+
         int cid = 0;
         int userid = 0;
         try {
+            if (CheckBad.isBad("reset-" + email, Config.bad_password_count)) {
+                Core.sendOK(response, "lockout");
+                return;
+            }
+
             String command = "SELECT id, cid, answer, question FROM og_KeyTable WHERE email = ?;";
             DbCommand com = new DbCommand(command);
             com.addVariables(email);
@@ -367,13 +378,16 @@ public class Guest {
                 question = db.rs.getString("question");
             }
             if (question != null) {  // PIN is not used for answer
-            	if (question.equals("PIN")) prevanswer = null;
+            	if (question.equals("PIN")) {
+                    prevanswer = null;
+                }
             }
             db.close();
             if (prevanswer != null) {
 	            EncrLib encr = new EncrLib();
-	            if (question.equals(""))
-	                question = "default";
+	            if (question.equals("")) {
+                    question = "default";
+                }
 	            prevanswer = encr.decryptAES(prevanswer, Config.rpass, question, Config.AESKeyLen);
 	            if (prevanswer == null) {
 	                Core.sendFail(response, "Error decoding answer");
@@ -410,11 +424,11 @@ public class Guest {
             Core.sendFail(response, "Failed to change");
         }
     }
-    
+
     //////////////////////////////////////////////////////////////////////////////////////////////////
     ///////////////  DEPRECIATED.  FOR OLD GUARD SUPPORT
-    
-    
+
+
     /**
      * Depreciated Guard proprietary format
      * @param jdata
@@ -425,15 +439,15 @@ public class Guest {
      */
     public void GetOldEmail(JsonObject jdata, HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws GuardMissingParameter {
 
-    	
+
         String auth = Core.getStringParameter(request, "auth", true);
         String itemid = Core.getStringFromJson(jdata, "item", true);
         String extraPass = Core.getStringFromJson(jdata, "extrapass");
-        
+
         int cid = Core.getIntFromJson(jdata, "cid", true);
         cookie.JSESSIONID = cookie.OxReaderID; // In case person uses both reader and OX main, we need to clear out the previous cookie
         UserData data = new UserData(auth, cookie);
-                
+
         if (data.encr_password == null) {
             response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
             return;
@@ -468,7 +482,7 @@ public class Guest {
                                                                                                                                  // to
                                                                                                                                  // email
                                                                                                                                  // json
-                        
+
                     }
                 }
 
@@ -485,8 +499,8 @@ public class Guest {
         }
 
     }
-    
-    
+
+
     public void getEncrAttachOld(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws Exception {
     	String emailid = Core.getStringParameter(request, "email", true);
         String filename = Core.getStringParameter(request, "attname", true);
@@ -546,11 +560,11 @@ public class Guest {
         com.openexchange.guard.util.Core.sendFail(response, GetText.gettext("Unable to decode", lang));
 
     }
-    
+
 
     /**
      * Send attachment contents to browser with flags to save to disk
-     * 
+     *
      * @param response
      * @param type
      * @param answer
@@ -569,5 +583,5 @@ public class Guest {
         	logger.error("Error writing output stream for file " + filename, e);
         }
     }
-    
+
 }
