@@ -98,6 +98,7 @@ import com.openexchange.guard.database.RetrievedItem;
 import com.openexchange.guard.encr.Crypto;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.EncryptedObj;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.logging.LogAction;
 import com.openexchange.guard.mail.Attachments;
@@ -149,7 +150,7 @@ public class MailHandler {
     	attachHandle.getEncrAttach(request, response, cookie);
     }
     
-    public void saveDecodedAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter {
+    public void saveDecodedAttach(HttpServletRequest request, HttpServletResponse response, com.openexchange.guard.server.OxCookie cookie) throws IOException, GuardMissingParameter, BadRSAException {
     	Attachments attachHandle = new Attachments(cookie);
     	attachHandle.saveDecodedAttach(request, response, cookie);
     }
