@@ -322,6 +322,7 @@ public class MainServlet extends HttpServlet {
 	            try {
 	                PGPPrivateKeyHandler pkh = new PGPPrivateKeyHandler();
 	                pkh.makeCurrent(request, response, cookie);
+	                Core.sendOK(response, "OK");
 	            } catch (Exception e) {
                     logger.error("Problem making key current");
                     handleException (e, request, response);
@@ -420,7 +421,7 @@ public class MainServlet extends HttpServlet {
                 try {
                     auth.logout(request, response, cookie);
                 } catch (Exception e) {
-                	logger.error("Error logging in", e);
+                	logger.error("Error logging out", e);
                 	handleException (e, request, response);
                 }
                 break;
@@ -675,7 +676,7 @@ public class MainServlet extends HttpServlet {
         	case ("uploadkeys"):
         		try {
         			PGPPublicHandler pgppub = new PGPPublicHandler ();
-        			pgppub.incomingPGPForm(request, response);
+        			pgppub.incomingPGPForm(request, response,cookie);
 
         		} catch (Exception e) {
             		logger.error("Error uploading PGP pulbic key ", e);
