@@ -78,6 +78,7 @@ import com.openexchange.guard.config.Config;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.mailcreator.Attachment;
 import com.openexchange.guard.translate.GetText;
+import com.openexchange.guard.util.Core;
 
 public class Sender {
 	
@@ -218,8 +219,8 @@ public class Sender {
                 replyaddr = new InternetAddress(fromaddr.getAddress(), fromaddr.getPersonal(), "UTF-8");
             } else {
                 String from =  email.get("from").getAsString();
-                fromaddr = new InternetAddress(from);
-                replyaddr = new InternetAddress(from);
+                fromaddr = Core.parseEmailAdress(from); 
+                replyaddr = Core.parseEmailAdress(from);
             }
 
             if (!sender.equals("")) {
