@@ -60,15 +60,17 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import com.openexchange.guard.config.Config;
+import com.openexchange.guard.database.OGInit;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.pgp.PgpKeys;
+import com.openexchange.guard.util.CheckOx;
 
 
 public class MasterPasswordCreator {
 
 	private static Logger logger = LoggerFactory.getLogger(MasterPasswordCreator.class);
-	
+
     public boolean createPasswordFile() {
         Writer writer;
         try {
@@ -78,6 +80,15 @@ public class MasterPasswordCreator {
         		logger.info("Password file already exists.  Using existing");
         		return(true);
         	}
+        	try { // Just try to see if we can connect with Guard.  If not, then this is a clean install
+        	    OGInit.initGuardDatabase();
+            	if (CheckOx.guardInstalled()) {
+            	    System.out.println("Guard already installed.  Copy password file from first installed server");
+            	    return (false);
+            	}
+        	} catch (Exception e) {
+        	    // If error, prob just not set up yet.  Quiet
+        	}
             PgpKeys keygen = new PgpKeys();
 
             writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(Config.getRoot() + "oxguardpass"), "utf-8"));
@@ -89,7 +100,7 @@ public class MasterPasswordCreator {
             Config.rpass = rpass;
             writer.close();
             try {
-            	
+
             	Runtime.getRuntime().exec("chmod 440 " + passfile);
             	Runtime.getRuntime().exec("chown open-xchange:root " + passfile);
             } catch (Exception ex) {
@@ -130,7 +141,7 @@ public class MasterPasswordCreator {
             return(false);
         }
     }
-    
+
     public void setPermissions() {
     	try {
 			Runtime.getRuntime().exec("chown -R open-xchange:root " + Config.logDir);
