@@ -62,6 +62,8 @@ import com.openexchange.guard.database.DbQuery;
 import com.openexchange.guard.database.DbStrings;
 import com.openexchange.guard.database.Keymanager;
 import com.openexchange.guard.database.OxResultSet;
+import com.openexchange.guard.database.globalSettings.GlobalSettingsStorage;
+import com.openexchange.guard.database.globalSettings.RestDbGlobalSettingsStorage;
 import com.openexchange.guard.database.paging.ConfigDbPagedDbQuery;
 import com.openexchange.guard.encr.GuardKeys;
 import com.openexchange.guard.util.Core;
@@ -101,24 +103,12 @@ public class Upgrader {
     final String alterSharding = "ALTER TABLE `sharding` MODIFY COLUMN `current_shard`  int(11) NOT NULL FIRST , ADD PRIMARY KEY (`current_shard`);";
 
     public boolean checkUpgrade () throws Exception {
-    	String command = "SELECT value FROM global_settings WHERE global_settings.key = 'version'";
-    	DbCommand com = new DbCommand(command);
-    	DbQuery db = new DbQuery();
-    	try {
-			db.readOG(com);
-		} catch (Exception e) {
-			logger.error("Problem reading current version" , e);
-			return(false);
-		}
-    	if (db.next()) {
-    		previousVersion = db.rs.getInt("value");
-    		if (previousVersion < Config.version) {
-    			db.close();
-    			return(true);
-    		}
-    	}
-    	db.close();
-    	return(false);
+
+        //TODO: Use a Service locator / DI-Container for resolving
+        GlobalSettingsStorage globalSettingsStorage = new RestDbGlobalSettingsStorage();
+
+        int previousVersion = Integer.parseInt(globalSettingsStorage.getByKey("version"));
+        return previousVersion < Config.version;
     }
     /**
      * For future database upgrade action.
@@ -144,7 +134,7 @@ public class Upgrader {
 
         if (previousVersion < 7) {
         	Keymanager keyman = new Keymanager();
-            GuardKeys keys = keyman.createMasterKeysNC(-2, 0, "dont@remove.com", "master", Config.rpass, "", false);
+            GuardKeys keys = keyman.createMasterKeysNC(-2, 0, "dont@remove.com", "master", Config.rpass, "", false,true);
             if (keys == null) {
             	logger.error("Unable to create master keys for client encryption");
                 return (false);
@@ -163,19 +153,15 @@ public class Upgrader {
     }
 
     private boolean setCurrent(int version) {
-        String command = "UPDATE global_settings SET value = ? WHERE global_settings.key = 'version'";
         try {
-        	DbCommand com = new DbCommand(command);
-        	com.addVariables(version);
-        	DbQuery db = new DbQuery();
-        	db.writeOxGuard(com);
-
-            return (true);
+            //TODO: Use a Service locator / DI-Container for resolving
+            GlobalSettingsStorage restDbGlobalSettingsStorage = new RestDbGlobalSettingsStorage();
+            restDbGlobalSettingsStorage.Update("version", Integer.toString(version));
+            return true;
         } catch (Exception e) {
-            // TODO Auto-generated catch block
             e.printStackTrace();
         }
-        return (false);
+        return false;
     }
 
     private void addPGPSchema (Schema schema) throws Exception {
