@@ -51,17 +51,18 @@ package com.openexchange.guard.storage;
 
 import java.io.IOException;
 import java.io.InputStream;
-
 import com.openexchange.guard.config.Config;
-import com.openexchange.guard.database.CacheManager;
 import com.openexchange.guard.database.Sharding;
+import com.openexchange.guard.database.fileCache.FileCacheItem;
+import com.openexchange.guard.database.fileCache.FileCacheStorage;
+import com.openexchange.guard.database.fileCache.RestDbFileCacheStorage;
 import com.openexchange.guard.encr.EncryptedObj;
 import com.openexchange.guard.storage.file.FileStorage;
 import com.openexchange.guard.storage.s3.S3Storage;
 
 /**
  * Interface handler for the file storage.
- * 
+ *
  * @author benjamin.otterbach@open-xchange.com
  */
 public class Storage {
@@ -70,10 +71,10 @@ public class Storage {
      * The delimiter character to separate the prefix from the keys
      */
     public static final String DELIMITER = "/";
-	
+
     /**
      * Read encrypted object from file
-     * 
+     *
      * @param userid User Id
      * @param contextid Users context id
      * @param ObjId Unique object id
@@ -82,16 +83,31 @@ public class Storage {
      */
     public static InputStream readObj(int userid, int contextid, String ObjId) throws Exception {
     	filterObjId(ObjId);
-    	switch (Config.getStorageType()) {
-	    	case "file":
-	    		return (FileStorage.getObjectStream(getDirectoryPrefix(userid, contextid, Sharding.getShard(userid, contextid)), ObjId));
-	    	case "s3":
-	    		return (S3Storage.getObjectStream(getDirectoryPrefix(userid, contextid, Sharding.getShard(userid, contextid)), ObjId));
-	    	default:
-	    		throw new Exception("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type while fetching encrypted file");
+    	// First try to pull from cache
+	    FileCacheStorage storage = new RestDbFileCacheStorage();
+	    FileCacheItem item = storage.getById(ObjId);
+	    if (item != null) {
+	        switch (Config.getStorageType()) {
+	            case "file":
+                    return (FileStorage.getObjectStream(item.getLocation()));
+                case "s3":
+                    return (S3Storage.getObjectStream(item.getLocation()));
+                default:
+                    throw new Exception("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type while fetching encrypted file");
+            }
+	    }
+    	    // default search, but probably removed
+	    switch (Config.getStorageType()) {
+            case "file":
+                return (FileStorage.getObjectStream(getDirectoryPrefix(userid, contextid, Sharding.getShard(userid, contextid)), ObjId));
+            case "s3":
+                return (S3Storage.getObjectStream(getDirectoryPrefix(userid, contextid, Sharding.getShard(userid, contextid)), ObjId));
+            default:
+                throw new Exception("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type while fetching encrypted file");
     	}
+
     }
-    
+
     public static EncryptedObj readEncrObjOld(int userid, int contextid, String ObjId) throws Exception {
     	filterObjId(ObjId);
     	switch (Config.getStorageType()) {
@@ -106,7 +122,7 @@ public class Storage {
 
     /**
      * Write encrypted object to file
-     * 
+     *
      * @param userid User Id
      * @param contextid Users context id
      * @param ObjId Unique object id
@@ -118,7 +134,7 @@ public class Storage {
     	boolean saveSuccess = false;
     	filterObjId(ObjId);
     	String directoryPrefix = getDirectoryPrefix(userid, contextid, Sharding.getShard(userid, contextid));
-    	
+
     	switch (Config.getStorageType()) {
 	    	case "file":
 	    		saveSuccess = FileStorage.writeObj(directoryPrefix, ObjId, data);
@@ -129,13 +145,20 @@ public class Storage {
 	    	default:
 	    		throw new Exception("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type while storing encrypted file");
     	}
-    	
-    	if (saveSuccess)
-    		CacheManager.addToCache(ObjId, userid, directoryPrefix + DELIMITER + ObjId);
-    	
+
+    	if (saveSuccess) {
+    	    //TODO: resolve using a service locator / DI-Container    	    )
+    	    FileCacheStorage fileCacheStorage = new RestDbFileCacheStorage();
+    	    //Only inserting the file cache item if it does not exist yet
+    	    //(It could exists when storing an attachment for multiple recipients)
+    	    if(fileCacheStorage.getById(ObjId) == null) {
+    	        fileCacheStorage.Insert(ObjId, userid, directoryPrefix + DELIMITER + ObjId);
+    	    }
+    	}
+
     	return saveSuccess;
     }
-    
+
     public static void deleteEncrObj(String location) throws IOException, Exception {
     	switch (Config.getStorageType()) {
 	    	case "file":
@@ -148,10 +171,10 @@ public class Storage {
 	    		throw new Exception("Unknown value \"" + Config.getStorageType() + "\" for configuration com.openexchange.guard.storage.type while deleting encrypted file");
     	}
     }
-    
+
     /**
      * Get directory prefix for the user account
-     * 
+     *
      * @param userid
      * @param contextid
      * @param shardid
@@ -166,10 +189,10 @@ public class Storage {
         }
         return (directory);
     }
-    
+
     /**
      * This is to get directory prefix for guard proprietary, which had an error.  Remove in near future
-     * 
+     *
      * @param userid
      * @param contextid
      * @param shardid
@@ -184,12 +207,12 @@ public class Storage {
         }
         return (directory);
     }
-    
+
     // Filter for .. attack
     private static void filterObjId (String ObjId) throws Exception {
     	if (ObjId.contains("..")) {
     		throw new Exception("Back Object Id " + ObjId);
     	}
     }
-	
+
 }
