@@ -142,6 +142,16 @@ public class S3Storage {
         }
     }
     
+    public static InputStream getObjectStream(String location) {
+        try {
+            InputStream in = S3ClientFactory.s3Client.getObject(Config.getS3BucketName(), location).getObjectContent();
+            return (in);
+        } catch (Exception e) {
+            logger.error("Error getEncrObj", e);
+            return (null);
+        }
+    }
+    
     public static void deleteObj(String location) {
     	S3ClientFactory.s3Client.deleteObject(Config.getS3BucketName(), location);
     }
