@@ -69,6 +69,7 @@ import javax.xml.transform.TransformerFactory;
 import javax.xml.transform.dom.DOMSource;
 import javax.xml.transform.stream.StreamResult;
 import org.apache.http.client.methods.CloseableHttpResponse;
+import org.bouncycastle.util.encoders.Base64;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import com.google.gson.Gson;
@@ -78,6 +79,7 @@ import com.openexchange.guard.config.Config;
 import com.openexchange.guard.database.Access;
 import com.openexchange.guard.encr.EncrLib;
 import com.openexchange.guard.encr.GuardKeys;
+import com.openexchange.guard.exceptions.BadRSAException;
 import com.openexchange.guard.exceptions.GuardMissingParameter;
 import com.openexchange.guard.logging.LogAction;
 
@@ -205,11 +207,14 @@ public class Core {
      * Get the json object from post request
      * @param request
      * @return
+     * @throws BadRSAException
      */
-    public static JsonObject getJSON(HttpServletRequest request) {
+    public static JsonObject getJSON(HttpServletRequest request) throws BadRSAException {
         JsonParser jsonParser = new JsonParser();
         String postBody = getPostBody(request);
-        if (postBody == null) return(null);
+        if (postBody == null) {
+            return(null);
+        }
         JsonObject json = (JsonObject) jsonParser.parse(postBody);
         return (checkEpass(json));
     }
@@ -331,8 +336,9 @@ public class Core {
      * Parse JSOn and decode all encrypted passwords
      * @param result
      * @return
+     * @throws BadRSAException
      */
-    private static JsonObject checkEpass (JsonObject result) {
+    private static JsonObject checkEpass (JsonObject result) throws BadRSAException {
     	if (result == null) {
             return(null);
         }
@@ -343,7 +349,7 @@ public class Core {
         return (result);
     }
 
-    private static JsonObject decr(JsonObject result, String key, String newkey) {
+    private static JsonObject decr(JsonObject result, String key, String newkey) throws BadRSAException {
     	if (result.has(key)) {
     		String encr = result.get(key).getAsString();
     		if (passwordKey == null) {
@@ -354,6 +360,10 @@ public class Core {
         		String decoded = encrlib.decryptRSA(encr, passwordKey);
         		if (decoded !=null) {
         			result.addProperty(newkey, decoded);
+        		} else {
+        		    if (encr.length() > 2) {
+        		        throw new BadRSAException ("Bad RSA");
+        		    }
         		}
         	}
     	}
@@ -367,7 +377,9 @@ public class Core {
      */
     public static String decodeRSApassword (String epass) {
         EncrLib encrlib = new EncrLib();
-        if (passwordKey == null) loadPrivate();
+        if (passwordKey == null) {
+            loadPrivate();
+        }
         return(encrlib.decryptRSA(epass, passwordKey));
     }
 
@@ -385,6 +397,26 @@ public class Core {
 
     }
 
+    // Check if able to load private keys, verify password file
+    public static boolean checkPrivate () {
+        try {
+            Access acc = new Access();
+            GuardKeys passwordKeys = acc.getKeys(-1, 0);
+            if (passwordKeys != null) {
+                PrivateKey testkey = passwordKeys.getDecodedPrivate(Config.mpass);
+            }
+            loadPrivate();
+            if (passwordKey == null) {
+                return(false);
+            }
+        } catch (Exception ex) {
+            logger.error("Problem loading password RSA key", ex);
+            return(false);
+
+        }
+        return (true);
+    }
+
     public static PrivateKey getPasswordKey () {
     	if (passwordKey == null) {
             loadPrivate();
@@ -437,6 +469,14 @@ public class Core {
      */
     public static String getStringParameter (HttpServletRequest request, String name, boolean mandatory) throws GuardMissingParameter {
     	String item = request.getParameter(name);
+    	try {
+	       if (item != null) {
+	            if (item.equals("null")) item = null;
+	       }
+    	} catch (Exception e) {
+    	    logger.error("Problem getting parameter " + name, e);
+    	}
+
     	if (mandatory && (item == null)) {
     		throw new com.openexchange.guard.exceptions.GuardMissingParameter("Missing parameter " + name);
     	}
@@ -505,11 +545,11 @@ public class Core {
     public static int getIntFromJson (JsonObject json, String name) throws GuardMissingParameter {
     	return (getIntFromJson (json, name, false));
     }
-    
+
     public static boolean getBooleanFromJson (JsonObject json, String name) throws GuardMissingParameter {
         return (getBooleanFromJson (json, name, false));
     }
-    
+
     public static boolean getBooleanFromJson (JsonObject json, String name, boolean mandatory) throws GuardMissingParameter {
         if (!json.has(name)) {
             if (mandatory) {
@@ -629,14 +669,37 @@ public class Core {
     		return (null);
     	}
     }
-    
+
     public static InternetAddress parseEmailAdress (String address) throws AddressException, UnsupportedEncodingException {
+        try {
+            if (address.contains("<")) {
+                String name = address.substring(0, address.indexOf("<"));
+                if (!name.contains("\"")) {
+                    address = "\"" + name.trim() + "\" " + address.substring(address.indexOf("<")); 
+                }
+            }
+        } catch (Exception e) {
+            logger.error("Problem parsing address ", e);
+        }
         InternetAddress[] recips = InternetAddress.parse(address, false);
         if (recips.length > 0) {
             return (new InternetAddress(recips[0].getAddress(), recips[0].getPersonal(), "UTF-8"));
         }
         return (null);
-        
+
+    }
+    
+    public static String decodeBase64 (String data) {
+        if (data.contains("\r\n\r\n")) {
+            data = data.substring(data.indexOf("\r\n\r\n"));
+        }
+        String ret = data;
+        try {
+            ret = new String(Base64.decode(data.trim()));
+        } catch (Exception e) {
+            logger.error("Problem decoding Base64 attachment");
+        }
+        return (ret);
     }
 
 }
